@@ -1,4 +1,4 @@
-// $Id: CommandLine.cpp,v 1.35 2004/04/22 05:59:08 geuzaine Exp $
+// $Id: CommandLine.cpp,v 1.50 2004/10/03 23:32:02 geuzaine Exp $
 //
 // Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
 //
@@ -19,6 +19,7 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
+#include <sys/types.h>
 #include <unistd.h>
 #include "Gmsh.h"
 #include "GmshUI.h"
@@ -46,8 +47,6 @@ extern Mesh *THEM;
 char *TheFileNameTab[MAX_OPEN_FILES];
 char *TheBgmFileName = NULL, *TheOptString = NULL;
 
-// *INDENT-OFF*
-
 char gmsh_progname[]  = "Gmsh, a 3D mesh generator with pre- and post-processing facilities" ;
 char gmsh_copyright[] = "Copyright (C) 1997-2004 Christophe Geuzaine and Jean-Francois Remacle";
 char gmsh_version[]   = "Version        : " ;
@@ -73,14 +72,15 @@ void Print_Usage(char *name){
   Msg(DIRECT, "  -1, -2, -3            perform batch 1D, 2D and 3D mesh generation");
   Msg(DIRECT, "  -saveall              save all elements (discard physical group definitions)");
   Msg(DIRECT, "  -o file               specify mesh output file name");
-  Msg(DIRECT, "  -format msh|unv|gref  set output mesh format (default: msh)");
-  Msg(DIRECT, "  -algo iso|tri|aniso   select 2D mesh algorithm (default: iso)");
-  Msg(DIRECT, "  -smooth int           set mesh smoothing (default: 0)");
-  Msg(DIRECT, "  -order int            set mesh order (default: 1)");
-  Msg(DIRECT, "  -scale float          set global scaling factor (default: 1.0)");
-  Msg(DIRECT, "  -meshscale float      set mesh scaling factor (default: 1.0)");
-  Msg(DIRECT, "  -clscale float        set characteristic length scaling factor (default: 1.0)");
-  Msg(DIRECT, "  -rand float           set random perturbation factor (default: 1.e-4)");
+  Msg(DIRECT, "  -format string        set output mesh format (msh, unv, gref)");
+  Msg(DIRECT, "  -algo string          select mesh algorithm (iso, tri, aniso, netgen)");
+  Msg(DIRECT, "  -smooth int           set number of mesh smoothing steps");
+  Msg(DIRECT, "  -optimize             optimize quality of tetrahedral elements");
+  Msg(DIRECT, "  -order int            set mesh order (1, 2)");
+  Msg(DIRECT, "  -scale float          set global scaling factor");
+  Msg(DIRECT, "  -meshscale float      set mesh scaling factor");
+  Msg(DIRECT, "  -clscale float        set characteristic length scaling factor");
+  Msg(DIRECT, "  -rand float           set random perturbation factor");
   Msg(DIRECT, "  -bgm file             load backround mesh from file");
   Msg(DIRECT, "  -constrain            constrain background mesh with characteristic lengths");
   Msg(DIRECT, "  -histogram            print mesh quality histogram");
@@ -89,29 +89,24 @@ void Print_Usage(char *name){
 #if defined(HAVE_FLTK)
   Msg(DIRECT, "  -interactive          display 2D mesh construction interactively");
   Msg(DIRECT, "Post-processing options:");
-  Msg(DIRECT, "  -dl                   enable display lists");
   Msg(DIRECT, "  -noview               hide all views on startup");
-  Msg(DIRECT, "  -link int             select link mode between views (default: 0)");
-  Msg(DIRECT, "  -smoothview           smooth views");
-  Msg(DIRECT, "  -combine              combine input views into multi time step ones");
+  Msg(DIRECT, "  -link int             select link mode between views (0, 1, 2, 3, 4)");
+  Msg(DIRECT, "  -combine              combine input views into multi-time-step ones");
   Msg(DIRECT, "Display options:");    
   Msg(DIRECT, "  -nodb                 disable double buffering");
-  Msg(DIRECT, "  -fontsize int         specify the font size for the GUI (default: 12)");
+  Msg(DIRECT, "  -fontsize int         specify the font size for the GUI");
   Msg(DIRECT, "  -scheme string        specify FLTK GUI scheme");
-  Msg(DIRECT, "  -alpha                enable alpha blending");
-  Msg(DIRECT, "  -notrack              don't use trackball mode for rotations");
   Msg(DIRECT, "  -display string       specify display");
-  Msg(DIRECT, "  -perspective          set projection mode to perspective");
 #endif
   Msg(DIRECT, "Other options:");      
 #if defined(HAVE_FLTK)
-  Msg(DIRECT, "  -a, -g, -m, -s, -p    start in automatic, geometry, mesh, solver or post-processing");
-  Msg(DIRECT, "                        mode (default: automatic)");
+  Msg(DIRECT, "  -a, -g, -m, -s, -p    start in automatic, geometry, mesh, solver or post-processing mode");
 #endif
-  Msg(DIRECT, "  -v int                set verbosity level (default: 2)");
+  Msg(DIRECT, "  -pid                  print process id on stdout");
+  Msg(DIRECT, "  -v int                set verbosity level");
   Msg(DIRECT, "  -string \"string\"      parse string before project file");
   Msg(DIRECT, "  -option file          parse option file before GUI creation");
-  Msg(DIRECT, "  -convert file file    perform batch conversion of view(s)/mesh into latest file formats");
+  Msg(DIRECT, "  -convert file file    perform batch conversion of views and meshes into latest file formats");
   Msg(DIRECT, "  -version              show version number");
   Msg(DIRECT, "  -info                 show detailed version information");
   Msg(DIRECT, "  -help                 show this message");
@@ -129,19 +124,26 @@ char *Get_BuildOptions(void)
 #if defined(HAVE_TRIANGLE)
     strcat(opt, "TRIANGLE ");
 #endif
+#if defined(HAVE_NETGEN)
+    strcat(opt, "NETGEN ");
+#endif
 #if defined(HAVE_LIBJPEG)
     strcat(opt, "JPEG ");
 #endif
 #if defined(HAVE_LIBPNG)
     strcat(opt, "PNG ");
+#endif
+#if defined(HAVE_LIBZ)
+    strcat(opt, "ZLIB ");
+#endif
+#if defined(HAVE_MATH_EVAL)
+    strcat(opt, "MATHEVAL ");
 #endif
     first = 0;
   }
   return opt;
 }
 
-// *INDENT-ON*
-
 void Get_Options(int argc, char *argv[], int *nbfiles)
 {
   int i = 1;
@@ -153,19 +155,24 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
 
   // Parse session and option files
 
-  ParseFile(CTX.sessionrc_filename, 1, 1);
-  ParseFile(CTX.optionsrc_filename, 1, 1);
+  ParseFile(CTX.session_filename_fullpath, 1, 1);
+  ParseFile(CTX.options_filename_fullpath, 1, 1);
 
   // Get command line options
 
-  TheFileNameTab[0] = CTX.default_filename;
+  TheFileNameTab[0] = CTX.default_filename_fullpath;
   *nbfiles = 0;
 
   while(i < argc) {
 
     if(argv[i][0] == '-') {
 
-      if(!strcmp(argv[i] + 1, "string")) {
+      if(!strcmp(argv[i] + 1, "pid")) {
+	fprintf(stdout, "%d\n", getpid());
+	fflush(stdout);
+        i++;
+      }
+      else if(!strcmp(argv[i] + 1, "string")) {
         i++;
         if(argv[i] != NULL)
           TheOptString = argv[i++];
@@ -214,11 +221,11 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
         CTX.mesh.save_all = 1;
         i++;
       }
-      else if(!strcmp(argv[i] + 1, "extrude")) {        //old extrusion mesh generator
+      else if(!strcmp(argv[i] + 1, "extrude")) { // old extrusion mesh generator
         CTX.mesh.oldxtrude = 1;
         i++;
       }
-      else if(!strcmp(argv[i] + 1, "recombine")) {      //old extrusion mesh generator
+      else if(!strcmp(argv[i] + 1, "recombine")) { // old extrusion mesh generator
         CTX.mesh.oldxtrude_recombine = 1;
         i++;
       }
@@ -230,6 +237,10 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
         CTX.mesh.histogram = 1;
         i++;
       }
+      else if(!strcmp(argv[i] + 1, "optimize")) {
+        CTX.mesh.optimize = 1;
+        i++;
+      }
       else if(!strcmp(argv[i] + 1, "option")) {
         i++;
         if(argv[i] != NULL)
@@ -395,11 +406,13 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
         i++;
         if(argv[i] != NULL) {
           if(!strncmp(argv[i], "iso", 3))
-            CTX.mesh.algo = DELAUNAY_ISO;
+            CTX.mesh.algo2d = DELAUNAY_ISO;
           else if(!strncmp(argv[i], "tri", 3))
-            CTX.mesh.algo = DELAUNAY_SHEWCHUK;
+            CTX.mesh.algo2d = DELAUNAY_TRIANGLE;
           else if(!strncmp(argv[i], "aniso", 5))
-            CTX.mesh.algo = DELAUNAY_ANISO;
+            CTX.mesh.algo2d = DELAUNAY_ANISO;
+          else if(!strncmp(argv[i], "netgen", 6))
+            CTX.mesh.algo3d = FRONTAL_NETGEN;
           else {
             fprintf(stderr, ERROR_STR "Unknown mesh algorithm\n");
             exit(1);
@@ -451,22 +464,10 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
         }
       }
 #if defined(HAVE_FLTK)
-      else if(!strcmp(argv[i] + 1, "noterm")) {
-        CTX.terminal = 0;
-        i++;
-      }
       else if(!strcmp(argv[i] + 1, "term")) {
         CTX.terminal = 1;
         i++;
       }
-      else if(!strcmp(argv[i] + 1, "alpha")) {
-        CTX.alpha = 1;
-        i++;
-      }
-      else if(!strcmp(argv[i] + 1, "notrack")) {
-        CTX.useTrackball = 0;
-        i++;
-      }
       else if(!strcmp(argv[i] + 1, "dual")) {
         CTX.mesh.dual = 1;
         i++;
@@ -479,14 +480,6 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
         opt_view_visible(0, GMSH_SET, 0);
         i++;
       }
-      else if(!strcmp(argv[i] + 1, "plugin")) {
-        opt_general_default_plugins(0, GMSH_SET, 1);
-        i++;
-      }
-      else if(!strcmp(argv[i] + 1, "noplugin")) {
-        opt_general_default_plugins(0, GMSH_SET, 0);
-        i++;
-      }
       else if(!strcmp(argv[i] + 1, "link")) {
         i++;
         if(argv[i] != NULL)
@@ -517,38 +510,6 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
           exit(1);
         }
       }
-      else if(!strcmp(argv[i] + 1, "command") || !strcmp(argv[i] + 1, "c")) {
-        CTX.command_win = 1;
-        i++;
-      }
-      else if(!strcmp(argv[i] + 1, "nocommand") ||
-              !strcmp(argv[i] + 1, "noc")) {
-        CTX.command_win = 0;
-        i++;
-      }
-      else if(!strcmp(argv[i] + 1, "overlay") || !strcmp(argv[i] + 1, "ov")) {
-        CTX.overlay = 1;
-        i++;
-      }
-      else if(!strcmp(argv[i] + 1, "nooverlay") ||
-              !strcmp(argv[i] + 1, "noov")) {
-        CTX.overlay = CTX.geom.highlight = 0;
-        i++;
-      }
-      else if(!strcmp(argv[i] + 1, "hh")) {
-        CTX.overlay = 0;
-        CTX.geom.highlight = 1;
-        i++;
-      }
-      else if(!strcmp(argv[i] + 1, "perspective") ||
-              !strcmp(argv[i] + 1, "p")) {
-        CTX.ortho = 0;
-        i++;
-      }
-      else if(!strcmp(argv[i] + 1, "ortho") || !strcmp(argv[i] + 1, "o")) {
-        CTX.ortho = 0;
-        i++;
-      }
       else if(!strcmp(argv[i] + 1, "threads")) {
         CTX.threads = 1;
         i++;
@@ -563,15 +524,6 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
       }
       else if(!strcmp(argv[i] + 1, "nodb")) {
         CTX.db = 0;
-        CTX.geom.highlight = 0;
-        i++;
-      }
-      else if(!strcmp(argv[i] + 1, "dl")) {
-        CTX.post.display_lists = 1;
-        i++;
-      }
-      else if(!strcmp(argv[i] + 1, "nodl")) {
-        CTX.post.display_lists = 0;
         i++;
       }
       else if(!strcmp(argv[i] + 1, "fontsize")) {
@@ -608,18 +560,19 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
         }
       }
 #endif
-      else {
 #if defined(__APPLE__)
-        // The Mac Finder launches programs with special command line
-        // arguments: just ignore them (and don't exit!)
-        fprintf(stderr, "Unknown option '%s'\n", argv[i]);
+      else if(!strncmp(argv[i] + 1, "psn", 3)) {
+        // The Mac Finder launches programs with a special command
+        // line argument of the form -psn_XXX: just ignore it silently
+        // (and don't exit!)
         i++;
-#else
+      }
+#endif
+      else {
         fprintf(stderr, "Unknown option '%s'\n", argv[i]);
         CTX.terminal = 1;
         Print_Usage(argv[0]);
         exit(1);
-#endif
       }
     }
 
