@@ -40,15 +40,15 @@
 // STRINGS
 
 StringXString GeneralOptions_String[] = {
-  { F|S, "DefaultFileName" , opt_general_default_filename , "untitled.geo" ,
+  { F|O, "DefaultFileName" , opt_general_default_filename , "untitled.geo" ,
     "Default project file name" },
   { F,   "Display" , opt_general_display , "" ,
     "X server to use (only for Unix versions)" },
 
-  { F|S, "ErrorFileName" , opt_general_error_filename , ".gmsh-errors" ,
+  { F|O, "ErrorFileName" , opt_general_error_filename , ".gmsh-errors" ,
     "File into which the log is saved if a fatal error occurs" },
 
-  { F|S, "GraphicsFont" , opt_general_graphics_font , "Helvetica" ,
+  { F|O, "GraphicsFont" , opt_general_graphics_font , "Helvetica" ,
     "Font used in the graphic window" }, 
 
   { F|S, "OptionsFileName" , opt_general_options_filename , ".gmsh-options" ,
@@ -56,7 +56,7 @@ StringXString GeneralOptions_String[] = {
 
   { 0,   "SessionFileName" , opt_general_session_filename , ".gmshrc" ,
     "Option file into which session specific information is saved; automatically read on startup" },
-  { F|S, "Scheme" , opt_general_scheme , "" ,
+  { F|O, "Scheme" , opt_general_scheme , "" ,
     "FLTK user interface scheme (try e.g. plastic)" },
 
   { F|O, "TextEditor" , opt_general_editor , 
@@ -119,6 +119,10 @@ StringXString SolverOptions_String[] = {
     "Default mesh file name for solver 0" },
   { F|O, "MeshCommand0" , opt_solver_mesh_command0 , "-msh %s" ,
     "Command used to specify the mesh file for solver 0" },
+  { F|O, "SocketCommand0", opt_solver_socket_command0, "-socket %s",
+    "Command to specify the socket to solver 0" },
+  { F|O, "NameCommand0", opt_solver_name_command0, "%s",
+    "Command to specify the problem name to solver 0" },
   { F|O, "OptionCommand0" , opt_solver_option_command0 , "" ,
     "Command to get options from solver 0" },
   { F|O, "FirstOption0" , opt_solver_first_option0 , "Resolution" ,
@@ -164,6 +168,10 @@ StringXString SolverOptions_String[] = {
     "Default mesh file name for solver 1" },
   { F|O, "MeshCommand1" , opt_solver_mesh_command1 , "" ,
     "Command used to specify the mesh file for solver 1" },
+  { F|O, "SocketCommand1", opt_solver_socket_command1, "-socket %s",
+    "Command to specify the socket to solver 1" },
+  { F|O, "NameCommand1", opt_solver_name_command1, "%s",
+    "Command to specify the problem name to solver 1" },
   { F|O, "OptionCommand1" , opt_solver_option_command1 , "" ,
     "Command to get options from solver 1" },
   { F|O, "FirstOption1" , opt_solver_first_option1 , "" ,
@@ -209,6 +217,10 @@ StringXString SolverOptions_String[] = {
     "Default mesh file name for solver 2" },
   { F|O, "MeshCommand2" , opt_solver_mesh_command2 , "" ,
     "Command used to specify the mesh file for solver 2" },
+  { F|O, "SocketCommand2", opt_solver_socket_command2, "-socket %s",
+    "Command to specify the socket to solver 2" },
+  { F|O, "NameCommand2", opt_solver_name_command2, "%s",
+    "Command to specify the problem name to solver 2" },
   { F|O, "OptionCommand2" , opt_solver_option_command2 , "" ,
     "Command to get options from solver 2" },
   { F|O, "FirstOption2" , opt_solver_first_option2 , "" ,
@@ -254,6 +266,10 @@ StringXString SolverOptions_String[] = {
     "Default mesh file name for solver 3" },
   { F|O, "MeshCommand3" , opt_solver_mesh_command3 , "" ,
     "Command used to specify the mesh file for solver 3" },
+  { F|O, "SocketCommand3", opt_solver_socket_command3, "-socket %s",
+    "Command to specify the socket to solver 3" },
+  { F|O, "NameCommand3", opt_solver_name_command3, "%s",
+    "Command to specify the problem name to solver 3" },
   { F|O, "OptionCommand3" , opt_solver_option_command3 , "" ,
     "Command to get options from solver 3" },
   { F|O, "FirstOption3" , opt_solver_first_option3 , "" ,
@@ -299,6 +315,10 @@ StringXString SolverOptions_String[] = {
     "Default mesh file name for solver 4" },
   { F|O, "MeshCommand4" , opt_solver_mesh_command4 , "" ,
     "Command used to specify the mesh file for solver 4" },
+  { F|O, "SocketCommand4", opt_solver_socket_command4, "-socket %s",
+    "Command to specify the socket to solver 4" },
+  { F|O, "NameCommand4", opt_solver_name_command1, "%s",
+    "Command to specify the problem name to solver 4" },
   { F|O, "OptionCommand4" , opt_solver_option_command4 , "" ,
     "Command to get options from solver 4" },
   { F|O, "FirstOption4" , opt_solver_first_option4 , "" ,
@@ -372,95 +392,104 @@ StringXNumber GeneralOptions_Number[] = {
     "Relative radius of arrow stem" },
   { F|O, "Axes" , opt_general_axes , 0. ,
     "Display the axes linked to the model" },
-
   { F,   "Clip0" , opt_general_clip0 , 0. ,
-    "Enable clip plane 0" },
-  { F,   "Clip0A" , opt_general_clip0a , 0.0 ,
-    "First coefficient in equation for clip plane 0 (`A' in `AX+BY+CZ+D=0')" },
+    "Enable clipping plane 0 (Geometry=2^0, Mesh=2^1, View[i]=2^(2+i))" },
+  { F,   "Clip0A" , opt_general_clip0a , 1.0 ,
+    "First coefficient in equation for clipping plane 0 (`A' in `AX+BY+CZ+D=0')" },
   { F,   "Clip0B" , opt_general_clip0b , 0.0 , 
-    "Second coefficient in equation for clip plane 0 (`B' in `AX+BY+CZ+D=0')" },
+    "Second coefficient in equation for clipping plane 0 (`B' in `AX+BY+CZ+D=0')" },
   { F,   "Clip0C" , opt_general_clip0c , 0.0 , 
-    "Third coefficient in equation for clip plane 0 (`C' in `AX+BY+CZ+D=0')" },
+    "Third coefficient in equation for clipping plane 0 (`C' in `AX+BY+CZ+D=0')" },
   { F,   "Clip0D" , opt_general_clip0d , 0.0 , 
-    "Fourth coefficient in equation for clip plane 0 (`D' in `AX+BY+CZ+D=0')" },
+    "Fourth coefficient in equation for clipping plane 0 (`D' in `AX+BY+CZ+D=0')" },
   { F,   "Clip1" , opt_general_clip1 , 0.,
-    "Enable clip plane 1" },
-  { F,   "Clip1A" , opt_general_clip1a , 0.0 , 
-    "First coefficient in equation for clip plane 1" },
+    "Enable clipping plane 1 (Geometry=2^0, Mesh=2^1, View[i]=2^(2+i))" },
+  { F,   "Clip1A" , opt_general_clip1a , 1.0 , 
+    "First coefficient in equation for clipping plane 1" },
   { F,   "Clip1B" , opt_general_clip1b , 0.0 , 
-    "Second coefficient in equation for clip plane 1" },
+    "Second coefficient in equation for clipping plane 1" },
   { F,   "Clip1C" , opt_general_clip1c , 0.0 , 
-    "Third coefficient in equation for clip plane 1" },
+    "Third coefficient in equation for clipping plane 1" },
   { F,   "Clip1D" , opt_general_clip1d , 0.0 , 
-    "Fourth coefficient in equation for clip plane 1" },
+    "Fourth coefficient in equation for clipping plane 1" },
   { F,   "Clip2" , opt_general_clip2 , 0.,
-    "Enable clip plane 2" },
-  { F,   "Clip2A" , opt_general_clip2a , 0.0 , 
-    "First coefficient in equation for clip plane 2" },
+    "Enable clipping plane 2 (Geometry=2^0, Mesh=2^1, View[i]=2^(2+i))" },
+  { F,   "Clip2A" , opt_general_clip2a , 1.0 , 
+    "First coefficient in equation for clipping plane 2" },
   { F,   "Clip2B" , opt_general_clip2b , 0.0 , 
-    "Second coefficient in equation for clip plane 2" },
+    "Second coefficient in equation for clipping plane 2" },
   { F,   "Clip2C" , opt_general_clip2c , 0.0 , 
-    "Third coefficient in equation for clip plane 2" },
+    "Third coefficient in equation for clipping plane 2" },
   { F,   "Clip2D" , opt_general_clip2d , 0.0 , 
-    "Fourth coefficient in equation for clip plane 2" },
+    "Fourth coefficient in equation for clipping plane 2" },
   { F,   "Clip3" , opt_general_clip3 , 0.,
-    "Enable clip plane 3" },
-  { F,   "Clip3A" , opt_general_clip3a , 0.0 , 
-    "First coefficient in equation for clip plane 3" },
+    "Enable clipping plane 3 (Geometry=2^0, Mesh=2^1, View[i]=2^(2+i))" },
+  { F,   "Clip3A" , opt_general_clip3a , 1.0 , 
+    "First coefficient in equation for clipping plane 3" },
   { F,   "Clip3B" , opt_general_clip3b , 0.0 , 
-    "Second coefficient in equation for clip plane 3" },
+    "Second coefficient in equation for clipping plane 3" },
   { F,   "Clip3C" , opt_general_clip3c , 0.0 , 
-    "Third coefficient in equation for clip plane 3" },
+    "Third coefficient in equation for clipping plane 3" },
   { F,   "Clip3D" , opt_general_clip3d , 0.0 , 
-    "Fourth coefficient in equation for clip plane 3" },
+    "Fourth coefficient in equation for clipping plane 3" },
   { F,   "Clip4" , opt_general_clip4 , 0.,
-    "Enable clip plane 4" },
-  { F,   "Clip4A" , opt_general_clip4a , 0.0 , 
-    "First coefficient in equation for clip plane 4" },
+    "Enable clipping plane 4 (Geometry=2^0, Mesh=2^1, View[i]=2^(2+i))" },
+  { F,   "Clip4A" , opt_general_clip4a , 1.0 , 
+    "First coefficient in equation for clipping plane 4" },
   { F,   "Clip4B" , opt_general_clip4b , 0.0 , 
-    "Second coefficient in equation for clip plane 4" },
+    "Second coefficient in equation for clipping plane 4" },
   { F,   "Clip4C" , opt_general_clip4c , 0.0 , 
-    "Third coefficient in equation for clip plane 4" },
+    "Third coefficient in equation for clipping plane 4" },
   { F,   "Clip4D" , opt_general_clip4d , 0.0 , 
-    "Fourth coefficient in equation for clip plane 4" },
+    "Fourth coefficient in equation for clipping plane 4" },
   { F,   "Clip5" , opt_general_clip5 , 0.,
-    "Enable clip plane 5" },
-  { F,   "Clip5A" , opt_general_clip5a , 0.0 , 
-    "First coefficient in equation for clip plane 5" },
+    "Enable clipping plane 5 (Geometry=2^0, Mesh=2^1, View[i]=2^(2+i))" },
+  { F,   "Clip5A" , opt_general_clip5a , 1.0 , 
+    "First coefficient in equation for clipping plane 5" },
   { F,   "Clip5B" , opt_general_clip5b , 0.0 , 
-    "Second coefficient in equation for clip plane 5" },
+    "Second coefficient in equation for clipping plane 5" },
   { F,   "Clip5C" , opt_general_clip5c , 0.0 , 
-    "Third coefficient in equation for clip plane 5" },
+    "Third coefficient in equation for clipping plane 5" },
   { F,   "Clip5D" , opt_general_clip5d , 0.0 , 
-    "Fourth coefficient in equation for clip plane 5" },
+    "Fourth coefficient in equation for clipping plane 5" },
+  { F|S, "ClipPositionX" , opt_general_clip_position0 , 650. , 
+    "Horizontal position (in pixels) of the upper left corner of the clipping planes window" }, 
+  { F|S, "ClipPositionY" , opt_general_clip_position1 , 150. , 
+    "Vertical position (in pixels) of the upper left corner of the clipping planes window" }, 
   { F|O, "ColorScheme", opt_general_color_scheme , 0. ,
     "Default color scheme (0, 1 or 2)" },
-  { F|S, "ConfirmOverwrite" , opt_general_confirm_overwrite, 1. , 
+  { F|O, "ConfirmOverwrite" , opt_general_confirm_overwrite, 1. , 
     "Ask confirmation before overwriting files?" }, 
   { F|S, "ContextPositionX" , opt_general_context_position0 , 650. , 
     "Horizontal position (in pixels) of the upper left corner of the contextual windows" }, 
   { F|S, "ContextPositionY" , opt_general_context_position1 , 150. ,
     "Vertical position (in pixels) of the upper left corner of the contextual windows" }, 
 
-  { F|O, "DefaultPlugins" , opt_general_default_plugins , 1. ,
-    "Load default plugins on startup" },
   { F|O, "DoubleBuffer" , opt_general_double_buffer , 1. ,
     "Use a double buffered graphic window (on Unix, should be set to 0 when working on a remote host without GLX)" },
+  { F|O, "DrawBoundingBoxes" , opt_general_draw_bounding_box, 0. ,
+    "Draw bounding boxes" },
 
-  { F|O, "FastRedraw" , opt_general_fast_redraw , 1. ,
-    "Fast redraw (no mesh or view display) when interacting with the model" },
-  { F|S, "FontSize" , opt_general_fontsize , 12. , 
+  { F|O, "FakeTransparency" , opt_general_fake_transparency , 0. ,
+    "Use fake transparency (cheaper than the real thing, but incorrect)" },
+  { F|O, "FastRedraw" , opt_general_fast_redraw, 1. ,
+    "Draw simplified model while rotating, panning and zooming" },
+  { F|S, "FileChooserPositionX" , opt_general_file_chooser_position0 , 200. , 
+    "Horizontal position (in pixels) of the upper left corner of the file chooser windows" }, 
+  { F|S, "FileChooserPositionY" , opt_general_file_chooser_position1 , 200. ,
+    "Vertical position (in pixels) of the upper left corner of the file chooser windows" }, 
+  { F|O, "FontSize" , opt_general_fontsize , 12. , 
     "Size of the font in the user interface" },
 
-  { F|S, "GraphicsFontSize" , opt_general_graphics_fontsize , 14. ,
+  { F|O, "GraphicsFontSize" , opt_general_graphics_fontsize , 14. ,
     "Size of the font in the graphic window" }, 
-  { F|S, "GraphicsHeight" , opt_general_viewport3 , 500. , 
+  { F|S, "GraphicsHeight" , opt_general_viewport3 , 600. , 
     "Height (in pixels) of the graphic window" }, 
-  { F|S, "GraphicsPositionX" , opt_general_graphics_position0 , 20. , 
+  { F|S, "GraphicsPositionX" , opt_general_graphics_position0 , 50. , 
     "Horizontal position (in pixels) of the upper left corner of the graphic window" }, 
-  { F|S, "GraphicsPositionY" , opt_general_graphics_position1 , 30. ,
+  { F|S, "GraphicsPositionY" , opt_general_graphics_position1 , 50. ,
     "Vertical position (in pixels) of the upper left corner of the graphic window" }, 
-  { F|S, "GraphicsWidth" , opt_general_viewport2 , 700. , 
+  { F|S, "GraphicsWidth" , opt_general_viewport2 , 600. , 
     "Width (in pixels) of the graphic window" }, 
 
   { F|O, "InitialModule", opt_general_initial_context, 0. , 
@@ -474,6 +503,8 @@ StringXNumber GeneralOptions_Number[] = {
     "Y position of light source 0" },
   { F|O, "Light0Z" , opt_general_light02 , 1.0 , 
     "Z position of light source 0" },
+  { F|O, "Light0W" , opt_general_light03 , 0.0 , 
+    "Divisor of the X, Y and Z coordinates of light source 0 (W=0 means infinitely far source)" },
   { F|O, "Light1" , opt_general_light1 , 0.,
     "Enable light source 1" },
   { F|O, "Light1X" , opt_general_light10 , 0.5 , 
@@ -482,6 +513,8 @@ StringXNumber GeneralOptions_Number[] = {
     "Y position of light source 1" },
   { F|O, "Light1Z" , opt_general_light12 , 1.0 , 
     "Z position of light source 1" },
+  { F|O, "Light1W" , opt_general_light13 , 0.0 , 
+    "Divisor of the X, Y and Z coordinates of light source 1 (W=0 means infinitely far source)" },
   { F|O, "Light2" , opt_general_light2 , 0.,
     "Enable light source 2" },
   { F|O, "Light2X" , opt_general_light20 , 0.5 , 
@@ -490,6 +523,8 @@ StringXNumber GeneralOptions_Number[] = {
     "Y position of light source 2" },
   { F|O, "Light2Z" , opt_general_light22 , 1.0 , 
     "Z position of light source 2" },
+  { F|O, "Light2W" , opt_general_light23 , 0.0 , 
+    "Divisor of the X, Y and Z coordinates of light source 2 (W=0 means infinitely far source)" },
   { F|O, "Light3" , opt_general_light3 , 0.,
     "Enable light source 3" },
   { F|O, "Light3X" , opt_general_light30 , 0.5 , 
@@ -498,6 +533,8 @@ StringXNumber GeneralOptions_Number[] = {
     "Y position of light source 3" },
   { F|O, "Light3Z" , opt_general_light32 , 1.0 , 
     "Z position of light source 3" },
+  { F|O, "Light3W" , opt_general_light33 , 0.0 , 
+    "Divisor of the X, Y and Z coordinates of light source 3 (W=0 means infinitely far source)" },
   { F|O, "Light4" , opt_general_light4 , 0.,
     "Enable light source 4" },
   { F|O, "Light4X" , opt_general_light40 , 0.5 , 
@@ -506,6 +543,8 @@ StringXNumber GeneralOptions_Number[] = {
     "Y position of light source 4" },
   { F|O, "Light4Z" , opt_general_light42 , 1.0 , 
     "Z position of light source 4" },
+  { F|O, "Light4W" , opt_general_light43 , 0.0 , 
+    "Divisor of the X, Y and Z coordinates of light source 4 (W=0 means infinitely far source)" },
   { F|O, "Light5" , opt_general_light5 , 0.,
     "Enable light source 5" },
   { F|O, "Light5X" , opt_general_light50 , 0.5 , 
@@ -514,6 +553,8 @@ StringXNumber GeneralOptions_Number[] = {
     "Y position of light source 5" },
   { F|O, "Light5Z" , opt_general_light52 , 1.0 , 
     "Z position of light source 5" },
+  { F|O, "Light5W" , opt_general_light53 , 0.0 , 
+    "Divisor of the X, Y and Z coordinates of light source 5 (W=0 means infinitely far source)" },
   { F|O, "LineWidth" , opt_general_line_width , 1.0 , 
     "Display width of lines (in pixels)" },
 
@@ -550,7 +591,7 @@ StringXNumber GeneralOptions_Number[] = {
   { F,   "RotationZ" , opt_general_rotation2 , 0.0 , 
     "Third Euler angle (used if Trackball=0)" }, 
   { F|O, "RotationCenterGravity" , opt_general_rotation_center_cg , 1. ,
-    "Rotate around the center of gravity instead of (RotationCenterX, RotationCenterY, RotationCenterZ)" },
+    "Rotate around the (pseudo) center of mass instead of (RotationCenterX, RotationCenterY, RotationCenterZ)" },
   { F,   "RotationCenterX" , opt_general_rotation_center0 , 0.0 , 
     "X coordinate of the center of rotation" }, 
   { F,   "RotationCenterY" , opt_general_rotation_center1 , 0.0 , 
@@ -569,7 +610,9 @@ StringXNumber GeneralOptions_Number[] = {
   { F,   "ScaleZ" , opt_general_scale2 , 1.0 , 
     "Z-axis scale factor" },
   { F|O, "Shininess" , opt_general_shine , 0.4 , 
-    "Material shininess (must be > 0)" },
+    "Material shininess" },
+  { F|O, "ShininessExponent" , opt_general_shine_exponent , 40. , 
+    "Material shininess exponent (between 0 and 128)" },
   { F|O, "SmallAxes" , opt_general_small_axes , 1. ,
     "Display the small axes" },
   { F|O, "SmallAxesPositionX" , opt_general_small_axes_position0 , -60. ,
@@ -615,9 +658,9 @@ StringXNumber GeneralOptions_Number[] = {
 
   { F|O, "VectorType" , opt_general_vector_type , DRAW_POST_ARROW3D ,
     "Default vector display type (for normals, etc.)" },
-  { F|O, "Verbosity" , opt_general_verbosity , 2. ,
+  { F|O, "Verbosity" , opt_general_verbosity , 3. ,
     "Level of information printed during processing (0=no information)" },
-  { F|S, "VisibilityMode" , opt_general_visibility_mode , 0. , 
+  { F|O, "VisibilityMode" , opt_general_visibility_mode , 0. , 
     "Default mode for the visibility browser (0=Geometry+Mesh, 1=Geometry, 2=Mesh)" }, 
   { F|S, "VisibilityPositionX" , opt_general_visibility_position0 , 650. , 
     "Horizontal position (in pixels) of the upper left corner of the visibility window" }, 
@@ -636,17 +679,12 @@ StringXNumber GeometryOptions_Number[] = {
 
   { F|O, "CirclePoints" , opt_geometry_circle_points, 20. ,
     "Number of points used to draw a circle/ellipse" },
-  { F|O, "ColorScheme" , opt_geometry_color_scheme , 0. , 
-    "Default geometry color scheme (0, 1 or 2)" },
   { F|O, "CircleWarning" , opt_geometry_circle_warning, 1. ,
     "Warn if circle arc is greater than Pi" },
 
   { F|O, "ExtrudeSplinePoints" , opt_geometry_extrude_spline_points, 5. ,
     "Number of control points for splines created during extrusion" },
 
-  { F|O, "Highlight" , opt_geometry_highlight , 1. , 
-    "Not used" },
-
   { F|O, "Light" , opt_geometry_light , 1. , 
     "Enable lighting for the geometry" },
   { F|O, "Lines" , opt_geometry_lines , 1. , 
@@ -702,17 +740,19 @@ StringXNumber GeometryOptions_Number[] = {
 } ;
 
 StringXNumber MeshOptions_Number[] = {
-  { F|O, "Algorithm" , opt_mesh_algo , DELAUNAY_ISO ,
+  { F|O, "Algorithm" , opt_mesh_algo2d , DELAUNAY_ISO ,
     "2D mesh algorithm (1=isotropic, 2=anisotropic, 3=triangle)" }, 
+  { F|O, "Algorithm3D" , opt_mesh_algo3d , DELAUNAY_ISO ,
+    "3D mesh algorithm (1=isotropic, 4=netgen)" }, 
   { F,   "AllowDegeneratedExtrude" , opt_mesh_allow_degenerated_extrude , 0. , 
     "Allow the generation of degenerated hexahedra or prisms during extrusion" },
+  { F|O, "AngleSmoothNormals" , opt_mesh_angle_smooth_normals , 180.0 ,
+    "Threshold angle below which normals are not smoothed" }, 
 
   { F|O, "CharacteristicLengthFactor" , opt_mesh_lc_factor , 1.0 ,
     "Factor applied to all characteristic lengths (and background meshes)" },
   { F|O, "ColorCarousel" , opt_mesh_color_carousel , 1. ,
-    "Color mesh by region? (0=no, 1=by elementary entity, 2=by physical entity, 3=by partition)" },
-  { F|O, "ColorScheme" , opt_mesh_color_scheme , 0. , 
-    "Default mesh color scheme (0, 1 or 2)" },
+    "Mesh coloring (0=by element type, 1=by elementary entity, 2=by physical entity, 3=by partition)" },
   { F|O, "ConstrainedBackgroundMesh" , opt_mesh_constrained_bgmesh, 0. ,
     "Should the background mesh be constrained by the characteristic lengths associated with the geometry?" },
   { F,   "CpuTime" , opt_mesh_cpu_time , 0. , 
@@ -721,24 +761,24 @@ StringXNumber MeshOptions_Number[] = {
     "Enable mesh cut plane" },
   { F,   "CutPlaneAsSurface" , opt_mesh_cut_plane_as_surface , 0 ,
     "Draw the intersection volume layer as a surface" },
-  { F,   "CutPlaneA" , opt_mesh_cut_planea , 1. , 
+  { F,   "CutPlaneOnlyVolume" , opt_mesh_cut_plane_only_volume , 0 ,
+    "Cut only the volume elements" },
+  { F,   "CutPlaneA" , opt_mesh_cut_planea , 1.0 , 
     "First cut plane equation coefficient (`A' in `AX+BY+CZ+D=0')" },
-  { F,   "CutPlaneB" , opt_mesh_cut_planeb , 0. , 
+  { F,   "CutPlaneB" , opt_mesh_cut_planeb , 0.0 , 
     "Second cut plane equation coefficient (`B' in `AX+BY+CZ+D=0')" },
-  { F,   "CutPlaneC" , opt_mesh_cut_planec , 0. , 
+  { F,   "CutPlaneC" , opt_mesh_cut_planec , 0.0 , 
     "Third cut plane equation coefficient (`C' in `AX+BY+CZ+D=0')" },
-  { F,   "CutPlaneD" , opt_mesh_cut_planed , 0. , 
+  { F,   "CutPlaneD" , opt_mesh_cut_planed , 0.0 , 
     "Fourth cut plane equation coefficient (`D' in `AX+BY+CZ+D=0')" },
 
-  { F|O, "DisplayLists" , opt_mesh_display_lists , 0. ,
-    "Use OpenGL display lists for drawing meshes" },
   { F|O, "Dual" , opt_mesh_dual , 0. ,
     "Display the dual mesh obtained by barycentric subdivision" },
 
   { F|O, "ElementOrder" , opt_mesh_order , 1. , // "Order" is already a lex token
     "Element order (1=linear elements, 2=quadratic elements)" },
   { F|O, "Explode" , opt_mesh_explode , 1.0 ,
-    "Display mesh with non adjacent elements (factor between 0 and 1)" },
+    "Explode elements (between 0=point and 1=non-transformed)" },
 
   { F|O, "Format" , opt_mesh_format , FORMAT_MSH , 
     "Mesh output format (1=MSH, 2=UNV, 3=GREF, 19=VRML)" },
@@ -753,6 +793,8 @@ StringXNumber MeshOptions_Number[] = {
 
   { F|O, "Light" , opt_mesh_light , 0. , 
     "Enable lighting for the mesh" },
+  { F|O, "LightTwoSide" , opt_mesh_light_two_side , 1. , 
+    "Light both sides of mesh elements (leads to slower rendering)" },
   { F|O, "Lines" , opt_mesh_lines , 0. , 
     "Display mesh lines (1D elements)?" },
   { F|O, "LineNumbers" , opt_mesh_lines_num , 0. , 
@@ -784,8 +826,13 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "Normals" , opt_mesh_normals , 0.0 ,
     "Display size of normal vectors (in pixels)" }, 
 
+  { F|O, "Optimize" , opt_mesh_optimize , 0. , 
+    "Optimize the mesh using Netgen to improve the quality of tetrahedral elements" },
+
   { F|O, "Points" , opt_mesh_points , 1. , 
     "Display mesh vertices (nodes)?" },
+  { F|O, "PointsPerElement" , opt_mesh_points_per_element , 0. , 
+    "Display mesh nodes per element (slower, but permits to visualize only a subset of the nodes)" },
   { F|O, "PointInsertion" , opt_mesh_point_insertion, CENTER_CIRCCIRC ,
     "Point insertion method for isotropic 2D algorithm (1=center of circumscribed circle, 2=cog)" },
   { F|O, "PointNumbers" , opt_mesh_points_num , 0. , 
@@ -795,8 +842,8 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "PointType" , opt_mesh_point_type , 0. , 
     "Display mesh vertices as solid color dots (0) or 3D spheres (1)" },
 
-  { F|O, "Quality" , opt_mesh_quality , 0.0 ,
-    "Target quality for tetrahedral elements (not fully functional)" },
+  //{ F|O, "Quality" , opt_mesh_quality , 0.0 ,
+  //  "Target quality for tetrahedral elements (not fully functional)" },
 
   { F|O, "RadiusInf" , opt_mesh_radius_inf , 0.0 , 
     "Only display elements whose Radius is greater than RadiusInf" },
@@ -809,8 +856,10 @@ StringXNumber MeshOptions_Number[] = {
     "Ignore Physical definitions and save all elements" },
   { F|O, "ScalingFactor" , opt_mesh_scaling_factor , 1.0 ,
     "Global scaling factor applied to the saved mesh" },
-  { F|O, "Smoothing" , opt_mesh_nb_smoothing , 0. ,
+  { F|O, "Smoothing" , opt_mesh_nb_smoothing , 1. ,
     "Number of smoothing steps applied to the final mesh" },
+  { F|O, "SmoothNormals" , opt_mesh_smooth_normals , 0. , 
+    "Smooth the mesh normals?" },
   { F|O, "SpeedMax" , opt_mesh_speed_max , 0. ,
     "Disable dubious point insertion tests" },
   { F|O, "SurfaceEdges" , opt_mesh_surfaces_edges , 1. , 
@@ -823,6 +872,8 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "Tangents" , opt_mesh_tangents , 0.0 , 
     "Display size of tangent vectors (in pixels)" }, 
 
+  { F|O, "VertexArrays" , opt_mesh_vertex_arrays , 1. , 
+    "Use OpenGL vertex arrays to draw triangular meshes?" },
   { F|O, "VolumeEdges" , opt_mesh_volumes_edges , 1. , 
     "Display edges of volume mesh?" },
   { F|O, "VolumeFaces" , opt_mesh_volumes_faces , 0. , 
@@ -834,6 +885,12 @@ StringXNumber MeshOptions_Number[] = {
 } ;
 
 StringXNumber SolverOptions_Number[] = {
+  { F|O, "MaximumDelay" , opt_solver_max_delay , 4.0 ,
+    "Maximum delay allowed for solver response (in seconds)" },
+
+  { F|O, "Plugins" , opt_solver_plugins , 0. ,
+    "Enable default solver plugins?" },
+
   { F|O, "ClientServer0" , opt_solver_client_server0 , 1. ,
     "Connect solver 0 to the Gmsh server" },
   { F|O, "MergeViews0" , opt_solver_merge_views0 , 1. , 
@@ -883,20 +940,32 @@ StringXNumber PostProcessingOptions_Number[] = {
   { F|O, "AnimationCycle" , opt_post_anim_cycle , 0. ,
     "Cycle through views instead of time steps in automatic animation mode" },
 
-  { F|O, "DisplayLists" , opt_post_display_lists , 0. ,
-    "Use OpenGL display lists for drawing post-processing views" },
+  { F|O, "CombineRemoveOriginal" , opt_post_combine_remove_orig , 1. ,
+    "Remove original views after a Combine operation" },
+
+  { F|0, "Format" , opt_post_file_format , 0. ,
+    "Default file format for post-processing views" },
+
+  { F|O, "HorizontalScales" , opt_post_horizontal_scales , 0. , 
+    "Display value scales horizontally" },
 
   { F|O, "Link" , opt_post_link , 0. ,
-    "Link post-processing views (0=none, 1,2=changes in visible/all, 3,4=everything in visible/all)" },
+    "Link post-processing views (0=none, 1/2=changes in visible/all, 3/4=everything in visible/all)" },
 
   { F,   "NbViews" , opt_post_nb_views , 0. ,//this default val is not used
     "Current number of views merged" },
 
+  { F|O, "Plugins" , opt_post_plugins , 1. ,
+    "Enable default post-processing plugins?" },
+
   { F|O, "Scales" , opt_post_scales , 1. , 
     "Show value scales" },
   { F|O, "Smoothing" , opt_post_smooth , 0. ,
     "Apply (non-reversible) smoothing to post-processing view when merged" },
 
+  { F|O, "VertexArrays" , opt_post_vertex_arrays , 1. ,
+    "Use OpenGL vertex arrays to draw triangles in post-processing views?" },
+
   { 0, NULL , NULL , 0. }
 } ;
 
@@ -911,6 +980,8 @@ StringXNumber ViewOptions_Number[] = {
     "Arrow location (1=cog, 2=node)" },
   { F|O, "ArrowSize" , opt_view_arrow_size , 60. ,
     "Display size of arrows (in pixels)" },
+  { F|O, "ArrowSizeProportional" , opt_view_arrow_size_proportional , 1. ,
+    "Scale the arrows according to the norm of the vector" },
   { F|O, "ArrowStemLength" , opt_view_arrow_stem_length , 0.56 ,
     "Relative length of arrow stem" },
   { F|O, "ArrowStemRadius" , opt_view_arrow_stem_radius , 0.02 ,
@@ -954,7 +1025,9 @@ StringXNumber ViewOptions_Number[] = {
     "Display vector values?" },
 
   { F|O, "Explode" , opt_view_explode , 1. , 
-    "Explode elements (0: reduced to point; 1: not transformed)" },
+    "Explode elements (between 0=point and 1=non-transformed)" },
+  { F|O, "ExternalView" , opt_view_external_view , -1. ,
+    "Index of the view used to color vector fields (-1=self)" },
 
   { F|O, "Grid" , opt_view_grid , 2 ,
     "Grid mode for 2D graphs (0=none, 1=simple, 2=frame, 3=grid)" },
@@ -967,6 +1040,8 @@ StringXNumber ViewOptions_Number[] = {
 
   { F|O, "Light" , opt_view_light , 1. ,
     "Enable lighting for the view" },
+  { F|O, "LightTwoSide" , opt_view_light_two_side , 1. , 
+    "Light both sides of view elements (leads to slower rendering)" },
   { F|O, "LineType" , opt_view_line_type , 0. , 
     "Display lines as solid color segments (0) or 3D cylinders (1)" },
   { F|O, "LineWidth" , opt_view_line_width , 1.0 , 
@@ -1025,9 +1100,7 @@ StringXNumber ViewOptions_Number[] = {
     "Tensor Visualization Type" },
   { F,   "TimeStep" , opt_view_timestep , 0. ,
     "Current time step displayed" },
-  { F|O, "TransparentScale" , opt_view_transparent_scale , 1. ,
-    "Display a `transparent' value scale?" },
-  { F|O, "Type" , opt_view_type , DRAW_POST_3D ,
+  { F, "Type" , opt_view_type , DRAW_POST_3D ,
     "Type of graph (1=3D, 2=2D-space, 3=2D-time)" },
 
   { F|O, "VectorType" , opt_view_vector_type , DRAW_POST_ARROW3D ,
@@ -1107,6 +1180,21 @@ StringXColor GeneralOptions_Color[] = {
     PACK_COLOR(0,   0,   0,   255),
     PACK_COLOR(0,   0,   0,   255),
     "Small axes color" },
+  { F|O, "AmbientLight" , opt_general_color_ambient_light,
+    PACK_COLOR(25, 25, 25, 255),
+    PACK_COLOR(25, 25, 25, 255),
+    PACK_COLOR(25, 25, 25, 255),
+    "Ambient light color" },
+  { F|O, "DiffuseLight" , opt_general_color_diffuse_light,
+    PACK_COLOR(255, 255, 255, 255),
+    PACK_COLOR(255, 255, 255, 255),
+    PACK_COLOR(255, 255, 255, 255),
+    "Diffuse light color" },
+  { F|O, "SpecularLight" , opt_general_color_specular_light,
+    PACK_COLOR(255, 255, 255, 255),
+    PACK_COLOR(255, 255, 255, 255),
+    PACK_COLOR(255, 255, 255, 255),
+    "Specular light color" },
   { 0, NULL , NULL ,  0, 0, 0 , NULL }
 } ;
 
@@ -1151,26 +1239,6 @@ StringXColor GeometryOptions_Color[] = {
     PACK_COLOR(255, 0,   0,   255),
     PACK_COLOR(0,   0,   0,   255),
     "Selected geometry volume color" },
-  { F|O, "PointsHighlight" , opt_geometry_color_points_highlight ,
-    PACK_COLOR(0,   255, 0,   255),
-    PACK_COLOR(0,   255, 0,   255),
-    PACK_COLOR(0,   0,   0,   255),
-    "Highlighted geometry point color" },
-  { F|O, "LinesHighlight" , opt_geometry_color_lines_highlight ,
-    PACK_COLOR(0,   0,   255, 255),
-    PACK_COLOR(0,   0,   255, 255),
-    PACK_COLOR(0,   0,   0,   255),
-    "Highlighted geometry curve color" },
-  { F|O, "SurfacesHighlight" , opt_geometry_color_surfaces_highlight ,
-    PACK_COLOR(128, 128, 128, 255),
-    PACK_COLOR(128, 128, 128, 255),
-    PACK_COLOR(0,   0,   0,   255),
-    "Highlighted geometry surface color" },
-  { F|O, "VolumesHighlight" , opt_geometry_color_volumes_highlight ,
-    PACK_COLOR(128, 128, 128, 255),
-    PACK_COLOR(128, 128, 128, 255),
-    PACK_COLOR(0,   0,   0,   255),
-    "Highlighted geometry volume color" },
   { F|O, "Tangents" , opt_geometry_color_tangents ,
     PACK_COLOR(255, 255, 0,   255),
     PACK_COLOR(255, 255, 0,   255),
