@@ -22,6 +22,13 @@
 
 #if defined(HAVE_FLTK)
 #  include <FL/Fl.H>
+#  if (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION == 1)
+#    if (FL_PATCH_VERSION >= 5)
+#      define HAVE_FLTK_1_1_5_OR_ABOVE
+#    endif
+#  else
+#    error "Gmsh requires FLTK 1.1.x"
+#  endif
 #  include <FL/gl.h>
 #  if defined(__APPLE__)
 #    include <OpenGL/glu.h>
