@@ -83,6 +83,18 @@ char * opt_solver_mesh_command1(OPT_ARGS_STR);
 char * opt_solver_mesh_command2(OPT_ARGS_STR);
 char * opt_solver_mesh_command3(OPT_ARGS_STR);
 char * opt_solver_mesh_command4(OPT_ARGS_STR);
+char * opt_solver_socket_command(OPT_ARGS_STR);
+char * opt_solver_socket_command0(OPT_ARGS_STR);
+char * opt_solver_socket_command1(OPT_ARGS_STR);
+char * opt_solver_socket_command2(OPT_ARGS_STR);
+char * opt_solver_socket_command3(OPT_ARGS_STR);
+char * opt_solver_socket_command4(OPT_ARGS_STR);
+char * opt_solver_name_command(OPT_ARGS_STR);
+char * opt_solver_name_command0(OPT_ARGS_STR);
+char * opt_solver_name_command1(OPT_ARGS_STR);
+char * opt_solver_name_command2(OPT_ARGS_STR);
+char * opt_solver_name_command3(OPT_ARGS_STR);
+char * opt_solver_name_command4(OPT_ARGS_STR);
 char * opt_solver_option_command(OPT_ARGS_STR);
 char * opt_solver_option_command0(OPT_ARGS_STR);
 char * opt_solver_option_command1(OPT_ARGS_STR);
@@ -196,6 +208,8 @@ double opt_general_solver_position0(OPT_ARGS_NUM);
 double opt_general_solver_position1(OPT_ARGS_NUM);
 double opt_general_context_position0(OPT_ARGS_NUM);
 double opt_general_context_position1(OPT_ARGS_NUM);
+double opt_general_file_chooser_position0(OPT_ARGS_NUM);
+double opt_general_file_chooser_position1(OPT_ARGS_NUM);
 double opt_general_viewport2(OPT_ARGS_NUM);
 double opt_general_viewport3(OPT_ARGS_NUM);
 double opt_general_menu_position0(OPT_ARGS_NUM);
@@ -211,6 +225,8 @@ double opt_general_statistics_position0(OPT_ARGS_NUM);
 double opt_general_statistics_position1(OPT_ARGS_NUM);
 double opt_general_visibility_position0(OPT_ARGS_NUM);
 double opt_general_visibility_position1(OPT_ARGS_NUM);
+double opt_general_clip_position0(OPT_ARGS_NUM);
+double opt_general_clip_position1(OPT_ARGS_NUM);
 double opt_general_visibility_mode(OPT_ARGS_NUM);
 double opt_general_session_save(OPT_ARGS_NUM);
 double opt_general_options_save(OPT_ARGS_NUM);
@@ -233,12 +249,14 @@ double opt_general_scale2(OPT_ARGS_NUM);
 double opt_general_point_size(OPT_ARGS_NUM);
 double opt_general_line_width(OPT_ARGS_NUM);
 double opt_general_shine(OPT_ARGS_NUM);
+double opt_general_shine_exponent(OPT_ARGS_NUM);
 double opt_general_color_scheme(OPT_ARGS_NUM);
 double opt_general_verbosity(OPT_ARGS_NUM);
 double opt_general_terminal(OPT_ARGS_NUM);
 double opt_general_tooltips(OPT_ARGS_NUM);
 double opt_general_confirm_overwrite(OPT_ARGS_NUM);
 double opt_general_orthographic(OPT_ARGS_NUM);
+double opt_general_draw_bounding_box(OPT_ARGS_NUM);
 double opt_general_fast_redraw(OPT_ARGS_NUM);
 double opt_general_axes(OPT_ARGS_NUM);
 double opt_general_small_axes(OPT_ARGS_NUM);
@@ -247,6 +265,7 @@ double opt_general_small_axes_position1(OPT_ARGS_NUM);
 double opt_general_quadric_subdivisions(OPT_ARGS_NUM);
 double opt_general_double_buffer(OPT_ARGS_NUM);
 double opt_general_alpha_blending(OPT_ARGS_NUM);
+double opt_general_fake_transparency(OPT_ARGS_NUM);
 double opt_general_vector_type(OPT_ARGS_NUM);
 double opt_general_arrow_head_radius(OPT_ARGS_NUM);
 double opt_general_arrow_stem_length(OPT_ARGS_NUM);
@@ -254,7 +273,6 @@ double opt_general_arrow_stem_radius(OPT_ARGS_NUM);
 double opt_general_trackball(OPT_ARGS_NUM);
 double opt_general_rotation_center_cg(OPT_ARGS_NUM);
 double opt_general_zoom_factor(OPT_ARGS_NUM);
-double opt_general_default_plugins(OPT_ARGS_NUM);
 double opt_general_clip0(OPT_ARGS_NUM);
 double opt_general_clip0a(OPT_ARGS_NUM);
 double opt_general_clip0b(OPT_ARGS_NUM);
@@ -289,26 +307,32 @@ double opt_general_light0(OPT_ARGS_NUM);
 double opt_general_light00(OPT_ARGS_NUM);
 double opt_general_light01(OPT_ARGS_NUM);
 double opt_general_light02(OPT_ARGS_NUM);
+double opt_general_light03(OPT_ARGS_NUM);
 double opt_general_light1(OPT_ARGS_NUM);
 double opt_general_light10(OPT_ARGS_NUM);
 double opt_general_light11(OPT_ARGS_NUM);
 double opt_general_light12(OPT_ARGS_NUM);
+double opt_general_light13(OPT_ARGS_NUM);
 double opt_general_light2(OPT_ARGS_NUM);
 double opt_general_light20(OPT_ARGS_NUM);
 double opt_general_light21(OPT_ARGS_NUM);
 double opt_general_light22(OPT_ARGS_NUM);
+double opt_general_light23(OPT_ARGS_NUM);
 double opt_general_light3(OPT_ARGS_NUM);
 double opt_general_light30(OPT_ARGS_NUM);
 double opt_general_light31(OPT_ARGS_NUM);
 double opt_general_light32(OPT_ARGS_NUM);
+double opt_general_light33(OPT_ARGS_NUM);
 double opt_general_light4(OPT_ARGS_NUM);
 double opt_general_light40(OPT_ARGS_NUM);
 double opt_general_light41(OPT_ARGS_NUM);
 double opt_general_light42(OPT_ARGS_NUM);
+double opt_general_light43(OPT_ARGS_NUM);
 double opt_general_light5(OPT_ARGS_NUM);
 double opt_general_light50(OPT_ARGS_NUM);
 double opt_general_light51(OPT_ARGS_NUM);
 double opt_general_light52(OPT_ARGS_NUM);
+double opt_general_light53(OPT_ARGS_NUM);
 double opt_geometry_auto_coherence(OPT_ARGS_NUM);
 double opt_geometry_normals(OPT_ARGS_NUM);
 double opt_geometry_tangents(OPT_ARGS_NUM);
@@ -320,6 +344,7 @@ double opt_geometry_points_num(OPT_ARGS_NUM);
 double opt_geometry_lines_num(OPT_ARGS_NUM);
 double opt_geometry_surfaces_num(OPT_ARGS_NUM);
 double opt_geometry_volumes_num(OPT_ARGS_NUM);
+double opt_mesh_points_per_element(OPT_ARGS_NUM);
 double opt_geometry_point_size(OPT_ARGS_NUM);
 double opt_geometry_point_sel_size(OPT_ARGS_NUM);
 double opt_geometry_point_type(OPT_ARGS_NUM);
@@ -327,16 +352,15 @@ double opt_geometry_line_width(OPT_ARGS_NUM);
 double opt_geometry_line_sel_width(OPT_ARGS_NUM);
 double opt_geometry_line_type(OPT_ARGS_NUM);
 double opt_geometry_light(OPT_ARGS_NUM);
-double opt_geometry_highlight(OPT_ARGS_NUM);
 double opt_geometry_old_circle(OPT_ARGS_NUM);
 double opt_geometry_old_newreg(OPT_ARGS_NUM);
 double opt_geometry_circle_points(OPT_ARGS_NUM);
 double opt_geometry_circle_warning(OPT_ARGS_NUM);
 double opt_geometry_extrude_spline_points(OPT_ARGS_NUM);
 double opt_geometry_scaling_factor(OPT_ARGS_NUM);
-double opt_geometry_color_scheme(OPT_ARGS_NUM);
 double opt_geometry_stl_create_elementary(OPT_ARGS_NUM);
 double opt_geometry_stl_create_physical(OPT_ARGS_NUM);
+double opt_mesh_optimize(OPT_ARGS_NUM);
 double opt_mesh_quality(OPT_ARGS_NUM);
 double opt_mesh_normals(OPT_ARGS_NUM);
 double opt_mesh_tangents(OPT_ARGS_NUM);
@@ -362,11 +386,16 @@ double opt_mesh_point_size(OPT_ARGS_NUM);
 double opt_mesh_point_type(OPT_ARGS_NUM);
 double opt_mesh_line_width(OPT_ARGS_NUM);
 double opt_mesh_line_type(OPT_ARGS_NUM);
+double opt_mesh_vertex_arrays(OPT_ARGS_NUM);
+double opt_mesh_smooth_normals(OPT_ARGS_NUM);
+double opt_mesh_angle_smooth_normals(OPT_ARGS_NUM);
 double opt_mesh_light(OPT_ARGS_NUM);
+double opt_mesh_light_two_side(OPT_ARGS_NUM);
 double opt_mesh_format(OPT_ARGS_NUM);
 double opt_mesh_msh_file_version(OPT_ARGS_NUM);
 double opt_mesh_nb_smoothing(OPT_ARGS_NUM);
-double opt_mesh_algo(OPT_ARGS_NUM);
+double opt_mesh_algo2d(OPT_ARGS_NUM);
+double opt_mesh_algo3d(OPT_ARGS_NUM);
 double opt_mesh_point_insertion(OPT_ARGS_NUM);
 double opt_mesh_speed_max(OPT_ARGS_NUM);
 double opt_mesh_min_circ_points(OPT_ARGS_NUM);
@@ -376,13 +405,13 @@ double opt_mesh_dual(OPT_ARGS_NUM);
 double opt_mesh_interactive(OPT_ARGS_NUM);
 double opt_mesh_use_cut_plane(OPT_ARGS_NUM);
 double opt_mesh_cut_plane_as_surface(OPT_ARGS_NUM);
+double opt_mesh_cut_plane_only_volume(OPT_ARGS_NUM);
 double opt_mesh_cut_planea(OPT_ARGS_NUM);
 double opt_mesh_cut_planeb(OPT_ARGS_NUM);
 double opt_mesh_cut_planec(OPT_ARGS_NUM);
 double opt_mesh_cut_planed(OPT_ARGS_NUM);
 double opt_mesh_allow_degenerated_extrude(OPT_ARGS_NUM);
 double opt_mesh_save_all(OPT_ARGS_NUM);
-double opt_mesh_color_scheme(OPT_ARGS_NUM);
 double opt_mesh_color_carousel(OPT_ARGS_NUM);
 double opt_mesh_nb_nodes(OPT_ARGS_NUM);
 double opt_mesh_nb_triangles(OPT_ARGS_NUM);
@@ -392,7 +421,8 @@ double opt_mesh_nb_hexahedra(OPT_ARGS_NUM);
 double opt_mesh_nb_prisms(OPT_ARGS_NUM);
 double opt_mesh_nb_pyramids(OPT_ARGS_NUM);
 double opt_mesh_cpu_time(OPT_ARGS_NUM);
-double opt_mesh_display_lists(OPT_ARGS_NUM);
+double opt_solver_max_delay(OPT_ARGS_NUM);
+double opt_solver_plugins(OPT_ARGS_NUM);
 double opt_solver_client_server(OPT_ARGS_NUM);
 double opt_solver_client_server0(OPT_ARGS_NUM);
 double opt_solver_client_server1(OPT_ARGS_NUM);
@@ -411,14 +441,17 @@ double opt_solver_merge_views1(OPT_ARGS_NUM);
 double opt_solver_merge_views2(OPT_ARGS_NUM);
 double opt_solver_merge_views3(OPT_ARGS_NUM);
 double opt_solver_merge_views4(OPT_ARGS_NUM);
-double opt_post_display_lists(OPT_ARGS_NUM);
+double opt_post_vertex_arrays(OPT_ARGS_NUM);
 double opt_post_scales(OPT_ARGS_NUM);
+double opt_post_horizontal_scales(OPT_ARGS_NUM);
 double opt_post_link(OPT_ARGS_NUM);
 double opt_post_smooth(OPT_ARGS_NUM);
 double opt_post_anim_delay(OPT_ARGS_NUM);
 double opt_post_anim_cycle(OPT_ARGS_NUM);
+double opt_post_combine_remove_orig(OPT_ARGS_NUM);
+double opt_post_plugins(OPT_ARGS_NUM);
 double opt_post_nb_views(OPT_ARGS_NUM);
-double opt_post_color_scheme(OPT_ARGS_NUM);
+double opt_post_file_format(OPT_ARGS_NUM);
 double opt_view_nb_timestep(OPT_ARGS_NUM);
 double opt_view_timestep(OPT_ARGS_NUM);
 double opt_view_min(OPT_ARGS_NUM);
@@ -432,6 +465,7 @@ double opt_view_raise0(OPT_ARGS_NUM);
 double opt_view_raise1(OPT_ARGS_NUM);
 double opt_view_raise2(OPT_ARGS_NUM);
 double opt_view_arrow_size(OPT_ARGS_NUM);
+double opt_view_arrow_size_proportional(OPT_ARGS_NUM);
 double opt_view_arrow_head_radius(OPT_ARGS_NUM);
 double opt_view_arrow_stem_length(OPT_ARGS_NUM);
 double opt_view_arrow_stem_radius(OPT_ARGS_NUM);
@@ -440,7 +474,10 @@ double opt_view_explode(OPT_ARGS_NUM);
 double opt_view_visible(OPT_ARGS_NUM);
 double opt_view_intervals_type(OPT_ARGS_NUM);
 double opt_view_saturate_values(OPT_ARGS_NUM);
+double opt_view_max_recursion_level(OPT_ARGS_NUM);
+double opt_view_target_error(OPT_ARGS_NUM);
 double opt_view_alpha_channel(OPT_ARGS_NUM);
+double opt_view_external_view(OPT_ARGS_NUM);
 double opt_view_type(OPT_ARGS_NUM);
 double opt_view_grid(OPT_ARGS_NUM);
 double opt_view_position0(OPT_ARGS_NUM);
@@ -452,6 +489,7 @@ double opt_view_nb_iso(OPT_ARGS_NUM);
 double opt_view_nb_abscissa(OPT_ARGS_NUM);
 double opt_view_boundary(OPT_ARGS_NUM);
 double opt_view_light(OPT_ARGS_NUM);
+double opt_view_light_two_side(OPT_ARGS_NUM);
 double opt_view_smooth_normals(OPT_ARGS_NUM);
 double opt_view_angle_smooth_normals(OPT_ARGS_NUM);
 double opt_view_show_element(OPT_ARGS_NUM);
@@ -469,7 +507,6 @@ double opt_view_draw_pyramids(OPT_ARGS_NUM);
 double opt_view_draw_scalars(OPT_ARGS_NUM);
 double opt_view_draw_vectors(OPT_ARGS_NUM);
 double opt_view_draw_tensors(OPT_ARGS_NUM);
-double opt_view_transparent_scale(OPT_ARGS_NUM);
 double opt_view_scale_type(OPT_ARGS_NUM);
 double opt_view_tensor_type(OPT_ARGS_NUM);
 double opt_view_range_type(OPT_ARGS_NUM);
@@ -501,6 +538,9 @@ unsigned int opt_general_color_foreground(OPT_ARGS_COL);
 unsigned int opt_general_color_text(OPT_ARGS_COL);
 unsigned int opt_general_color_axes(OPT_ARGS_COL);
 unsigned int opt_general_color_small_axes(OPT_ARGS_COL);
+unsigned int opt_general_color_ambient_light(OPT_ARGS_COL);
+unsigned int opt_general_color_diffuse_light(OPT_ARGS_COL);
+unsigned int opt_general_color_specular_light(OPT_ARGS_COL);
 unsigned int opt_geometry_color_points(OPT_ARGS_COL); 
 unsigned int opt_geometry_color_lines(OPT_ARGS_COL);
 unsigned int opt_geometry_color_surfaces(OPT_ARGS_COL);
@@ -509,10 +549,6 @@ unsigned int opt_geometry_color_points_select(OPT_ARGS_COL);
 unsigned int opt_geometry_color_lines_select(OPT_ARGS_COL);
 unsigned int opt_geometry_color_surfaces_select(OPT_ARGS_COL);
 unsigned int opt_geometry_color_volumes_select(OPT_ARGS_COL);
-unsigned int opt_geometry_color_points_highlight(OPT_ARGS_COL);
-unsigned int opt_geometry_color_lines_highlight(OPT_ARGS_COL);
-unsigned int opt_geometry_color_surfaces_highlight(OPT_ARGS_COL);
-unsigned int opt_geometry_color_volumes_highlight(OPT_ARGS_COL);
 unsigned int opt_geometry_color_tangents(OPT_ARGS_COL);
 unsigned int opt_geometry_color_normals(OPT_ARGS_COL);
 unsigned int opt_mesh_color_points(OPT_ARGS_COL); 
@@ -572,7 +608,7 @@ typedef struct {
 void Init_Options (int num);
 void Init_Options_GUI (int num);
 void ReInit_Options (int num);
-void Print_Options(int num, int level, char *filename);
+void Print_Options(int num, int level, int diff, char *filename);
 void Print_OptionsDoc();
 
 StringXString * Get_StringOptionCategory(char * cat);
@@ -591,9 +627,12 @@ void * Get_StringOption(char *str, StringXString s[]);
 void * Get_NumberOption(char *str, StringXNumber s[]);
 void * Get_ColorOption(char *str, StringXColor s[]);
 
-void Print_StringOptions(int num, int level, StringXString s[], char *prefix, FILE *file);
-void Print_NumberOptions(int num, int level, StringXNumber s[], char *prefix, FILE *file);
-void Print_ColorOptions(int num, int level, StringXColor s[], char *prefix, FILE *file);
+void Print_StringOptions(int num, int level, int diff, StringXString s[], 
+			 char *prefix, FILE *file);
+void Print_NumberOptions(int num, int level, int diff, StringXNumber s[], 
+			 char *prefix, FILE *file);
+void Print_ColorOptions(int num, int level, int diff, StringXColor s[], 
+			char *prefix, FILE *file);
 
 void Print_StringOptionsDoc(StringXString s[], char *prefix, FILE * file);
 void Print_NumberOptionsDoc(StringXNumber s[], char *prefix, FILE * file);
