@@ -1,4 +1,4 @@
-// $Id: Views.cpp,v 1.115 2004/02/20 17:57:59 geuzaine Exp $
+// $Id: Views.cpp,v 1.144 2004/10/30 15:23:44 geuzaine Exp $
 //
 // Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
 //
@@ -26,6 +26,7 @@
 #include "Context.h"
 #include "Options.h"
 #include "ColorTable.h"
+#include "SmoothNormals.h"
 
 extern Context_T CTX;
 
@@ -33,8 +34,6 @@ extern Context_T CTX;
 void UpdateViewsInGUI();
 #endif
 
-#define VAL_INF 1.e200
-
 // Static reference view
 
 Post_View *Post_ViewReference = NULL;
@@ -51,6 +50,16 @@ int fcmpPostViewDuplicateOf(const void *v1, const void *v2)
   return (((Post_View *) v1)->DuplicateOf - ((Post_View *) v2)->DuplicateOf);
 }
 
+int fcmpPostViewName(const void *v1, const void *v2)
+{
+  return strcmp(((Post_View *) v1)->Name, ((Post_View *) v2)->Name);
+}
+
+int fcmpPostViewVisibility(const void *v1, const void *v2)
+{
+  return (((Post_View *) v2)->Visible - ((Post_View *) v1)->Visible);
+}
+
 Post_View *BeginView(int allocate)
 {
   Post_View vv, *v;
@@ -60,8 +69,12 @@ Post_View *BeginView(int allocate)
   if(!CTX.post.list)
     CTX.post.list = List_Create(100, 100, sizeof(Post_View));
 
+  // Important notes:
+  // - each view *must* have a unique number
+  // - the view list is assumned to be sorted with increasing nums
+
   if(!CTX.post.force_num) {
-    vv.Num = ++UniqueNum;       // each view _must_ have a unique number
+    vv.Num = ++UniqueNum;
     List_Add(CTX.post.list, &vv);
   }
   else {
@@ -74,7 +87,7 @@ Post_View *BeginView(int allocate)
 
   v->Index = i;
   v->Dirty = 1;
-  v->DisplayListNum = -1;
+  v->TriVertexArray = NULL;
   v->NbTimeStep = 0;
   v->TimeStepMin = NULL;
   v->TimeStepMax = NULL;
@@ -146,15 +159,16 @@ Post_View *BeginView(int allocate)
 
   // Copy all options from the reference view initialized in InitOptions()
   CopyViewOptions(Post_ViewReference, v);
-
+  
   v->Changed = 1;
   v->Links = 0;
   v->DuplicateOf = 0;
   v->ScalarOnly = 1;
   v->TextOnly = 1;
-  v->normals = NULL;
+  v->normals = new smooth_normals;
   v->Min = VAL_INF;
   v->Max = -VAL_INF;
+  v->adaptive = 0;
   for(i = 0; i < 3; i++) {
     v->BBox[2 * i] = VAL_INF;
     v->BBox[2 * i + 1] = -VAL_INF;
@@ -191,14 +205,12 @@ void Stat_Element(Post_View * v, int type, int nbnod, int N,
 
   case 0:      // scalar
     if(v->Min == VAL_INF || v->Max == -VAL_INF) {
-      v->Min = V[0];
-      v->Max = V[0];
       v->NbTimeStep = N / nbnod;
       v->TimeStepMin = (double*)Malloc(v->NbTimeStep * sizeof(double));
       v->TimeStepMax = (double*)Malloc(v->NbTimeStep * sizeof(double));
       for(i = 0; i < v->NbTimeStep; i++){
-	v->TimeStepMin[i] = V[0];
-	v->TimeStepMax[i] = V[0];
+	v->TimeStepMin[i] = VAL_INF;
+	v->TimeStepMax[i] = -VAL_INF;
       }
     }
     else if(N / nbnod < v->NbTimeStep){
@@ -224,15 +236,12 @@ void Stat_Element(Post_View * v, int type, int nbnod, int N,
 
   case 1:      // vector
     if(v->Min == VAL_INF || v->Max == -VAL_INF) {
-      l0 = sqrt(DSQR(V[0]) + DSQR(V[1]) + DSQR(V[2]));
-      v->Min = l0;
-      v->Max = l0;
       v->NbTimeStep = N / (3 * nbnod);
       v->TimeStepMin = (double*)Malloc(v->NbTimeStep * sizeof(double));
       v->TimeStepMax = (double*)Malloc(v->NbTimeStep * sizeof(double));
       for(i = 0; i < v->NbTimeStep; i++){
-	v->TimeStepMin[i] = l0;
-	v->TimeStepMax[i] = l0;
+	v->TimeStepMin[i] = VAL_INF;
+	v->TimeStepMax[i] = -VAL_INF;
       }
     }
     else if(N / (3 * nbnod) < v->NbTimeStep){
@@ -257,19 +266,13 @@ void Stat_Element(Post_View * v, int type, int nbnod, int N,
     break;
 
   case 2:      // tensor
-    // by lack of any current better solution,
-    // tensors are displayed as their Von Mises
-    // invariant (J2 invariant)
     if(v->Min == VAL_INF || v->Max == -VAL_INF) {
-      l0 = ComputeVonMises(V);
-      v->Min = l0;
-      v->Max = l0;
       v->NbTimeStep = N / (9 * nbnod);
       v->TimeStepMin = (double*)Malloc(v->NbTimeStep * sizeof(double));
       v->TimeStepMax = (double*)Malloc(v->NbTimeStep * sizeof(double));
       for(i = 0; i < v->NbTimeStep; i++){
-	v->TimeStepMin[i] = l0;
-	v->TimeStepMax[i] = l0;
+	v->TimeStepMin[i] = VAL_INF;
+	v->TimeStepMax[i] = -VAL_INF;
       }
     }
     else if(N / (9 * nbnod) < v->NbTimeStep){
@@ -277,6 +280,8 @@ void Stat_Element(Post_View * v, int type, int nbnod, int N,
     }
 
     for(i = 0; i < N; i += 9) {
+      // by lack of any current better solution, tensors are displayed
+      // as their Von Mises invariant (J2 invariant)
       l0 = ComputeVonMises(V+i);
       if(l0 < v->Min)
         v->Min = l0;
@@ -296,18 +301,12 @@ void Stat_Element(Post_View * v, int type, int nbnod, int N,
   }
 
   for(i = 0; i < nbnod; i++) {
-    if(X[i] < v->BBox[0])
-      v->BBox[0] = X[i];
-    if(X[i] > v->BBox[1])
-      v->BBox[1] = X[i];
-    if(Y[i] < v->BBox[2])
-      v->BBox[2] = Y[i];
-    if(Y[i] > v->BBox[3])
-      v->BBox[3] = Y[i];
-    if(Z[i] < v->BBox[4])
-      v->BBox[4] = Z[i];
-    if(Z[i] > v->BBox[5])
-      v->BBox[5] = Z[i];
+    if(X[i] < v->BBox[0]) v->BBox[0] = X[i];
+    if(X[i] > v->BBox[1]) v->BBox[1] = X[i];
+    if(Y[i] < v->BBox[2]) v->BBox[2] = Y[i];
+    if(Y[i] > v->BBox[3]) v->BBox[3] = Y[i];
+    if(Z[i] < v->BBox[4]) v->BBox[4] = Z[i];
+    if(Z[i] > v->BBox[5]) v->BBox[5] = Z[i];
   }
 
   v->TextOnly = 0;
@@ -516,9 +515,6 @@ bool RemoveViewByIndex(int index)
     v->Index = i;
   }
 
-#if defined(HAVE_FLTK)
-  UpdateViewsInGUI();
-#endif
   Msg(DEBUG, "Removed View[%d] (%d views left)", index, List_Nbr(CTX.post.list));
   return true;
 }
@@ -595,9 +591,13 @@ void FreeView(Post_View * v)
     v->SY = v->VY = v->TY = NULL;
     v->T2D = v->T2C = NULL;
     v->T3D = v->T3C = NULL;
-    v->reset_normals();
+    if(v->normals) delete v->normals;
+    v->normals = NULL;
+    if(v->TriVertexArray) delete v->TriVertexArray;
+    v->TriVertexArray = NULL;
+    if(v->adaptive) delete v->adaptive;
+    v->adaptive = 0;
   }
-
 }
 
 void CopyViewOptions(Post_View * src, Post_View * dest)
@@ -620,6 +620,7 @@ void CopyViewOptions(Post_View * src, Post_View * dest)
   dest->Raise[1] = src->Raise[1];
   dest->Raise[2] = src->Raise[2];
   dest->ArrowSize = src->ArrowSize;
+  dest->ArrowSizeProportional = src->ArrowSizeProportional;
   dest->ArrowRelHeadRadius = src->ArrowRelHeadRadius;
   dest->ArrowRelStemLength = src->ArrowRelStemLength;
   dest->ArrowRelStemRadius = src->ArrowRelStemRadius;
@@ -633,6 +634,7 @@ void CopyViewOptions(Post_View * src, Post_View * dest)
   dest->NbAbscissa = src->NbAbscissa;
   dest->NbIso = src->NbIso;
   dest->Light = src->Light;
+  dest->LightTwoSide = src->LightTwoSide;
   dest->SmoothNormals = src->SmoothNormals;
   dest->AngleSmoothNormals = src->AngleSmoothNormals;
   dest->ShowElement = src->ShowElement;
@@ -650,7 +652,6 @@ void CopyViewOptions(Post_View * src, Post_View * dest)
   dest->DrawVectors = src->DrawVectors;
   dest->DrawTensors = src->DrawTensors;
   dest->DrawStrings = src->DrawStrings;
-  dest->TransparentScale = src->TransparentScale;
   dest->ScaleType = src->ScaleType;
   dest->RangeType = src->RangeType;
   dest->VectorType = src->VectorType;
@@ -662,6 +663,7 @@ void CopyViewOptions(Post_View * src, Post_View * dest)
   dest->PointType = src->PointType;
   dest->LineType = src->LineType;
   dest->Grid = src->Grid;
+  dest->ExternalViewIndex = src->ExternalViewIndex;
   ColorTable_Copy(&src->CT);
   ColorTable_Paste(&dest->CT);
 }
@@ -680,7 +682,7 @@ GmshColorTable *Get_ColorTable(int num)
     return NULL;
 }
 
-void Print_ColorTable(int num, char *prefix, FILE * file)
+void Print_ColorTable(int num, int diff, char *prefix, FILE * file)
 {
   char tmp[1024];
   Post_View *v;
@@ -690,6 +692,10 @@ void Print_ColorTable(int num, char *prefix, FILE * file)
     v = (Post_View *) List_Pointer_Test(CTX.post.list, num);
   if(!v)
     return;
+
+  if(diff && !ColorTable_Diff(&Post_ViewReference->CT, &v->CT))
+    return;
+
   sprintf(tmp, "%s = {", prefix);
   if(file)
     fprintf(file, "%s\n", tmp);
@@ -834,9 +840,11 @@ void ReadView(FILE *file, char *filename)
       v->DataSize = size;
 
       // Time values
-      v->Time = List_CreateFromFile(v->NbTimeStep, size, file, format, swap);
+      v->Time = List_CreateFromFile(v->NbTimeStep, 10, size, file, format, swap);
 
-#define LL List_CreateFromFile(nb, size, file, format, swap)
+      // if nb==0, this still allocates a list (empty, but ready to be
+      // filled later, e.g. in a plugin)
+#define LL List_CreateFromFile(nb, 100, size, file, format, swap)
 
       // Points
       nb = v->NbSP ? v->NbSP * (v->NbTimeStep + 3) : 0;
@@ -906,12 +914,12 @@ void ReadView(FILE *file, char *filename)
 
       // Strings
       nb = v->NbT2 ? v->NbT2 * 4 : 0;
-      v->T2D = List_CreateFromFile(nb, size, file, format, swap);
-      v->T2C = List_CreateFromFile(t2l, sizeof(char), file, format, swap);
+      v->T2D = List_CreateFromFile(nb, 10, size, file, format, swap);
+      v->T2C = List_CreateFromFile(t2l, 10, sizeof(char), file, format, swap);
 
       nb = v->NbT3 ? v->NbT3 * 5 : 0;
-      v->T3D = List_CreateFromFile(nb, size, file, format, swap);
-      v->T3C = List_CreateFromFile(t3l, sizeof(char), file, format, swap);
+      v->T3D = List_CreateFromFile(nb, 10, size, file, format, swap);
+      v->T3C = List_CreateFromFile(t3l, 10, sizeof(char), file, format, swap);
 
 
       Msg(DEBUG,
@@ -949,15 +957,66 @@ void ReadView(FILE *file, char *filename)
   Msg(STATUS2N, "Read '%s'", filename);
 }
 
-// FIXME: add an integer per simplex (region num)
-// FIXME: add a format similar to the msh format (node list + simplex list)
-// FIXME: add a structured format
+static void write_parsed_elements(char *str, int nbnod, int nb, List_T *list, FILE *fp)
+{
+  if(nb) {
+    int n = List_Nbr(list) / nb;
+    for(int i = 0; i < List_Nbr(list); i += n) {
+      double *x = (double *)List_Pointer(list, i);
+      double *y = (double *)List_Pointer(list, i + nbnod);
+      double *z = (double *)List_Pointer(list, i + 2 * nbnod);
+      fprintf(fp, "%s(", str);
+      for(int j = 0; j < nbnod; j++) {
+	if(j) fprintf(fp, ",");
+	fprintf(fp, "%.16g,%.16g,%.16g", x[j], y[j], z[j]);
+      }
+      fprintf(fp, "){", str);
+      for(int j = 3 * nbnod; j < n; j++) {
+	if(j - 3 * nbnod) fprintf(fp, ",");
+	fprintf(fp, "%.16g", *(double *)List_Pointer(list, i + j));
+      }
+      fprintf(fp, "};\n", str);
+    }
+  }
+}
 
-void WriteView(Post_View *v, char *filename, int binary, int append)
+static void write_parsed_strings(int nbc, int nb, List_T *TD, List_T *TC, FILE *fp)
+{
+  if(!nb || (nbc != 4 && nbc != 5)) return;
+  for(int j = 0; j < List_Nbr(TD); j += nbc){
+    double x, y, z, style, start, end;
+    List_Read(TD, j, &x);
+    List_Read(TD, j+1, &y);
+    if(nbc == 5)
+      List_Read(TD, j+2, &z);
+    List_Read(TD, j+nbc-2, &style);
+    if(nbc == 4)
+      fprintf(fp, "T2(%g,%g,%g){", x, y, style);
+    else
+      fprintf(fp, "T3(%g,%g,%g,%g){", x, y, z, style);
+    List_Read(TD, j+nbc-1, &start);
+    if(j+nbc*2-1 < List_Nbr(TD))
+      List_Read(TD, j+nbc*2-1, &end);
+    else
+      end = List_Nbr(TC);
+    int l = 0;
+    while(l < end-start){
+      char *str = (char*)List_Pointer(TC, (int)start + l);
+      if(l) fprintf(fp, ",");
+      fprintf(fp, "\"%s\"", str);
+      l += strlen(str)+1;
+    }
+    fprintf(fp, "};\n");
+  }
+}
+
+void WriteView(Post_View *v, char *filename, int format, int append)
 {
   FILE *file;
   char name[256];
-  int i, f, One = 1;
+  int f, One = 1;
+  int binary = (format == 1) ? 1 : 0;
+  int parsed = (format == 2);
 
   if(filename) {
     file = fopen(filename, append ? (binary ? "ab" : "a") : (binary ? "wb" : "w"));
@@ -971,13 +1030,14 @@ void WriteView(Post_View *v, char *filename, int binary, int append)
   else
     file = stdout;
 
-  if(!append){
+  if(!parsed && !append){
     fprintf(file, "$PostFormat /* Gmsh 1.2, %s */\n",
 	    binary ? "binary" : "ascii");
     fprintf(file, "1.2 %d %d\n", binary, (int)sizeof(double));
     fprintf(file, "$EndPostFormat\n");
   }
 
+  int i;
   for(i = 0; i < (int)strlen(v->Name); i++) {
     if(v->Name[i] == ' ')
       name[i] = '^';
@@ -986,53 +1046,85 @@ void WriteView(Post_View *v, char *filename, int binary, int append)
   }
   name[i] = '\0';
 
-  fprintf(file, "$View /* %s */\n", v->Name);
-  fprintf(file, "%s ", name);
-  fprintf(file, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d "
-          "%d %d %d %d %d %d %d %d %d %d %d %d\n",
-          List_Nbr(v->Time),
-          v->NbSP, v->NbVP, v->NbTP, v->NbSL, v->NbVL, v->NbTL,
-          v->NbST, v->NbVT, v->NbTT, v->NbSQ, v->NbVQ, v->NbTQ,
-          v->NbSS, v->NbVS, v->NbTS, v->NbSH, v->NbVH, v->NbTH,
-          v->NbSI, v->NbVI, v->NbTI, v->NbSY, v->NbVY, v->NbTY,
-          v->NbT2, List_Nbr(v->T2C), v->NbT3, List_Nbr(v->T3C));
-  if(binary) {
-    f = LIST_FORMAT_BINARY;
-    fwrite(&One, sizeof(int), 1, file);
+  if(!parsed){
+    fprintf(file, "$View /* %s */\n", v->Name);
+    fprintf(file, "%s ", name);
+    fprintf(file, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d "
+	    "%d %d %d %d %d %d %d %d %d %d %d %d\n",
+	    List_Nbr(v->Time),
+	    v->NbSP, v->NbVP, v->NbTP, v->NbSL, v->NbVL, v->NbTL,
+	    v->NbST, v->NbVT, v->NbTT, v->NbSQ, v->NbVQ, v->NbTQ,
+	    v->NbSS, v->NbVS, v->NbTS, v->NbSH, v->NbVH, v->NbTH,
+	    v->NbSI, v->NbVI, v->NbTI, v->NbSY, v->NbVY, v->NbTY,
+	    v->NbT2, List_Nbr(v->T2C), v->NbT3, List_Nbr(v->T3C));
+    if(binary) {
+      f = LIST_FORMAT_BINARY;
+      fwrite(&One, sizeof(int), 1, file);
+    }
+    else
+      f = LIST_FORMAT_ASCII;
+    List_WriteToFile(v->Time, file, f);
+    List_WriteToFile(v->SP, file, f);
+    List_WriteToFile(v->VP, file, f);
+    List_WriteToFile(v->TP, file, f);
+    List_WriteToFile(v->SL, file, f);
+    List_WriteToFile(v->VL, file, f);
+    List_WriteToFile(v->TL, file, f);
+    List_WriteToFile(v->ST, file, f);
+    List_WriteToFile(v->VT, file, f);
+    List_WriteToFile(v->TT, file, f);
+    List_WriteToFile(v->SQ, file, f);
+    List_WriteToFile(v->VQ, file, f);
+    List_WriteToFile(v->TQ, file, f);
+    List_WriteToFile(v->SS, file, f);
+    List_WriteToFile(v->VS, file, f);
+    List_WriteToFile(v->TS, file, f);
+    List_WriteToFile(v->SH, file, f);
+    List_WriteToFile(v->VH, file, f);
+    List_WriteToFile(v->TH, file, f);
+    List_WriteToFile(v->SI, file, f);
+    List_WriteToFile(v->VI, file, f);
+    List_WriteToFile(v->TI, file, f);
+    List_WriteToFile(v->SY, file, f);
+    List_WriteToFile(v->VY, file, f);
+    List_WriteToFile(v->TY, file, f);
+    List_WriteToFile(v->T2D, file, f);
+    List_WriteToFile(v->T2C, file, f);
+    List_WriteToFile(v->T3D, file, f);
+    List_WriteToFile(v->T3C, file, f);
+    fprintf(file, "\n");
+    fprintf(file, "$EndView\n");
+  }
+  else{
+    fprintf(file, "View \"%s\" {\n", v->Name);
+    write_parsed_elements("SP", 1, v->NbSP, v->SP, file);
+    write_parsed_elements("VP", 1, v->NbVP, v->VP, file);
+    write_parsed_elements("TP", 1, v->NbTP, v->TP, file);
+    write_parsed_elements("SL", 2, v->NbSL, v->SL, file);
+    write_parsed_elements("VL", 2, v->NbVL, v->VL, file);
+    write_parsed_elements("TL", 2, v->NbTL, v->TL, file);
+    write_parsed_elements("ST", 3, v->NbST, v->ST, file);
+    write_parsed_elements("VT", 3, v->NbVT, v->VT, file);
+    write_parsed_elements("TT", 3, v->NbTT, v->TT, file);
+    write_parsed_elements("SQ", 4, v->NbSQ, v->SQ, file);
+    write_parsed_elements("VQ", 4, v->NbVQ, v->VQ, file);
+    write_parsed_elements("TQ", 4, v->NbTQ, v->TQ, file);
+    write_parsed_elements("SS", 4, v->NbSS, v->SS, file);
+    write_parsed_elements("VS", 4, v->NbVS, v->VS, file);
+    write_parsed_elements("TS", 4, v->NbTS, v->TS, file);
+    write_parsed_elements("SH", 8, v->NbSH, v->SH, file);
+    write_parsed_elements("VH", 8, v->NbVH, v->VH, file);
+    write_parsed_elements("TH", 8, v->NbTH, v->TH, file);
+    write_parsed_elements("SI", 6, v->NbSI, v->SI, file);
+    write_parsed_elements("VI", 6, v->NbVI, v->VI, file);
+    write_parsed_elements("TI", 6, v->NbTI, v->TI, file);
+    write_parsed_elements("SY", 5, v->NbSY, v->SY, file);
+    write_parsed_elements("VY", 5, v->NbVY, v->VY, file);
+    write_parsed_elements("TY", 5, v->NbTY, v->TY, file);
+    write_parsed_strings(4, v->NbT2, v->T2D, v->T2C, file);
+    write_parsed_strings(5, v->NbT3, v->T3D, v->T3C, file);
+    fprintf(file, "};\n");
   }
-  else
-    f = LIST_FORMAT_ASCII;
-  List_WriteToFile(v->Time, file, f);
-  List_WriteToFile(v->SP, file, f);
-  List_WriteToFile(v->VP, file, f);
-  List_WriteToFile(v->TP, file, f);
-  List_WriteToFile(v->SL, file, f);
-  List_WriteToFile(v->VL, file, f);
-  List_WriteToFile(v->TL, file, f);
-  List_WriteToFile(v->ST, file, f);
-  List_WriteToFile(v->VT, file, f);
-  List_WriteToFile(v->TT, file, f);
-  List_WriteToFile(v->SQ, file, f);
-  List_WriteToFile(v->VQ, file, f);
-  List_WriteToFile(v->TQ, file, f);
-  List_WriteToFile(v->SS, file, f);
-  List_WriteToFile(v->VS, file, f);
-  List_WriteToFile(v->TS, file, f);
-  List_WriteToFile(v->SH, file, f);
-  List_WriteToFile(v->VH, file, f);
-  List_WriteToFile(v->TH, file, f);
-  List_WriteToFile(v->SI, file, f);
-  List_WriteToFile(v->VI, file, f);
-  List_WriteToFile(v->TI, file, f);
-  List_WriteToFile(v->SY, file, f);
-  List_WriteToFile(v->VY, file, f);
-  List_WriteToFile(v->TY, file, f);
-  List_WriteToFile(v->T2D, file, f);
-  List_WriteToFile(v->T2C, file, f);
-  List_WriteToFile(v->T3D, file, f);
-  List_WriteToFile(v->T3C, file, f);
-  fprintf(file, "\n");
-  fprintf(file, "$EndView\n");
 
   if(filename) {
     fclose(file);
@@ -1044,118 +1136,15 @@ void WriteView(Post_View *v, char *filename, int binary, int append)
 
 // Smoothing
 
-using namespace std;
-
-struct xyzv
-{
-private:
-public:
-  double x, y, z, *vals;
-  int nbvals;
-  int nboccurences;
-  static double eps;
-  void update(int nbVals, double *);
-    xyzv(double x, double y, double z);
-   ~xyzv();
-    xyzv & operator =(const xyzv &);
-    xyzv(const xyzv &);
-};
-
-double xyzv::eps = 0.0;
-
-xyzv::xyzv(double xx, double yy, double zz)
-:x(xx), y(yy), z(zz), vals(0), nbvals(0), nboccurences(0)
+void Post_View::reset_normals()
 {
+  if(normals)
+    delete normals;
+  normals = new smooth_normals;
 }
 
-xyzv::~xyzv()
-{
-  if(vals)
-    delete[]vals;
-}
-
-xyzv::xyzv(const xyzv & other)
-{
-  x = other.x;
-  y = other.y;
-  z = other.z;
-  nbvals = other.nbvals;
-  nboccurences = other.nboccurences;
-  if(other.vals && other.nbvals) {
-    vals = new double[other.nbvals];
-    for(int i = 0; i < nbvals; i++)
-      vals[i] = other.vals[i];
-  }
-}
-
-xyzv & xyzv::operator =(const xyzv & other)
-{
-  if(this != &other) {
-    x = other.x;
-    y = other.y;
-    z = other.z;
-    nbvals = other.nbvals;
-    nboccurences = other.nboccurences;
-    if(other.vals && other.nbvals) {
-      vals = new double[other.nbvals];
-      for(int i = 0; i < nbvals; i++)
-        vals[i] = other.vals[i];
-    }
-  }
-  return *this;
-}
-
-void xyzv::update(int n, double *v)
-{
-  int i;
-  if(!vals) {
-    vals = new double[n];
-    for(i = 0; i < n; i++)
-      vals[i] = 0.0;
-    nbvals = n;
-    nboccurences = 0;
-  }
-  else if(nbvals != n) {
-    throw n;
-  }
-
-  double x1 = (double)(nboccurences) / (double)(nboccurences + 1);
-  double x2 = 1. / (double)(nboccurences + 1);
-  for(i = 0; i < nbvals; i++)
-    vals[i] = (x1 * vals[i] + x2 * v[i]);
-  nboccurences++;
-
-  //printf("val(%d,%f,%f,%f) = %f\n",nboccurences,x,y,z,vals[0]);
-}
-
-struct lessthanxyzv
-{
-  bool operator () (const xyzv & p2, const xyzv & p1)const
-  {
-    if(p1.x - p2.x > xyzv::eps)
-      return true;
-    if(p1.x - p2.x < -xyzv::eps)
-      return false;
-    if(p1.y - p2.y > xyzv::eps)
-      return true;
-    if(p1.y - p2.y < -xyzv::eps)
-      return false;
-    if(p1.z - p2.z > xyzv::eps)
-      return true;
-    return false;
-  }
-};
-
-typedef set < xyzv, lessthanxyzv > mycont;
-typedef mycont::const_iterator iter;
-
-class smooth_container
-{
-  public: mycont c;
-};
-
 void generate_connectivities(List_T * list, int nbList, int nbTimeStep, int nbVert,
-                             mycont & connectivities)
+                             xyzcont & connectivities)
 {
   double *x, *y, *z, *v;
   int i, j, k;
@@ -1174,7 +1163,7 @@ void generate_connectivities(List_T * list, int nbList, int nbTimeStep, int nbVe
       for(k = 0; k < nbTimeStep; k++)
         vals[k] = v[j + k * nbVert];
       xyzv xyz(x[j], y[j], z[j]);
-      iter it = connectivities.find(xyz);
+      xyziter it = connectivities.find(xyz);
       if(it == connectivities.end()) {
         xyz.update(nbTimeStep, vals);
         connectivities.insert(xyz);
@@ -1191,7 +1180,7 @@ void generate_connectivities(List_T * list, int nbList, int nbTimeStep, int nbVe
 }
 
 void smooth_list(List_T * list, int nbList, double *min, double *max,
-                 int nbTimeStep, int nbVert, mycont & connectivities)
+                 int nbTimeStep, int nbVert, xyzcont & connectivities)
 {
   double *x, *y, *z, *v;
   int i, j, k;
@@ -1210,7 +1199,7 @@ void smooth_list(List_T * list, int nbList, double *min, double *max,
     v = (double *)List_Pointer_Fast(list, i + 3 * nbVert);
     for(j = 0; j < nbVert; j++) {
       xyzv xyz(x[j], y[j], z[j]);
-      iter it = connectivities.find(xyz);
+      xyziter it = connectivities.find(xyz);
       if(it != connectivities.end()) {
         for(k = 0; k < nbTimeStep; k++) {
           v[j + k * nbVert] = (*it).vals[k];
@@ -1226,10 +1215,11 @@ void smooth_list(List_T * list, int nbList, double *min, double *max,
 
 void Post_View::smooth()
 {
+  double old_eps = xyzv::eps;
   xyzv::eps = CTX.lc * 1.e-8;
 
   if(NbSL || NbST || NbSQ || NbSS || NbSH || NbSI || NbSY) {
-    mycont con;
+    xyzcont con;
     Msg(INFO, "Smoothing scalar primitives in View[%d]", Index);
     generate_connectivities(SL, NbSL, NbTimeStep, 2, con);
     generate_connectivities(ST, NbST, NbTimeStep, 3, con);
@@ -1247,81 +1237,8 @@ void Post_View::smooth()
     smooth_list(SY, NbSY, &Min, &Max, NbTimeStep, 5, con);
     Changed = 1;
   }
-}
 
-// Normal smoothing
-
-void Post_View::reset_normals()
-{
-  if(normals)
-    delete normals;
-  normals = 0;
-}
-
-void Post_View::add_normal(double x, double y, double z,
-                           double nx, double ny, double nz)
-{
-  if(!normals)
-    normals = new smooth_container;
-
-  double n[3] = { nx, ny, nz };
-  xyzv xyz(x, y, z);
-
-  iter it = normals->c.find(xyz);
-
-  if(it == normals->c.end()) {
-    xyz.update(3, n);
-    normals->c.insert(xyz);
-  }
-  else {
-    xyzv *xx = (xyzv *) & (*it);
-    xx->update(3, n);
-  }
-}
-
-double get_angle(double *aa, double *bb)
-{
-  double angplan, cosc, sinc, a[3], b[3], c[3];
-  if(!aa || !bb)
-    return 0.;
-  a[0] = aa[0];
-  a[1] = aa[1];
-  a[2] = aa[2];
-  b[0] = bb[0];
-  b[1] = bb[1];
-  b[2] = bb[2];
-  norme(a);
-  norme(b);
-  prodve(a, b, c);
-  prosca(a, b, &cosc);
-  sinc = sqrt(c[0] * c[0] + c[1] * c[1] + c[2] * c[2]);
-  angplan = myatan2(sinc, cosc);
-  return angplan * 180. / Pi;
-}
-
-bool Post_View::get_normal(double x, double y, double z,
-                           double &nx, double &ny, double &nz)
-{
-  if(!normals)
-    return false;
-
-  double n[3] = { nx, ny, nz };
-  xyzv xyz(x, y, z);
-
-  iter it = normals->c.find(xyz);
-
-  if(it == normals->c.end())
-    return false;
-
-  double angle = get_angle((*it).vals, n);
-
-  if(fabs(angle) < AngleSmoothNormals) {
-    nx = (*it).vals[0];
-    ny = (*it).vals[1];
-    nz = (*it).vals[2];
-  }
-
-  return true;
+  xyzv::eps = old_eps;  
 }
 
 // Transformation
@@ -1334,7 +1251,7 @@ static void transform(double mat[3][3], double v[3],
   *z = mat[2][0] * v[0] + mat[2][1] * v[1] + mat[2][2] * v[2];
 }
 
-static void transform_list(List_T *list, int nbList, 
+static void transform_list(Post_View *view, List_T *list, int nbList, 
 			   int nbVert, double mat[3][3])
 {
   double *x, *y, *z, v[3];
@@ -1352,43 +1269,65 @@ static void transform_list(List_T *list, int nbList,
       v[1] = y[j];
       v[2] = z[j];
       transform(mat, v, &x[j], &y[j], &z[j]);
+      if(x[j] < view->BBox[0]) view->BBox[0] = x[j];
+      if(x[j] > view->BBox[1]) view->BBox[1] = x[j];
+      if(y[j] < view->BBox[2]) view->BBox[2] = y[j];
+      if(y[j] > view->BBox[3]) view->BBox[3] = y[j];
+      if(z[j] < view->BBox[4]) view->BBox[4] = z[j];
+      if(z[j] > view->BBox[5]) view->BBox[5] = z[j];
     }
   }
 }
 
 void Post_View::transform(double mat[3][3])
 {
-  transform_list(SP, NbSP, 1, mat);
-  transform_list(SL, NbSL, 2, mat);
-  transform_list(ST, NbST, 3, mat);
-  transform_list(SQ, NbSQ, 4, mat);
-  transform_list(SS, NbSS, 4, mat);
-  transform_list(SH, NbSH, 8, mat);
-  transform_list(SI, NbSI, 6, mat);
-  transform_list(SY, NbSY, 5, mat);
+  for(int i = 0; i < 3; i++) {
+    BBox[2 * i] = VAL_INF;
+    BBox[2 * i + 1] = -VAL_INF;
+  }
 
-  transform_list(VP, NbVP, 1, mat);
-  transform_list(VL, NbVL, 2, mat);
-  transform_list(VT, NbVT, 3, mat);
-  transform_list(VQ, NbVQ, 4, mat);
-  transform_list(VS, NbVS, 4, mat);
-  transform_list(VH, NbVH, 8, mat);
-  transform_list(VI, NbVI, 6, mat);
-  transform_list(VY, NbVY, 5, mat);
+  transform_list(this, SP, NbSP, 1, mat);
+  transform_list(this, SL, NbSL, 2, mat);
+  transform_list(this, ST, NbST, 3, mat);
+  transform_list(this, SQ, NbSQ, 4, mat);
+  transform_list(this, SS, NbSS, 4, mat);
+  transform_list(this, SH, NbSH, 8, mat);
+  transform_list(this, SI, NbSI, 6, mat);
+  transform_list(this, SY, NbSY, 5, mat);
 
-  transform_list(TP, NbTP, 1, mat);
-  transform_list(TL, NbTL, 2, mat);
-  transform_list(TT, NbTT, 3, mat);
-  transform_list(TQ, NbTQ, 4, mat);
-  transform_list(TS, NbTS, 4, mat);
-  transform_list(TH, NbTH, 8, mat);
-  transform_list(TI, NbTI, 6, mat);
-  transform_list(TY, NbTY, 5, mat);
+  transform_list(this, VP, NbVP, 1, mat);
+  transform_list(this, VL, NbVL, 2, mat);
+  transform_list(this, VT, NbVT, 3, mat);
+  transform_list(this, VQ, NbVQ, 4, mat);
+  transform_list(this, VS, NbVS, 4, mat);
+  transform_list(this, VH, NbVH, 8, mat);
+  transform_list(this, VI, NbVI, 6, mat);
+  transform_list(this, VY, NbVY, 5, mat);
+
+  transform_list(this, TP, NbTP, 1, mat);
+  transform_list(this, TL, NbTL, 2, mat);
+  transform_list(this, TT, NbTT, 3, mat);
+  transform_list(this, TQ, NbTQ, 4, mat);
+  transform_list(this, TS, NbTS, 4, mat);
+  transform_list(this, TH, NbTH, 8, mat);
+  transform_list(this, TI, NbTI, 6, mat);
+  transform_list(this, TY, NbTY, 5, mat);
 
   Changed = 1;
 }
 
-// Combine views (spatially)
+// Combine views (merge elements or merge time steps)
+
+struct nameidx{
+  char name[256];
+  List_T *indices;
+};
+
+static int fcmp_name(const void *a, const void *b){
+  char *name1 = ((struct nameidx*)a)->name;
+  char *name2 = ((struct nameidx*)b)->name;
+  return strcmp(name1, name2);
+}
 
 static void combine(List_T * a, List_T * b)
 {
@@ -1399,96 +1338,77 @@ static void combine(List_T * a, List_T * b)
   }
 }
 
-void CombineViews(int all, int remove)
+static void combine_space(struct nameidx *id, List_T *to_remove)
 {
+  int index;
+
   // sanity check
-  int first = 1, nbt = 0;
-  for(int i = 0; i < List_Nbr(CTX.post.list); i++) {
-    Post_View *v = (Post_View *) List_Pointer(CTX.post.list, i);
-    if(all || v->Visible) {
-      if(first){
-	nbt = v->NbTimeStep;
-	first = 0;
-      }
-      else{
-	if(v->NbTimeStep != nbt){
-	  Msg(GERROR, "Cannot combine views having different number of time steps");
-	  return;
-	}
+  int nbt = 0;
+  for(int i = 0; i < List_Nbr(id->indices); i++) {
+    List_Read(id->indices, i, &index);
+    Post_View *v = (Post_View*)List_Pointer(CTX.post.list, index);
+    if(!i){
+      nbt = v->NbTimeStep;
+    }
+    else{
+      if(v->NbTimeStep != nbt){
+	Msg(GERROR, "Cannot combine views having different number of time steps");
+	return;
       }
     }
   }
 
-  List_T *to_remove = List_Create(10, 10, sizeof(int));
-
   Post_View *vm = BeginView(1);
-  for(int i = 0; i < List_Nbr(CTX.post.list) - 1; i++) {
-    Post_View *v = (Post_View *) List_Pointer(CTX.post.list, i);
-    if(all || v->Visible) {
-      List_Insert(to_remove, &v->Num, fcmp_int);
-      combine(v->SP,vm->SP); vm->NbSP += v->NbSP;
-      combine(v->VP,vm->VP); vm->NbVP += v->NbVP; 
-      combine(v->TP,vm->TP); vm->NbTP += v->NbTP;
-      combine(v->SL,vm->SL); vm->NbSL += v->NbSL;
-      combine(v->VL,vm->VL); vm->NbVL += v->NbVL;
-      combine(v->TL,vm->TL); vm->NbTL += v->NbTL;
-      combine(v->ST,vm->ST); vm->NbST += v->NbST;
-      combine(v->VT,vm->VT); vm->NbVT += v->NbVT;
-      combine(v->TT,vm->TT); vm->NbTT += v->NbTT;
-      combine(v->SQ,vm->SQ); vm->NbSQ += v->NbSQ;
-      combine(v->VQ,vm->VQ); vm->NbVQ += v->NbVQ;
-      combine(v->TQ,vm->TQ); vm->NbTQ += v->NbTQ;
-      combine(v->SS,vm->SS); vm->NbSS += v->NbSS;
-      combine(v->VS,vm->VS); vm->NbVS += v->NbVS;
-      combine(v->TS,vm->TS); vm->NbTS += v->NbTS;
-      combine(v->SH,vm->SH); vm->NbSH += v->NbSH;
-      combine(v->VH,vm->VH); vm->NbVH += v->NbVH;
-      combine(v->TH,vm->TH); vm->NbTH += v->NbTH;
-      combine(v->SI,vm->SI); vm->NbSI += v->NbSI;
-      combine(v->VI,vm->VI); vm->NbVI += v->NbVI;
-      combine(v->TI,vm->TI); vm->NbTI += v->NbTI;
-      combine(v->SY,vm->SY); vm->NbSY += v->NbSY;
-      combine(v->VY,vm->VY); vm->NbVY += v->NbVY;
-      combine(v->TY,vm->TY); vm->NbTY += v->NbTY;
-      /* this more complicated: we have to recompute the indices
-         combine(v->T2D,vm->T2D);
-         combine(v->T2C,vm->T2C); v->NbT2 += vm->NbT2;
-         combine(v->T3D,vm->T3D);
-         combine(v->T3C,vm->T3C); v->NbT2 += vm->NbT2;
-       */
-    }
+  for(int i = 0; i < List_Nbr(id->indices); i++) {
+    List_Read(id->indices, i, &index);
+    Post_View *v = (Post_View*)List_Pointer(CTX.post.list, index);
+    List_Insert(to_remove, &v->Num, fcmp_int);
+    combine(v->SP,vm->SP); vm->NbSP += v->NbSP;
+    combine(v->VP,vm->VP); vm->NbVP += v->NbVP; 
+    combine(v->TP,vm->TP); vm->NbTP += v->NbTP;
+    combine(v->SL,vm->SL); vm->NbSL += v->NbSL;
+    combine(v->VL,vm->VL); vm->NbVL += v->NbVL;
+    combine(v->TL,vm->TL); vm->NbTL += v->NbTL;
+    combine(v->ST,vm->ST); vm->NbST += v->NbST;
+    combine(v->VT,vm->VT); vm->NbVT += v->NbVT;
+    combine(v->TT,vm->TT); vm->NbTT += v->NbTT;
+    combine(v->SQ,vm->SQ); vm->NbSQ += v->NbSQ;
+    combine(v->VQ,vm->VQ); vm->NbVQ += v->NbVQ;
+    combine(v->TQ,vm->TQ); vm->NbTQ += v->NbTQ;
+    combine(v->SS,vm->SS); vm->NbSS += v->NbSS;
+    combine(v->VS,vm->VS); vm->NbVS += v->NbVS;
+    combine(v->TS,vm->TS); vm->NbTS += v->NbTS;
+    combine(v->SH,vm->SH); vm->NbSH += v->NbSH;
+    combine(v->VH,vm->VH); vm->NbVH += v->NbVH;
+    combine(v->TH,vm->TH); vm->NbTH += v->NbTH;
+    combine(v->SI,vm->SI); vm->NbSI += v->NbSI;
+    combine(v->VI,vm->VI); vm->NbVI += v->NbVI;
+    combine(v->TI,vm->TI); vm->NbTI += v->NbTI;
+    combine(v->SY,vm->SY); vm->NbSY += v->NbSY;
+    combine(v->VY,vm->VY); vm->NbVY += v->NbVY;
+    combine(v->TY,vm->TY); vm->NbTY += v->NbTY;
+    /* this more complicated: we have to recompute the indices
+       combine(v->T2D,vm->T2D);
+       combine(v->T2C,vm->T2C); v->NbT2 += vm->NbT2;
+       combine(v->T3D,vm->T3D);
+       combine(v->T3C,vm->T3C); v->NbT2 += vm->NbT2;
+    */
   }
-  EndView(vm, 0, "combined.pos", "combined");
 
-  // remove original views?
-  if(remove){
-    for(int i = 0; i < List_Nbr(to_remove); i++) {
-      int num;
-      List_Read(to_remove, i, &num);
-      RemoveViewByNumber(num);
-    }
-  }
-  List_Delete(to_remove);
-
-#if defined(HAVE_FLTK)
-  UpdateViewsInGUI();
-#endif
+  // finalize
+  char name[256], filename[256], tmp[256];
+  if(!strcmp(id->name, "__all__"))
+    strcpy(tmp, "all");
+  else if(!strcmp(id->name, "__vis__"))
+    strcpy(tmp, "visible");
+  else
+    strcpy(tmp, id->name);
+  sprintf(name, "%s_Combine", tmp);
+  sprintf(filename, "%s_Combine.pos", tmp);
+  EndView(vm, 0, filename, name);
 }
 
-// Combine views (merge time steps)
-
-struct nameidx{
-  char name[256];
-  List_T *indices;
-};
-
-int fcmp_name(const void *a, const void *b){
-  char *name1 = ((struct nameidx*)a)->name;
-  char *name2 = ((struct nameidx*)b)->name;
-  return strcmp(name1, name2);
-}
-
-void combine_time(struct nameidx *id, List_T *to_remove)
+static void combine_time(struct nameidx *id, List_T *to_remove)
 {
   int index, *nbe=0, *nbe2=0, nbn, nbn2, nbc, nbc2;
   List_T *list=0, *list2=0;
@@ -1534,28 +1454,51 @@ void combine_time(struct nameidx *id, List_T *to_remove)
     }
   }
 
-  if(vm->empty()) {
-    RemoveViewByNumber(vm->Num);
-  }
-  else{
-    // create the time data
-    for(int i = 0; i < List_Nbr(id->indices); i++){
-      List_Read(id->indices, i, &index);
-      v = (Post_View*)List_Pointer(CTX.post.list, index);
-      for(int j = 0; j < List_Nbr(v->Time); j++){
-	List_Add(vm->Time, List_Pointer(v->Time, j));
-      }
+  // create the time data
+  for(int i = 0; i < List_Nbr(id->indices); i++){
+    List_Read(id->indices, i, &index);
+    v = (Post_View*)List_Pointer(CTX.post.list, index);
+    for(int j = 0; j < List_Nbr(v->Time); j++){
+      List_Add(vm->Time, List_Pointer(v->Time, j));
     }
-    // finalize
-    char name[256], filename[256];
-    sprintf(name, "combined-%s", id->name);
-    sprintf(filename, "combined-%s.pos", id->name);
-    EndView(vm, 0, filename, name);
   }
+  // if all the time values are the same, it probably means that the
+  // original views didn't have any time data: let's put some indices,
+  // then
+  double time0 = *(double*)List_Pointer(vm->Time, 0);
+  int nbtime = List_Nbr(vm->Time), ok = 0;
+  for(int i = 1; i < nbtime; i++){
+    if(time0 != *(double*)List_Pointer(vm->Time, i)){
+      ok = 1;
+      break;
+    }
+  }
+  if(!ok){
+    List_Reset(vm->Time);
+    for(int i = 0; i < nbtime; i++){
+      double time = i;
+      List_Add(vm->Time, &time);
+    }
+  }
+
+  // finalize
+  char name[256], filename[256], tmp[256];
+  if(!strcmp(id->name, "__all__"))
+    strcpy(tmp, "all");
+  else if(!strcmp(id->name, "__vis__"))
+    strcpy(tmp, "visible");
+  else
+    strcpy(tmp, id->name);
+  sprintf(name, "%s_Combine", tmp);
+  sprintf(filename, "%s_Combine.pos", tmp);
+  EndView(vm, 0, filename, name);
 }
 
-void CombineViews_Time(int how, int remove)
+void CombineViews(int time, int how, int remove)
 {
+  // time==0: combine the elements
+  // time==1: combine the timesteps
+
   // how==0: try to combine all visible views
   // how==1: try to combine all views
   // how==2: try to combine all views having identical names
@@ -1568,10 +1511,14 @@ void CombineViews_Time(int how, int remove)
     Post_View *v = (Post_View *) List_Pointer(CTX.post.list, i);
     if(how || v->Visible) {
       nameidx id;
+      // this might potentially lead to unwanted results if there are
+      // views named "__all__" or "__vis__", but what the heck...
       if(how == 2)
 	strcpy(id.name, v->Name);
+      else if(how == 1)
+	strcpy(id.name, "__all__");
       else
-	strcpy(id.name, "all");
+	strcpy(id.name, "__vis__");
       if((pid = (struct nameidx*)List_PQuery(ids, &id, fcmp_name))){
 	List_Add(pid->indices, &i);
       }
@@ -1585,7 +1532,10 @@ void CombineViews_Time(int how, int remove)
 
   for(int i = 0; i < List_Nbr(ids); i++){
     pid = (struct nameidx*)List_Pointer(ids, i);
-    combine_time(pid, to_remove);
+    if(time)
+      combine_time(pid, to_remove);
+    else
+      combine_space(pid, to_remove);
     List_Delete(pid->indices);
   }
   List_Delete(ids);
@@ -1607,16 +1557,6 @@ void CombineViews_Time(int how, int remove)
 
 // generic access functions
 
-int Post_View::get_val(int list, int node, int timestep, double *value)
-{
-  return 0;
-}
-
-void Post_View::add_val(int list, int node, int timestep, double value)
-{
-  ;
-}
-
 int Post_View::empty(){
   if(NbSP || NbVP || NbTP ||
      NbSL || NbVL || NbTL ||
@@ -1658,8 +1598,6 @@ void Post_View::get_raw_data(int type, List_T **list, int **nbe, int *nbc, int *
   case 21: *list = SY; *nbe = &NbSY; *nbc = 1; *nbn = 5; break;
   case 22: *list = VY; *nbe = &NbVY; *nbc = 3; *nbn = 5; break;
   case 23: *list = TY; *nbe = &NbTY; *nbc = 9; *nbn = 5; break;
-  default: 
-    Msg(FATAL, "Wrong type in Post_View::get_raw_data");
-    *list = NULL; *nbc = 0; *nbn = 0; break;
+  default: Msg(GERROR, "Wrong type in Post_View::get_raw_data"); break;
   }
 }
