@@ -31,7 +31,7 @@ void ManualPlay(int time, int step);
 
 void cancel_cb(CALLBACK_ARGS) ;
 void color_cb(CALLBACK_ARGS) ;
-void set_changed_cb(CALLBACK_ARGS);
+void redraw_cb(CALLBACK_ARGS);
 
 // Graphical window
 
@@ -84,14 +84,14 @@ void general_options_color_scheme_cb(CALLBACK_ARGS) ;
 void general_options_rotation_center_cb(CALLBACK_ARGS) ;
 void general_options_rotation_center_select_cb(CALLBACK_ARGS) ;
 void general_options_ok_cb(CALLBACK_ARGS) ;
+void general_arrow_param_cb(CALLBACK_ARGS);
 
 void geometry_options_cb(CALLBACK_ARGS) ;
-void geometry_options_color_scheme_cb(CALLBACK_ARGS) ;
 void geometry_options_ok_cb(CALLBACK_ARGS) ;
 
 void mesh_options_cb(CALLBACK_ARGS) ;
-void mesh_options_color_scheme_cb(CALLBACK_ARGS) ;
 void mesh_options_ok_cb(CALLBACK_ARGS) ;
+void mesh_cut_plane_cb(CALLBACK_ARGS) ;
 
 void solver_options_cb(CALLBACK_ARGS) ;
 void solver_options_ok_cb(CALLBACK_ARGS) ;
@@ -107,29 +107,28 @@ void view_remove_cb(CALLBACK_ARGS) ;
 void view_remove_all_cb(CALLBACK_ARGS) ;
 void view_remove_visible_cb(CALLBACK_ARGS) ;
 void view_remove_invisible_cb(CALLBACK_ARGS) ;
+void view_remove_empty_cb(CALLBACK_ARGS) ;
 void view_save_ascii_cb(CALLBACK_ARGS) ;
 void view_save_binary_cb(CALLBACK_ARGS) ;
+void view_save_parsed_cb(CALLBACK_ARGS) ;
 void view_duplicate_cb(CALLBACK_ARGS) ;
 void view_duplicate_with_options_cb(CALLBACK_ARGS) ;
-void view_combine_all_cb(CALLBACK_ARGS) ;
-void view_combine_all_and_remove_cb(CALLBACK_ARGS) ;
-void view_combine_visible_cb(CALLBACK_ARGS) ;
-void view_combine_visible_and_remove_cb(CALLBACK_ARGS) ;
+void view_combine_space_all_cb(CALLBACK_ARGS) ;
+void view_combine_space_visible_cb(CALLBACK_ARGS) ;
+void view_combine_space_by_name_cb(CALLBACK_ARGS) ;
 void view_combine_time_all_cb(CALLBACK_ARGS) ;
-void view_combine_time_all_and_remove_cb(CALLBACK_ARGS) ;
 void view_combine_time_visible_cb(CALLBACK_ARGS) ;
-void view_combine_time_visible_and_remove_cb(CALLBACK_ARGS) ;
 void view_combine_time_by_name_cb(CALLBACK_ARGS) ;
-void view_combine_time_by_name_and_remove_cb(CALLBACK_ARGS) ;
+void view_all_visible_cb(CALLBACK_ARGS) ;
 void view_applybgmesh_cb(CALLBACK_ARGS) ;
+void view_plugin_options_cb(CALLBACK_ARGS) ;
 void view_options_cb(CALLBACK_ARGS) ;
-void view_plugin_cb(CALLBACK_ARGS) ;
-void view_options_plugin_cb(CALLBACK_ARGS) ;
 void view_options_custom_cb(CALLBACK_ARGS) ;
 void view_options_timestep_cb(CALLBACK_ARGS) ;
 void view_options_timestep_decr_cb(CALLBACK_ARGS);
 void view_options_timestep_incr_cb(CALLBACK_ARGS);
 void view_options_ok_cb(CALLBACK_ARGS) ;
+void view_arrow_param_cb(CALLBACK_ARGS);
 
 // Statistics Menu
 
@@ -151,12 +150,21 @@ void visibility_sort_cb(CALLBACK_ARGS) ;
 void visibility_number_cb(CALLBACK_ARGS);
 void visibility_ok_cb(CALLBACK_ARGS) ;
 
+// Clipping planes Menu
+
+void clip_cb(CALLBACK_ARGS) ;
+void clip_update_cb(CALLBACK_ARGS) ;
+void clip_num_cb(CALLBACK_ARGS) ;
+void clip_reset_cb(CALLBACK_ARGS) ;
+
 // Help Menu
 
 void help_short_cb(CALLBACK_ARGS) ;
+void help_mouse_cb(CALLBACK_ARGS) ;
 void help_command_line_cb(CALLBACK_ARGS) ;
 void help_online_cb(CALLBACK_ARGS) ;
 void help_license_cb(CALLBACK_ARGS) ;
+void help_credits_cb(CALLBACK_ARGS) ;
 void help_about_cb(CALLBACK_ARGS) ;
 
 // Module Menu
@@ -239,10 +247,6 @@ void geometry_reload_cb(CALLBACK_ARGS) ;
 
 void con_geometry_define_parameter_cb(CALLBACK_ARGS) ;
 void con_geometry_define_point_cb(CALLBACK_ARGS) ;
-void con_geometry_define_translation_cb(CALLBACK_ARGS) ;
-void con_geometry_define_rotation_cb(CALLBACK_ARGS) ;
-void con_geometry_define_scale_cb(CALLBACK_ARGS) ;
-void con_geometry_define_symmetry_cb(CALLBACK_ARGS) ;
 
 // Dynamic Mesh Menus
 
@@ -253,6 +257,7 @@ void mesh_1d_cb(CALLBACK_ARGS) ;
 void mesh_2d_cb(CALLBACK_ARGS) ; 
 void mesh_3d_cb(CALLBACK_ARGS) ; 
 void mesh_degree_cb(CALLBACK_ARGS) ; 
+void mesh_optimize_cb(CALLBACK_ARGS) ; 
 void mesh_define_length_cb (CALLBACK_ARGS) ;
 void mesh_define_recombine_cb (CALLBACK_ARGS) ;
 void mesh_define_transfinite_cb (CALLBACK_ARGS) ; 
@@ -260,10 +265,6 @@ void mesh_define_transfinite_line_cb(CALLBACK_ARGS) ;
 void mesh_define_transfinite_surface_cb(CALLBACK_ARGS) ;
 void mesh_define_transfinite_volume_cb(CALLBACK_ARGS) ; 
 
-void con_mesh_define_length_cb(CALLBACK_ARGS) ;
-void con_mesh_define_transfinite_line_cb(CALLBACK_ARGS) ;
-void con_mesh_define_transfinite_volume_cb(CALLBACK_ARGS) ;
-
 // Dynamic Solver Menus
 
 void solver_cb(CALLBACK_ARGS);
@@ -275,10 +276,5 @@ void solver_kill_cb(CALLBACK_ARGS);
 void solver_choose_executable_cb(CALLBACK_ARGS);
 void solver_ok_cb(CALLBACK_ARGS);
 
-// Arrow editor windows
-
-void general_arrow_param_cb(CALLBACK_ARGS);
-void view_arrow_param_cb(CALLBACK_ARGS);
-
 #endif
 
