@@ -1,4 +1,4 @@
-// $Id: CAD.cpp,v 1.74 2004/04/13 18:47:10 geuzaine Exp $
+// $Id: CAD.cpp,v 1.79 2004/09/29 21:08:25 geuzaine Exp $
 //
 // Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
 //
@@ -362,7 +362,7 @@ void CopyShape(int Type, int Num, int *New)
   switch (Type) {
   case MSH_POINT:
     if(!(v = FindPoint(Num, THEM))) {
-      Msg(GERROR, "Unknown Vertex %d", Num);
+      Msg(GERROR, "Unknown vertex %d", Num);
       return;
     }
     newv = DuplicateVertex(v);
@@ -377,7 +377,7 @@ void CopyShape(int Type, int Num, int *New)
   case MSH_SEGM_NURBS:
   case MSH_SEGM_PARAMETRIC:
     if(!(c = FindCurve(Num, THEM))) {
-      Msg(GERROR, "Unknown Curve %d", Num);
+      Msg(GERROR, "Unknown curve %d", Num);
       return;
     }
     newc = DuplicateCurve(c);
@@ -388,7 +388,7 @@ void CopyShape(int Type, int Num, int *New)
   case MSH_SURF_REGL:
   case MSH_SURF_PLAN:
     if(!(s = FindSurface(Num, THEM))) {
-      Msg(GERROR, "Unknown Surface %d", Num);
+      Msg(GERROR, "Unknown surface %d", Num);
       return;
     }
     news = DuplicateSurface(s);
@@ -400,8 +400,6 @@ void CopyShape(int Type, int Num, int *New)
   }
 }
 
-// FIXME: we still need to actually free the entity!
-
 void DeletePoint(int ip)
 {
   Vertex *v = FindPoint(ip, THEM);
@@ -420,6 +418,7 @@ void DeletePoint(int ip)
   if(v->Num == THEM->MaxPointNum)
     THEM->MaxPointNum--;
   Tree_Suppress(THEM->Points, &v);
+  Free_Vertex(&v, NULL);
 }
 
 void DeleteCurve(int ip)
@@ -440,12 +439,11 @@ void DeleteCurve(int ip)
   if(c->Num == THEM->MaxLineNum)
     THEM->MaxLineNum--;
   Tree_Suppress(THEM->Curves, &c);
+  Free_Curve(&c, NULL);
 }
 
 void DeleteSurface(int is)
 {
-  // Il faut absolument coder une
-  // structure coherente pour les volumes.
   Surface *s = FindSurface(is, THEM);
   if(!s)
     return;
@@ -454,7 +452,7 @@ void DeleteSurface(int is)
     Volume *v;
     List_Read(Vols, i, &v);
     for(int j = 0; j < List_Nbr(v->Surfaces); j++) {
-      if(!compareCurve(List_Pointer(v->Surfaces, j), &s))
+      if(!compareSurface(List_Pointer(v->Surfaces, j), &s))
         return;
     }
   }
@@ -462,6 +460,18 @@ void DeleteSurface(int is)
   if(s->Num == THEM->MaxSurfaceNum)
     THEM->MaxSurfaceNum--;
   Tree_Suppress(THEM->Surfaces, &s);
+  Free_Surface(&s, NULL);
+}
+
+void DeleteVolume(int iv)
+{
+  Volume *v = FindVolume(iv, THEM);
+  if(!v)
+    return;
+  if(v->Num == THEM->MaxVolumeNum)
+    THEM->MaxVolumeNum--;
+  Tree_Suppress(THEM->Volumes, &v);
+  Free_Volume(&v, NULL);
 }
 
 void DeleteShape(int Type, int Num)
@@ -486,6 +496,9 @@ void DeleteShape(int Type, int Num)
   case MSH_SURF_PLAN:
     DeleteSurface(Num);
     break;
+  case MSH_VOLUME:
+    DeleteVolume(Num);
+    break;
   default:
     Msg(GERROR, "Impossible to delete entity %d (of type %d)", Num, Type);
     break;
@@ -950,7 +963,7 @@ void ApplicationOnShapes(double matrix[4][4], List_T * ListShapes)
       if(v)
         ApplyTransformationToPoint(matrix, v);
       else
-        Msg(GERROR, "Unknown Point %d", O.Num);
+        Msg(GERROR, "Unknown point %d", O.Num);
       break;
     case MSH_SEGM_LINE:
     case MSH_SEGM_SPLN:
@@ -964,7 +977,7 @@ void ApplicationOnShapes(double matrix[4][4], List_T * ListShapes)
       if(c)
         ApplyTransformationToCurve(matrix, c);
       else
-        Msg(GERROR, "Unknown Curve %d", O.Num);
+        Msg(GERROR, "Unknown curve %d", O.Num);
       break;
     case MSH_SURF_NURBS:
     case MSH_SURF_REGL:
@@ -974,7 +987,7 @@ void ApplicationOnShapes(double matrix[4][4], List_T * ListShapes)
       if(s)
         ApplyTransformationToSurface(matrix, s);
       else
-        Msg(GERROR, "Unknown Surface %d", O.Num);
+        Msg(GERROR, "Unknown surface %d", O.Num);
       break;
     default:
       Msg(GERROR, "Impossible to transform entity %d (of type %d)", O.Num,
@@ -1413,13 +1426,14 @@ int Extrude_ProtudeSurface(int type, int is,
 			   double T0, double T1, double T2,
 			   double A0, double A1, double A2,
 			   double X0, double X1, double X2, double alpha,
-			   int NewVolume, ExtrudeParams * e)
+			   Volume **pv, ExtrudeParams * e)
 {
   double matrix[4][4], T[3], Ax[3];
   Curve *c, *c2;
   int i;
   Surface *s, *ps, *chapeau;
-  Volume *pv = NULL;
+
+  *pv = NULL;
 
   if(!(ps = FindSurface(is, THEM)))
     return 0;
@@ -1439,7 +1453,7 @@ int Extrude_ProtudeSurface(int type, int is,
     List_Read(chapeau->Generatrices, i, &c);
     if(c->Num < 0)
       if(!(c = FindCurve(-c->Num, THEM))) {
-        Msg(GERROR, "Unknown Curve %d", -c->Num);
+        Msg(GERROR, "Unknown curve %d", -c->Num);
         return ps->Num;
       }
     c->Extrude = new ExtrudeParams(COPIED_ENTITY);
@@ -1450,25 +1464,41 @@ int Extrude_ProtudeSurface(int type, int is,
       c->Extrude->mesh = e->mesh;
   }
 
-  if(NewVolume) {
-    pv = Create_Volume(NewVolume, 0);
-    pv->Extrude = new ExtrudeParams;
-    pv->Extrude->fill(type, T0, T1, T2, A0, A1, A2, X0, X1, X2, alpha);
-    pv->Extrude->geo.Source = is;
-    if(e)
-      pv->Extrude->mesh = e->mesh;
-    if(pv)
-      List_Add(pv->Surfaces, &ps);
-  }
-  if(pv)
-    List_Add(pv->Surfaces, &chapeau);
+  // FIXME: this is a really ugly hack for backward compatibility, so
+  // that we don't screw up the old .geo files too much. (Before
+  // version 1.54, we didn't always create new volumes during "Extrude
+  // Surface". Now we do, but with "CTX.geom.old_newreg==1", this
+  // bumps the NEWREG() counter, and thus changes the whole automatic
+  // numbering sequence.) So we locally force old_newreg to 0: in most
+  // cases, since we define points, curves, etc., before defining
+  // volumes, the NEWVOLUME() call below will return a fairly low
+  // number, that will not interfere with the other numbers...
+  int tmp = CTX.geom.old_newreg;
+  CTX.geom.old_newreg = 0;
+  Volume *v = Create_Volume(NEWVOLUME(), MSH_VOLUME);
+  CTX.geom.old_newreg = tmp;
+
+  v->Extrude = new ExtrudeParams;
+  v->Extrude->fill(type, T0, T1, T2, A0, A1, A2, X0, X1, X2, alpha);
+  v->Extrude->geo.Source = is;
+  if(e)
+    v->Extrude->mesh = e->mesh;
+  int ori = 1;;
+  List_Add(v->Surfaces, &ps);
+  List_Add(v->SurfacesOrientations, &ori);
+  ori = -1;
+  List_Add(v->Surfaces, &chapeau);
+  List_Add(v->SurfacesOrientations, &ori);
 
   for(i = 0; i < List_Nbr(ps->Generatrices); i++) {
     List_Read(ps->Generatrices, i, &c);
     Extrude_ProtudeCurve(type, c->Num, T0, T1, T2, A0, A1, A2, X0, X1, X2,
 			 alpha, &s, 0, e);
-    if(pv && s)
-      List_Add(pv->Surfaces, &s);
+    if(s){
+      ori = -1;
+      List_Add(v->Surfaces, &s);
+      List_Add(v->SurfacesOrientations, &ori);
+    }
   }
 
   switch (type) {
@@ -1536,14 +1566,15 @@ int Extrude_ProtudeSurface(int type, int is,
     return ps->Num;
   }
 
-  // why do we do this? only for backward compatibility?
+  // FIXME: why do we do this? only for backward compatibility?
   Tree_Suppress(THEM->Surfaces, &chapeau);
   chapeau->Num = NEWSURFACE();
   THEM->MaxSurfaceNum = chapeau->Num;
   Tree_Add(THEM->Surfaces, &chapeau);
 
-  if(pv)
-    Tree_Add(THEM->Volumes, &pv);
+  Tree_Add(THEM->Volumes, &v);
+
+  *pv = v;
 
   if(CTX.geom.auto_coherence)
     ReplaceAllDuplicates(THEM);
@@ -1562,9 +1593,17 @@ int compareTwoCurves(const void *a, const void *b)
   c2 = *(Curve **) b;
   int comp;
 
-  if(c1->Typ != c2->Typ)
-    return c1->Typ - c2->Typ;
-  
+  if(c1->Typ != c2->Typ){
+    if((c1->Typ == MSH_SEGM_CIRC && c2->Typ == MSH_SEGM_CIRC_INV) ||
+       (c1->Typ == MSH_SEGM_CIRC_INV && c2->Typ == MSH_SEGM_CIRC) ||
+       (c1->Typ == MSH_SEGM_ELLI && c2->Typ == MSH_SEGM_ELLI_INV) ||
+       (c1->Typ == MSH_SEGM_ELLI_INV && c2->Typ == MSH_SEGM_ELLI)){
+      // this is still ok
+    }
+    else
+      return c1->Typ - c2->Typ;
+  }
+
   if(List_Nbr(c1->Control_Points) != List_Nbr(c2->Control_Points))
     return List_Nbr(c1->Control_Points) - List_Nbr(c2->Control_Points);
   
@@ -1628,10 +1667,11 @@ void MaxNumSurface(void *a, void *b)
 void ReplaceDuplicatePoints(Mesh * m)
 {
   List_T *All;
-  Tree_T *allNonDulpicatedPoints;
+  Tree_T *allNonDuplicatedPoints;
   Vertex *v, **pv, **pv2;
   Curve *c;
   Surface *s;
+  Volume *vol;
   int i, j, start, end;
 
   List_T *points2delete = List_Create(100, 100, sizeof(Vertex *));
@@ -1641,11 +1681,11 @@ void ReplaceDuplicatePoints(Mesh * m)
   start = Tree_Nbr(m->Points);
 
   All = Tree2List(m->Points);
-  allNonDulpicatedPoints = Tree_Create(sizeof(Vertex *), comparePosition);
+  allNonDuplicatedPoints = Tree_Create(sizeof(Vertex *), comparePosition);
   for(i = 0; i < List_Nbr(All); i++) {
     List_Read(All, i, &v);
-    if(!Tree_Search(allNonDulpicatedPoints, &v)) {
-      Tree_Insert(allNonDulpicatedPoints, &v);
+    if(!Tree_Search(allNonDuplicatedPoints, &v)) {
+      Tree_Insert(allNonDuplicatedPoints, &v);
     }
     else {
       Tree_Suppress(m->Points, &v);
@@ -1658,7 +1698,7 @@ void ReplaceDuplicatePoints(Mesh * m)
   end = Tree_Nbr(m->Points);
 
   if(start == end) {
-    Tree_Delete(allNonDulpicatedPoints);
+    Tree_Delete(allNonDuplicatedPoints);
     List_Delete(points2delete);
     return;
   }
@@ -1676,13 +1716,13 @@ void ReplaceDuplicatePoints(Mesh * m)
   All = Tree2List(m->Curves);
   for(i = 0; i < List_Nbr(All); i++) {
     List_Read(All, i, &c);
-    if(!Tree_Query(allNonDulpicatedPoints, &c->beg))
+    if(!Tree_Query(allNonDuplicatedPoints, &c->beg))
       Msg(GERROR, "Weird point %d in Coherence", c->beg->Num);
-    if(!Tree_Query(allNonDulpicatedPoints, &c->end))
+    if(!Tree_Query(allNonDuplicatedPoints, &c->end))
       Msg(GERROR, "Weird point %d in Coherence", c->end->Num);
     for(j = 0; j < List_Nbr(c->Control_Points); j++) {
       pv = (Vertex **) List_Pointer(c->Control_Points, j);
-      if(!(pv2 = (Vertex **) Tree_PQuery(allNonDulpicatedPoints, pv)))
+      if(!(pv2 = (Vertex **) Tree_PQuery(allNonDuplicatedPoints, pv)))
         Msg(GERROR, "Weird point %d in Coherence", (*pv)->Num);
       else
         List_Write(c->Control_Points, j, pv2);
@@ -1697,11 +1737,33 @@ void ReplaceDuplicatePoints(Mesh * m)
     List_Read(All, i, &s);
     for(j = 0; j < List_Nbr(s->Control_Points); j++) {
       pv = (Vertex **) List_Pointer(s->Control_Points, j);
-      if(!(pv2 = (Vertex **) Tree_PQuery(allNonDulpicatedPoints, pv)))
+      if(!(pv2 = (Vertex **) Tree_PQuery(allNonDuplicatedPoints, pv)))
         Msg(GERROR, "Weird point %d in Coherence", (*pv)->Num);
       else
         List_Write(s->Control_Points, j, pv2);
     }
+    for(j = 0; j < List_Nbr(s->TrsfPoints); j++){
+      pv = (Vertex **) List_Pointer(s->TrsfPoints, j);
+      if(!(pv2 = (Vertex **) Tree_PQuery(allNonDuplicatedPoints, pv)))
+	Msg(GERROR, "Weird point %d in Coherence", (*pv)->Num);
+      else
+	List_Write(s->TrsfPoints, j, pv2);
+    }
+  }
+  List_Delete(All);
+  
+  // Replace old points in volumes
+
+  All = Tree2List(m->Volumes);
+  for(i = 0; i < List_Nbr(All); i++) {
+    List_Read(All, i, &vol);
+    for(j = 0; j < List_Nbr(vol->TrsfPoints); j++){
+      pv = (Vertex **) List_Pointer(vol->TrsfPoints, j);
+      if(!(pv2 = (Vertex **) Tree_PQuery(allNonDuplicatedPoints, pv)))
+	Msg(GERROR, "Weird point %d in Coherence", (*pv)->Num);
+      else
+	List_Write(vol->TrsfPoints, j, pv2);
+    }
   }
   List_Delete(All);
 
@@ -1710,7 +1772,7 @@ void ReplaceDuplicatePoints(Mesh * m)
     Free_Vertex(&v, 0);
   }
 
-  Tree_Delete(allNonDulpicatedPoints);
+  Tree_Delete(allNonDuplicatedPoints);
   List_Delete(points2delete);
 
 }
@@ -1718,7 +1780,7 @@ void ReplaceDuplicatePoints(Mesh * m)
 void ReplaceDuplicateCurves(Mesh * m)
 {
   List_T *All;
-  Tree_T *allNonDulpicatedCurves;
+  Tree_T *allNonDuplicatedCurves;
   Curve *c, *c2, **pc, **pc2;
   Surface *s;
   int i, j, start, end;
@@ -1728,23 +1790,23 @@ void ReplaceDuplicateCurves(Mesh * m)
   start = Tree_Nbr(m->Curves);
 
   All = Tree2List(m->Curves);
-  allNonDulpicatedCurves = Tree_Create(sizeof(Curve *), compareTwoCurves);
+  allNonDuplicatedCurves = Tree_Create(sizeof(Curve *), compareTwoCurves);
   for(i = 0; i < List_Nbr(All); i++) {
     List_Read(All, i, &c);
     if(c->Num > 0) {
-      if(!Tree_Search(allNonDulpicatedCurves, &c)) {
-        Tree_Insert(allNonDulpicatedCurves, &c);
+      if(!Tree_Search(allNonDuplicatedCurves, &c)) {
+        Tree_Insert(allNonDuplicatedCurves, &c);
         if(!(c2 = FindCurve(-c->Num, m))) {
-          Msg(GERROR, "Unknown Curve %d", -c->Num);
+          Msg(GERROR, "Unknown curve %d", -c->Num);
           List_Delete(All);
           return;
         }
-        Tree_Insert(allNonDulpicatedCurves, &c2);
+        Tree_Insert(allNonDuplicatedCurves, &c2);
       }
       else {
         Tree_Suppress(m->Curves, &c);
         if(!(c2 = FindCurve(-c->Num, m))) {
-          Msg(GERROR, "Unknown Curve %d", -c->Num);
+          Msg(GERROR, "Unknown curve %d", -c->Num);
           List_Delete(All);
           return;
         }
@@ -1757,7 +1819,7 @@ void ReplaceDuplicateCurves(Mesh * m)
   end = Tree_Nbr(m->Curves);
 
   if(start == end) {
-    Tree_Delete(allNonDulpicatedCurves);
+    Tree_Delete(allNonDuplicatedCurves);
     return;
   }
 
@@ -1775,7 +1837,7 @@ void ReplaceDuplicateCurves(Mesh * m)
     List_Read(All, i, &s);
     for(j = 0; j < List_Nbr(s->Generatrices); j++) {
       pc = (Curve **) List_Pointer(s->Generatrices, j);
-      if(!(pc2 = (Curve **) Tree_PQuery(allNonDulpicatedCurves, pc)))
+      if(!(pc2 = (Curve **) Tree_PQuery(allNonDuplicatedCurves, pc)))
         Msg(GERROR, "Weird curve %d in Coherence", (*pc)->Num);
       else {
         List_Write(s->Generatrices, j, pc2);
@@ -1786,13 +1848,13 @@ void ReplaceDuplicateCurves(Mesh * m)
   }
   List_Delete(All);
 
-  Tree_Delete(allNonDulpicatedCurves);
+  Tree_Delete(allNonDuplicatedCurves);
 }
 
 void ReplaceDuplicateSurfaces(Mesh * m)
 {
   List_T *All;
-  Tree_T *allNonDulpicatedSurfaces;
+  Tree_T *allNonDuplicatedSurfaces;
   Surface *s, **ps, **ps2;
   Volume *vol;
   int i, j, start, end;
@@ -1802,12 +1864,12 @@ void ReplaceDuplicateSurfaces(Mesh * m)
   start = Tree_Nbr(m->Surfaces);
 
   All = Tree2List(m->Surfaces);
-  allNonDulpicatedSurfaces = Tree_Create(sizeof(Curve *), compareTwoSurfaces);
+  allNonDuplicatedSurfaces = Tree_Create(sizeof(Curve *), compareTwoSurfaces);
   for(i = 0; i < List_Nbr(All); i++) {
     List_Read(All, i, &s);
     if(s->Num > 0) {
-      if(!Tree_Search(allNonDulpicatedSurfaces, &s)) {
-        Tree_Insert(allNonDulpicatedSurfaces, &s);
+      if(!Tree_Search(allNonDuplicatedSurfaces, &s)) {
+        Tree_Insert(allNonDuplicatedSurfaces, &s);
       }
       else {
         Tree_Suppress(m->Surfaces, &s);
@@ -1819,7 +1881,7 @@ void ReplaceDuplicateSurfaces(Mesh * m)
   end = Tree_Nbr(m->Surfaces);
 
   if(start == end) {
-    Tree_Delete(allNonDulpicatedSurfaces);
+    Tree_Delete(allNonDuplicatedSurfaces);
     return;
   }
 
@@ -1837,7 +1899,7 @@ void ReplaceDuplicateSurfaces(Mesh * m)
     List_Read(All, i, &vol);
     for(j = 0; j < List_Nbr(vol->Surfaces); j++) {
       ps = (Surface **) List_Pointer(vol->Surfaces, j);
-      if(!(ps2 = (Surface **) Tree_PQuery(allNonDulpicatedSurfaces, ps)))
+      if(!(ps2 = (Surface **) Tree_PQuery(allNonDuplicatedSurfaces, ps)))
         Msg(GERROR, "Weird surface %d in Coherence", (*ps)->Num);
       else
         List_Write(vol->Surfaces, j, ps2);
@@ -1845,7 +1907,7 @@ void ReplaceDuplicateSurfaces(Mesh * m)
   }
   List_Delete(All);
 
-  Tree_Delete(allNonDulpicatedSurfaces);
+  Tree_Delete(allNonDuplicatedSurfaces);
 }
 
 void ReplaceAllDuplicates(Mesh * m)
