@@ -1,4 +1,4 @@
-// $Id: Geom.cpp,v 1.54 2004/04/20 18:55:36 geuzaine Exp $
+// $Id: Geom.cpp,v 1.73 2004/10/28 03:44:37 geuzaine Exp $
 //
 // Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
 //
@@ -23,11 +23,14 @@
 #include "GmshUI.h"
 #include "Numeric.h"
 #include "Geo.h"
+#include "CAD.h"
 #include "Mesh.h"
 #include "Utils.h"
 #include "Draw.h"
 #include "Context.h"
 #include "Interpolation.h"
+#include "Plugin.h"
+#include "PluginManager.h"
 #include "STL.h"
 #include "gl2ps.h"
 
@@ -36,14 +39,11 @@ extern Mesh *THEM;
 
 // Points
 
-static int Highlighted = 0;
-
-void Draw_GeoPoint(void *a, void *b)
+void Draw_Geo_Point(void *a, void *b)
 {
-  Vertex *v;
   char Num[100];
 
-  v = *(Vertex **) a;
+  Vertex *v = *(Vertex **) a;
 
   if(!(v->Visible & VIS_GEOM))
     return;
@@ -58,11 +58,6 @@ void Draw_GeoPoint(void *a, void *b)
     gl2psPointSize(CTX.geom.point_sel_size * CTX.print.eps_point_size_factor);
     glColor4ubv((GLubyte *) & CTX.color.geom.point_sel);
   }
-  else if(Highlighted) {
-    glPointSize(CTX.geom.point_sel_size);
-    gl2psPointSize(CTX.geom.point_sel_size * CTX.print.eps_point_size_factor);
-    glColor4ubv((GLubyte *) & CTX.color.geom.point_hlt);
-  }
   else {
     glPointSize(CTX.geom.point_size);
     gl2psPointSize(CTX.geom.point_size * CTX.print.eps_point_size_factor);
@@ -70,24 +65,34 @@ void Draw_GeoPoint(void *a, void *b)
   }
 
   if(CTX.geom.points) {
-    if(CTX.geom.point_type) {
-      if(v->Frozen || Highlighted)
+    if(CTX.geom.point_type == 1) {
+      if(v->Frozen)
         Draw_Sphere(CTX.geom.point_sel_size, v->Pos.X, v->Pos.Y, v->Pos.Z, 
 		    CTX.geom.light);
       else
         Draw_Sphere(CTX.geom.point_size, v->Pos.X, v->Pos.Y, v->Pos.Z,
 		    CTX.geom.light);
     }
+    else if(CTX.geom.point_type == 2) {
+      GMSH_Solve_Plugin *sp = GMSH_PluginManager::instance()->findSolverPlugin();
+      if(sp) {
+	sp-> GL_enhancePoint (v);
+      }
+      glBegin(GL_POINTS);
+      glVertex3d(v->Pos.X, v->Pos.Y, v->Pos.Z);
+      glEnd();
+    }
     else {
       glBegin(GL_POINTS);
       glVertex3d(v->Pos.X, v->Pos.Y, v->Pos.Z);
       glEnd();
     }
+
   }
 
   if(CTX.geom.points_num) {
     sprintf(Num, "%d", v->Num);
-    double offset = 0.5 * (CTX.geom.point_size + CTX.gl_fontsize) * CTX.pixel_equiv_x;
+    double offset = (0.5 * CTX.geom.point_size + 0.3 * CTX.gl_fontsize) * CTX.pixel_equiv_x;
     glRasterPos3d(v->Pos.X + offset / CTX.s[0],
                   v->Pos.Y + offset / CTX.s[1],
                   v->Pos.Z + offset / CTX.s[2]);
@@ -103,13 +108,12 @@ void Draw_GeoPoint(void *a, void *b)
 
 void Draw_Curve(void *a, void *b)
 {
-  int i, N;
+  int N;
   double mod, x[2], y[2], z[2];
   char Num[100];
-  Curve *c;
   Vertex v, dv;
 
-  c = *(Curve **) a;
+  Curve *c = *(Curve **) a;
 
   if(c->Num < 0 || !(c->Visible & VIS_GEOM) || c->Dirty)
     return;
@@ -119,16 +123,11 @@ void Draw_Curve(void *a, void *b)
     glPushName(c->Num);
   }
 
-  if((c)->ipar[3]) {
+  if(c->ipar[3]) {
     glLineWidth(CTX.geom.line_sel_width);
     gl2psLineWidth(CTX.geom.line_sel_width * CTX.print.eps_line_width_factor);
     glColor4ubv((GLubyte *) & CTX.color.geom.line_sel);
   }
-  else if(Highlighted) {
-    glLineWidth(CTX.geom.line_sel_width);
-    gl2psLineWidth(CTX.geom.line_sel_width * CTX.print.eps_line_width_factor);
-    glColor4ubv((GLubyte *) & CTX.color.geom.line_hlt);
-  }
   else {
     glLineWidth(CTX.geom.line_width);
     gl2psLineWidth(CTX.geom.line_width * CTX.print.eps_line_width_factor);
@@ -154,7 +153,7 @@ void Draw_Curve(void *a, void *b)
     if(c->Typ == MSH_SEGM_DISCRETE) {
       Simplex *s;
       List_T *temp = Tree2List(c->Simplexes);
-      for(i = 0; i < List_Nbr(temp); i++) {
+      for(int i = 0; i < List_Nbr(temp); i++) {
         List_Read(temp, i, &s);
         glBegin(GL_LINE_STRIP);
         glVertex3d(s->V[0]->Pos.X, s->V[0]->Pos.Y, s->V[0]->Pos.Z);
@@ -164,8 +163,8 @@ void Draw_Curve(void *a, void *b)
       List_Delete(temp);
     }
     else {
-      if(CTX.geom.line_type) {
-        for(i = 0; i < N - 1; i++) {
+      if(CTX.geom.line_type >= 1) {
+        for(int i = 0; i < N - 1; i++) {
           v = InterpolateCurve(c, (double)i / (double)(N - 1), 0);
           dv = InterpolateCurve(c, (double)(i + 1) / (double)(N - 1), 0);
           x[0] = v.Pos.X;
@@ -174,12 +173,25 @@ void Draw_Curve(void *a, void *b)
           x[1] = dv.Pos.X;
           y[1] = dv.Pos.Y;
           z[1] = dv.Pos.Z;
-          Draw_Cylinder(CTX.geom.line_width, x, y, z, CTX.geom.light);
+          Draw_Cylinder(c->ipar[3] ? CTX.geom.line_sel_width : CTX.geom.line_width,
+			x, y, z, CTX.geom.light);
         }
+	if(CTX.geom.line_type == 2) {
+	  GMSH_Solve_Plugin *sp = GMSH_PluginManager::instance()->findSolverPlugin();
+	  if(sp) {
+	    int NN=(N>1)?N:1;
+	    const double eps=0.e-2;
+	    for(int i = 0; i < NN - 1; i++) {
+	      v = InterpolateCurve(c, (double)i / (double)(NN - 1)-eps, 0);
+	      dv = InterpolateCurve(c, (double)(i + 1) / (double)(NN - 1)+eps, 0);
+	      sp-> GL_enhanceLine (c->Num,&v,&dv);
+	    }
+	  }
+	}
       }
       else {
         glBegin(GL_LINE_STRIP);
-        for(i = 0; i < N; i++) {
+        for(int i = 0; i < N; i++) {
           v = InterpolateCurve(c, (double)i / (double)(N - 1), 0);
           glVertex3d(v.Pos.X, v.Pos.Y, v.Pos.Z);
         }
@@ -191,7 +203,7 @@ void Draw_Curve(void *a, void *b)
   if(CTX.geom.lines_num) {
     v = InterpolateCurve(c, 0.5, 0);
     sprintf(Num, "%d", c->Num);
-    double offset = 0.5 * (CTX.geom.line_width + CTX.gl_fontsize) * CTX.pixel_equiv_x;
+    double offset = (0.5 * CTX.geom.line_width + 0.3 * CTX.gl_fontsize) * CTX.pixel_equiv_x;
     glRasterPos3d(v.Pos.X + offset / CTX.s[0],
                   v.Pos.Y + offset / CTX.s[1],
                   v.Pos.Z + offset / CTX.s[2]);
@@ -219,7 +231,7 @@ void Draw_Curve(void *a, void *b)
 
 // Surfaces
 
-void put_Z(Vertex * v, Surface * s)
+void putZ(Vertex * v, Surface * s)
 {
   Vertex V;
   V.Pos.X = s->a;
@@ -237,42 +249,75 @@ void put_Z(Vertex * v, Surface * s)
 int isPointOnPlanarSurface(Surface * S, double X, double Y, double Z,
                            double n[3])
 {
-  Curve *C;
-  Vertex V, P1, P2;
-  int i, j, N;
-  double Angle, u1, u2;
-
-  Angle = 0.0;
+  double Angle = 0.0;
+  Vertex V;
   V.Pos.X = X;
   V.Pos.Y = Y;
   V.Pos.Z = Z;
 
-  for(i = 0; i < List_Nbr(S->Generatrices); i++) {
+  for(int i = 0; i < List_Nbr(S->Generatrices); i++) {
 
+    Curve *C;
     List_Read(S->Generatrices, i, &C);
 
-    if(C->Typ == MSH_SEGM_LINE)
-      N = 1;
-    else
-      N = 10;
+    int N = (C->Typ == MSH_SEGM_LINE) ? 1 : 10;
 
-    for(j = 0; j < N; j++) {
-      u1 = (double)j / (double)(N);
-      u2 = (double)(j + 1) / (double)(N);
-      P1 = InterpolateCurve(C, u1, 0);
-      P2 = InterpolateCurve(C, u2, 0);
+    for(int j = 0; j < N; j++) {
+      double u1 = (double)j / (double)(N);
+      double u2 = (double)(j + 1) / (double)(N);
+      Vertex P1 = InterpolateCurve(C, u1, 0);
+      Vertex P2 = InterpolateCurve(C, u2, 0);
       Angle += angle_plan(&V, &P1, &P2, n);
     }
 
   }
 
-  //printf(" %d -> angle %g\n", S->Num, fabs(Angle));
+  // if Angle == 2*pi, we're inside
 
-  if(fabs(Angle) > 6.0 && fabs(Angle) < 7.0)    // Should be 2 * Pi or 0
+  if(fabs(Angle) > 2*M_PI-0.5 && fabs(Angle) < 2*M_PI+0.5) 
     return 1;
   return 0;
 }
 
+void getPlaneSurfaceNormal(Surface *s, double x, double y, double z, double n[3])
+{
+  double t1[3], t2[3];
+  Curve *c;
+
+  for(int i = 0; i < 3; i++)
+    n[i] = 0.0;
+
+  if(s->Typ != MSH_SURF_PLAN)
+    return;
+
+  // We cannot use s->plan or InterpolateSurface here (they both rely
+  // on the mean plane, which borks the orientation). So we need to
+  // use this trick:
+
+  if(List_Nbr(s->Generatrices) > 1){
+    List_Read(s->Generatrices, 0, &c);
+    t1[0] = x - c->beg->Pos.X;
+    t1[1] = y - c->beg->Pos.Y;
+    t1[2] = z - c->beg->Pos.Z;
+    for(int i = 1; i < List_Nbr(s->Generatrices); i++){
+      List_Read(s->Generatrices, i, &c);
+      t2[0] = x - c->beg->Pos.X;
+      t2[1] = y - c->beg->Pos.Y;
+      t2[2] = z - c->beg->Pos.Z;
+      prodve(t1, t2, n);
+      if(norme(n))
+	break;
+    }
+  }
+  if(List_Nbr(s->Generatrices) < 2 || !norme(n)){
+    Msg(WARNING, "Reverting to mean plane normal for surface %d", s->Num);
+    n[0] = s->a;
+    n[1] = s->b;
+    n[2] = s->c;
+    norme(n);
+  }
+}
+
 void Draw_Triangulated_Surface(Surface * s)
 {
   int k = 0;
@@ -289,11 +334,11 @@ void Draw_Triangulated_Surface(Surface * s)
 	p1 = (double*)List_Pointer(s->thePolyRep->points_and_normals, 6*(int)points[1]);
 	p2 = (double*)List_Pointer(s->thePolyRep->points_and_normals, 6*(int)points[2]);
 	p3 = (double*)List_Pointer(s->thePolyRep->points_and_normals, 6*(int)points[3]);
-	glNormal3dv(&p1[3]);
+	if(CTX.geom.light) glNormal3dv(&p1[3]);
 	glVertex3d(p1[0],p1[1],p1[2]);
-	glNormal3dv(&p2[3]);
+	if(CTX.geom.light) glNormal3dv(&p2[3]);
 	glVertex3d(p2[0],p2[1],p2[2]);
-	glNormal3dv(&p3[3]);
+	if(CTX.geom.light) glNormal3dv(&p3[3]);
 	glVertex3d(p3[0],p3[1],p3[2]);
       }
     }
@@ -303,7 +348,6 @@ void Draw_Triangulated_Surface(Surface * s)
   }  
 }
 
-
 void Draw_Plane_Surface(Surface * s)
 {
   int i, j, k;
@@ -311,34 +355,25 @@ void Draw_Plane_Surface(Surface * s)
   double minx = 0., miny = 0., maxx = 0., maxy = 0., t, n[3];
   Vertex P1, P2, P3, V[4], vv, vv1, vv2;
   char Num[100];
+  const int numPoints = 100;
 
   if (s->thePolyRep) {
     Draw_Triangulated_Surface(s);
     return;
   }
 
-  static List_T *points;
-  static int deb = 1;
-
-  if(!s->Orientations) {
-
-    s->Orientations = List_Create(20, 2, sizeof(Vertex));
-
-    if(deb) {
-      points = List_Create(10, 10, sizeof(Vertex *));
-      deb = 0;
-    }
-    else
-      List_Reset(points);
+  if(!CTX.threads_lock && List_Nbr(s->Orientations) < 1) {
+    CTX.threads_lock = 1;
 
+    List_T *points = List_Create(10, 10, sizeof(Vertex *));
     for(i = 0; i < List_Nbr(s->Generatrices); i++) {
       List_Read(s->Generatrices, i, &c);
       for(j = 0; j < List_Nbr(c->Control_Points); j++) {
         List_Add(points, List_Pointer(c->Control_Points, j));
       }
     }
-
     MeanPlane(points, s);
+    List_Delete(points);
 
     k = 0;
 
@@ -372,7 +407,7 @@ void Draw_Plane_Surface(Surface * s)
 
     for(i = 0; i < 4; i++) {
       V[i].Pos.Z = 0.0;
-      put_Z(&V[i], s);
+      putZ(&V[i], s);
     }
 
     n[0] = s->plan[2][0];
@@ -381,8 +416,8 @@ void Draw_Plane_Surface(Surface * s)
     norme(n);
 
     k = 0;
-    for(i = 0; i < 100; i++) {
-      t = (double)i / (double)(100);
+    for(i = 0; i < numPoints; i++) {
+      t = (double)i / (double)(numPoints-1);
       vv.Pos.X = t * 0.5 * (V[0].Pos.X + V[1].Pos.X) + (1. - t) *
         0.5 * (V[2].Pos.X + V[3].Pos.X);
       vv.Pos.Y = t * 0.5 * (V[0].Pos.Y + V[1].Pos.Y) + (1. - t) *
@@ -407,8 +442,8 @@ void Draw_Plane_Surface(Surface * s)
       List_Add(s->Orientations, &vv);
 
     k = 0;
-    for(i = 0; i < 100; i++) {
-      t = (double)i / (double)(100);
+    for(i = 0; i < numPoints; i++) {
+      t = (double)i / (double)(numPoints-1);
       vv.Pos.X = t * .5 * (V[0].Pos.X + V[3].Pos.X) + 
 	(1. - t) * .5 * (V[2].Pos.X + V[1].Pos.X);
       vv.Pos.Y = t * .5 * (V[0].Pos.Y + V[3].Pos.Y) + 
@@ -431,26 +466,35 @@ void Draw_Plane_Surface(Surface * s)
     if(k)
       List_Add(s->Orientations, &vv);
 
-    Msg(STATUS2, "Plane Surface %d (%d points)", s->Num,
-        List_Nbr(s->Orientations));
+    Msg(STATUS2, "Plane Surface %d (%d points)", s->Num, List_Nbr(s->Orientations));
+
+    if(!List_Nbr(s->Orientations)) // add dummy
+      List_Add(s->Orientations, &vv);
+
+    CTX.threads_lock = 0;
   }
 
-  if(CTX.geom.surfaces) {
-    glBegin(GL_LINES);
-    for(i = 0; i < List_Nbr(s->Orientations); i++) {
-      List_Read(s->Orientations, i, &vv);
-      glVertex3d(vv.Pos.X, vv.Pos.Y, vv.Pos.Z);
+  if(List_Nbr(s->Orientations) > 1) {
+
+    if(CTX.geom.surfaces) {
+      glEnable(GL_LINE_STIPPLE);
+      glLineStipple(1, 0x0F0F);
+      gl2psEnable(GL2PS_LINE_STIPPLE);
+      glBegin(GL_LINES);
+      for(i = 0; i < List_Nbr(s->Orientations); i++) {
+	List_Read(s->Orientations, i, &vv);
+	glVertex3d(vv.Pos.X, vv.Pos.Y, vv.Pos.Z);
+      }
+      glEnd();
+      glDisable(GL_LINE_STIPPLE);
+      gl2psDisable(GL2PS_LINE_STIPPLE);
     }
-    glEnd();
-  }
-
-  if(List_Nbr(s->Orientations) > 1) {   //draw_surface can get called during the computation...
 
     if(CTX.geom.surfaces_num) {
       List_Read(s->Orientations, 0, &vv1);
       List_Read(s->Orientations, 1, &vv2);
       sprintf(Num, "%d", s->Num);
-      double offset = 0.5 * CTX.gl_fontsize * CTX.pixel_equiv_x;
+      double offset = 0.3 * CTX.gl_fontsize * CTX.pixel_equiv_x;
       glRasterPos3d((vv2.Pos.X + vv1.Pos.X) / 2. + offset / CTX.s[0],
                     (vv2.Pos.Y + vv1.Pos.Y) / 2. + offset / CTX.s[1],
                     (vv2.Pos.Z + vv1.Pos.Z) / 2. + offset / CTX.s[2]);
@@ -458,22 +502,19 @@ void Draw_Plane_Surface(Surface * s)
     }
 
     if(CTX.geom.normals) {
-      glDisable(GL_LINE_STIPPLE);
       List_Read(s->Orientations, 0, &vv1);
       List_Read(s->Orientations, 1, &vv2);
-      n[0] = s->plan[2][0];
-      n[1] = s->plan[2][1];
-      n[2] = s->plan[2][2];
-      norme(n);
+      double x = (vv1.Pos.X + vv2.Pos.X) / 2.;
+      double y = (vv1.Pos.Y + vv2.Pos.Y) / 2.;
+      double z = (vv1.Pos.Z + vv2.Pos.Z) / 2.;
+      getPlaneSurfaceNormal(s, x, y, z, n);
       n[0] *= CTX.geom.normals * CTX.pixel_equiv_x / CTX.s[0];
       n[1] *= CTX.geom.normals * CTX.pixel_equiv_x / CTX.s[1];
       n[2] *= CTX.geom.normals * CTX.pixel_equiv_x / CTX.s[2];
       glColor4ubv((GLubyte *) & CTX.color.geom.normals);
       Draw_Vector(CTX.vector_type, 0, CTX.arrow_rel_head_radius, 
 		  CTX.arrow_rel_stem_length, CTX.arrow_rel_stem_radius, 
-		  (vv2.Pos.X + vv1.Pos.X) / 2., (vv2.Pos.Y + vv1.Pos.Y) / 2., 
-		  (vv2.Pos.Z + vv1.Pos.Z) / 2., n[0], n[1], n[2], NULL, 
-		  CTX.geom.light);
+		  x, y, z, n[0], n[1], n[2], NULL, CTX.geom.light);
     }
 
   }
@@ -501,6 +542,9 @@ void Draw_NonPlane_Surface(Surface * s)
       glDisable(GL_LIGHTING);
     }
     else{
+      glEnable(GL_LINE_STIPPLE);
+      glLineStipple(1, 0x0F0F);
+      gl2psEnable(GL2PS_LINE_STIPPLE);
       int N = 50;
       glBegin(GL_LINE_STRIP);
       for(int i = 0; i < N + 1; i++) {
@@ -516,6 +560,8 @@ void Draw_NonPlane_Surface(Surface * s)
 	glVertex3d(v.Pos.X, v.Pos.Y, v.Pos.Z);
       }
       glEnd();
+      glDisable(GL_LINE_STIPPLE);
+      gl2psDisable(GL2PS_LINE_STIPPLE);
     }
   }
 
@@ -523,7 +569,7 @@ void Draw_NonPlane_Surface(Surface * s)
     Vertex v = InterpolateSurface(s, 0.5, 0.5, 0, 0);
     char Num[100];
     sprintf(Num, "%d", s->Num);
-    double offset = 0.5 * CTX.gl_fontsize * CTX.pixel_equiv_x;
+    double offset = 0.3 * CTX.gl_fontsize * CTX.pixel_equiv_x;
     glRasterPos3d(v.Pos.X + offset / CTX.s[0],
                   v.Pos.Y + offset / CTX.s[1],
                   v.Pos.Z + offset / CTX.s[2]);
@@ -531,18 +577,17 @@ void Draw_NonPlane_Surface(Surface * s)
   }
 
   if(CTX.geom.normals) {
-    glDisable(GL_LINE_STIPPLE);
-    Vertex n1 = InterpolateSurface(s, 0.5, 0.5, 0, 0);
-    Vertex n2 = InterpolateSurface(s, 0.6, 0.5, 0, 0);
-    Vertex n3 = InterpolateSurface(s, 0.5, 0.6, 0, 0);
-    double nx[3], ny[3], n[3];
-    nx[0] = n2.Pos.X - n1.Pos.X;
-    nx[1] = n2.Pos.Y - n1.Pos.Y;
-    nx[2] = n2.Pos.Z - n1.Pos.Z;
-    ny[0] = n3.Pos.X - n1.Pos.X;
-    ny[1] = n3.Pos.Y - n1.Pos.Y;
-    ny[2] = n3.Pos.Z - n1.Pos.Z;
-    prodve(nx, ny, n);
+    Vertex v1 = InterpolateSurface(s, 0.5, 0.5, 0, 0);
+    Vertex v2 = InterpolateSurface(s, 0.6, 0.5, 0, 0);
+    Vertex v3 = InterpolateSurface(s, 0.5, 0.6, 0, 0);
+    double t1[3], t2[3], n[3];
+    t1[0] = v2.Pos.X - v1.Pos.X;
+    t1[1] = v2.Pos.Y - v1.Pos.Y;
+    t1[2] = v2.Pos.Z - v1.Pos.Z;
+    t2[0] = v3.Pos.X - v1.Pos.X;
+    t2[1] = v3.Pos.Y - v1.Pos.Y;
+    t2[2] = v3.Pos.Z - v1.Pos.Z;
+    prodve(t1, t2, n);
     norme(n);
     n[0] *= CTX.geom.normals * CTX.pixel_equiv_x / CTX.s[0];
     n[1] *= CTX.geom.normals * CTX.pixel_equiv_x / CTX.s[1];
@@ -550,16 +595,14 @@ void Draw_NonPlane_Surface(Surface * s)
     glColor4ubv((GLubyte *) & CTX.color.geom.normals);
     Draw_Vector(CTX.vector_type, 0, CTX.arrow_rel_head_radius, 
 		CTX.arrow_rel_stem_length, CTX.arrow_rel_stem_radius,
-		n1.Pos.X, n1.Pos.Y, n1.Pos.Z, n[0], n[1], n[2], NULL,
+		v1.Pos.X, v1.Pos.Y, v1.Pos.Z, n[0], n[1], n[2], NULL,
 		CTX.geom.light);
   }
 }
 
 void Draw_Surface(void *a, void *b)
 {
-  Surface *s;
-
-  s = *(Surface **) a;
+  Surface *s = *(Surface **) a;
 
   if(!s || !s->Support || !(s->Visible & VIS_GEOM) || s->Dirty)
     return;
@@ -575,27 +618,17 @@ void Draw_Surface(void *a, void *b)
 		   CTX.print.eps_line_width_factor);
     glColor4ubv((GLubyte *) & CTX.color.geom.surface_sel);
   }
-  else if(Highlighted) {
-    glLineWidth(CTX.geom.line_sel_width);
-    gl2psLineWidth(CTX.geom.line_sel_width *
-		   CTX.print.eps_line_width_factor);
-    glColor4ubv((GLubyte *) & CTX.color.geom.surface_hlt);
-  }
   else {
     glLineWidth(CTX.geom.line_width);
     gl2psLineWidth(CTX.geom.line_width * CTX.print.eps_line_width_factor);
     glColor4ubv((GLubyte *) & CTX.color.geom.surface);
   }
-  glEnable(GL_LINE_STIPPLE);
-  glLineStipple(1, 0x0F0F);
 
   if(s->Typ == MSH_SURF_STL) {
-    glDisable(GL_LINE_STIPPLE);
-    Tree_Action(s->STL->Simplexes, Draw_Simplex_Surface);
+    Tree_Action(s->STL->Simplexes, Draw_Mesh_Triangle);
   }
   else if(s->Typ == MSH_SURF_DISCRETE) {
-    glDisable(GL_LINE_STIPPLE);
-    Tree_Action(s->Simplexes, Draw_Simplex_Surface);
+    Tree_Action(s->Simplexes, Draw_Mesh_Triangle);
   }
   else if(s->Typ == MSH_SURF_PLAN)
     Draw_Plane_Surface(s);
@@ -605,55 +638,10 @@ void Draw_Surface(void *a, void *b)
   if(CTX.render_mode == GMSH_SELECT) {
     glPopName();
   }
-
-  glDisable(GL_LINE_STIPPLE);
 }
 
 // Volumes
 
-int TheVolume;
-
-void Draw_Curve_For_Volume(void *a, void *b)
-{
-  int i, N;
-  Curve *c;
-
-  glLineWidth(CTX.geom.line_width);
-  gl2psLineWidth(CTX.geom.line_width * CTX.print.eps_line_width_factor);
-
-  c = *(Curve **) a;
-
-  if(CTX.render_mode == GMSH_SELECT) {
-    glLoadName(3);
-    glPushName(TheVolume);
-  }
-
-  if(c->Typ == MSH_SEGM_LINE)
-    N = List_Nbr(c->Control_Points);
-  else
-    N = 10;
-
-  glBegin(GL_LINE_STRIP);
-  for(i = 0; i < N; i++) {
-    Vertex v = InterpolateCurve(c, 0.2 * (double)i / (double)(N - 1), 0);
-    glVertex3d(v.Pos.X, v.Pos.Y, v.Pos.Z);
-  }
-  glEnd();
-
-  glBegin(GL_LINE_STRIP);
-  for(i = N - 1; i >= 0; i--) {
-    Vertex v = InterpolateCurve(c, 1. - 0.2 * (double)i / (double)(N - 1), 0);
-    glVertex3d(v.Pos.X, v.Pos.Y, v.Pos.Z);
-  }
-  glEnd();
-
-
-  if(CTX.render_mode == GMSH_SELECT) {
-    glPopName();
-  }
-}
-
-
 void DrawVolumes(Mesh * m)
 {
 }
@@ -666,7 +654,7 @@ void Draw_Geom(Mesh * m)
     return;
 
   if(CTX.geom.points || CTX.geom.points_num)
-    Tree_Action(m->Points, Draw_GeoPoint);
+    Tree_Action(m->Points, Draw_Geo_Point);
   if(CTX.geom.lines || CTX.geom.lines_num || CTX.geom.tangents)
     Tree_Action(m->Curves, Draw_Curve);
   if(CTX.geom.surfaces || CTX.geom.surfaces_num || CTX.geom.normals)
@@ -675,21 +663,103 @@ void Draw_Geom(Mesh * m)
     DrawVolumes(m);
 }
 
-void ZeroCurve(void *a, void *b)
+// Highlight routines
+
+void HighlightEntity(Vertex * v, Curve * c, Surface * s, int permanent)
 {
-  Curve *c;
-  c = *(Curve **) a;
-  c->ipar[3] = 0;
+  Curve *cc;
+  char Message[256], temp[256];
+
+  if(permanent){
+    // we want to draw incrementally (in-between to "Draw()" calls):
+    // we need to make sure that the opengl context is set correctly
+    SetOpenglContext();
+  }
+
+  if(v) {
+    if(permanent){
+      v->Frozen = 1;
+      Draw_Geo_Point(&v,NULL);
+    }
+    else{
+      Msg(STATUS1N, "Point %d {%.5g,%.5g,%.5g} (%.5g)", v->Num, v->Pos.X,
+	  v->Pos.Y, v->Pos.Z, v->lc);
+    }
+  }
+  else if(c) {
+    if(permanent){
+      c->ipar[3] = 1;
+      Draw_Curve(&c,NULL);
+    }
+    else{
+      Msg(STATUS1N, "Curve %d  {%d->%d}", c->Num, c->beg->Num, c->end->Num);
+    }
+  }
+  else if(s) {
+    if(permanent){
+      s->ipar[4] = 1;
+      Draw_Surface(&s,NULL);
+    }
+    else{
+      sprintf(Message, "Surface %d {", s->Num);
+      int nbg = List_Nbr(s->Generatrices);
+      if(nbg < 10) {
+	for(int i = 0; i < nbg; i++) {
+	  List_Read(s->Generatrices, i, &cc);
+	  if(!i)
+	    sprintf(temp, "%d", cc->Num);
+	  else
+	    sprintf(temp, ",%d", cc->Num);
+	  strcat(Message, temp);
+	}
+      }
+      else {
+	strcat(Message, "...");
+      }
+      strcat(Message, "}");
+      Msg(STATUS1N, Message);
+    }
+  }
+  else{
+    if(!permanent)
+      Msg(STATUS1N, " ");
+  }
 }
 
-void ZeroPoint(void *a, void *b)
+void HighlightEntityNum(int v, int c, int s, int permanent)
+{
+  if(v) {
+    Vertex *pv = FindPoint(v, THEM);
+    if(pv)
+      HighlightEntity(pv, NULL, NULL, permanent);
+  }
+  if(c) {
+    Curve *pc = FindCurve(c, THEM);
+    if(pc)
+      HighlightEntity(NULL, pc, NULL, permanent);
+  }
+  if(s) {
+    Surface *ps = FindSurface(s, THEM);
+    if(ps)
+      HighlightEntity(NULL, NULL, ps, permanent);
+  }
+}
+
+void ZeroHighlightPoint(void *a, void *b)
 {
   Vertex *v;
   v = *(Vertex **) a;
   v->Frozen = 0;
 }
 
-void ZeroSurface(void *a, void *b)
+void ZeroHighlightCurve(void *a, void *b)
+{
+  Curve *c;
+  c = *(Curve **) a;
+  c->ipar[3] = 0;
+}
+
+void ZeroHighlightSurface(void *a, void *b)
 {
   Surface *s;
   s = *(Surface **) a;
@@ -698,118 +768,26 @@ void ZeroSurface(void *a, void *b)
 
 void ZeroHighlight(Mesh * m)
 {
-  Tree_Action(m->Points, ZeroPoint);
-  Tree_Action(m->Curves, ZeroCurve);
-  Tree_Action(m->Surfaces, ZeroSurface);
+  Tree_Action(m->Points, ZeroHighlightPoint);
+  Tree_Action(m->Curves, ZeroHighlightCurve);
+  Tree_Action(m->Surfaces, ZeroHighlightSurface);
 }
 
-// Highlight routines
-
-void BeginHighlight(void)
+void ZeroHighlightEntityNum(int v, int c, int s)
 {
-  if(CTX.geom.highlight) {
-    if(CTX.overlay)
-      InitOverlay();
-    else
-      InitOpengl();
-    Highlighted = 1;
-    glPushMatrix();
-    InitPosition();
-  }
-}
-
-
-void EndHighlight(int permanent)
-{
-  Highlighted = 0;
-  if(permanent)
-    Draw();
-  else {
-    if(CTX.geom.highlight) {
-      glPopMatrix();
-    }
-  }
-}
-
-void HighlightEntity(Vertex * v, Curve * c, Surface * s, int permanent)
-{
-  Curve *cc;
-  char Message[256], temp[256];
-  int i, nbg;
-
   if(v) {
-    if(permanent)
-      v->Frozen = 1;
-    if(CTX.geom.highlight)
-      Draw_GeoPoint(&v, NULL);
-    Msg(STATUS1N, "Point %d {%.5g,%.5g,%.5g} (%.5g)", v->Num, v->Pos.X,
-        v->Pos.Y, v->Pos.Z, v->lc);
-  }
-  else if(c) {
-    if(permanent)
-      c->ipar[3] = 1;
-    if(CTX.geom.highlight)
-      Draw_Curve(&c, NULL);
-    Msg(STATUS1N, "Curve %d  {%d->%d}", c->Num, c->beg->Num, c->end->Num);
-  }
-  else if(s) {
-    if(permanent && s->ipar[4] == 1)
-      return;
-    if(permanent)
-      s->ipar[4] = 1;
-    if(CTX.geom.highlight)
-      Draw_Surface(&s, NULL);
-    sprintf(Message, "Surface %d {", s->Num);
-
-    nbg = List_Nbr(s->Generatrices);
-
-    if(nbg < 10) {
-      for(i = 0; i < nbg; i++) {
-        List_Read(s->Generatrices, i, &cc);
-        if(!i)
-          sprintf(temp, "%d", cc->Num);
-        else
-          sprintf(temp, ",%d", cc->Num);
-        strcat(Message, temp);
-      }
-    }
-    else {
-      strcat(Message, "...");
-    }
-    strcat(Message, "}");
-    Msg(STATUS1N, Message);
-  }
-  else {
-    Msg(STATUS1N, " ");
-  }
-  glFlush();
-}
-
-
-void HighlightEntityNum(int v, int c, int s, int permanant)
-{
-  Vertex *pv, V;
-  Curve *pc, C;
-  Surface *ps, S;
-  if(v) {
-    pv = &V;
-    pv->Num = v;
-    if(Tree_Query(THEM->Vertices, &pv)) {
-      HighlightEntity(pv, NULL, NULL, permanant);
-    }
+    Vertex *pv = FindPoint(v, THEM);
+    if(pv)
+      ZeroHighlightPoint(&pv, NULL);
   }
   if(c) {
-    pc = &C;
-    pc->Num = c;
-    if(Tree_Query(THEM->Curves, &pc)) {
-      HighlightEntity(NULL, pc, NULL, permanant);
-    }
+    Curve *pc = FindCurve(c, THEM);
+    if(pc)
+      ZeroHighlightCurve(&pc, NULL);
   }
   if(s) {
-    ps = &S;
-    ps->Num = s;
-    if(Tree_Query(THEM->Surfaces, &ps)) {
-      HighlightEntity(NULL, NULL, ps, permanant);
-    }
+    Surface *ps = FindSurface(s, THEM);
+    if(ps)
+      ZeroHighlightSurface(&ps, NULL);
   }
 }
