@@ -1,4 +1,4 @@
-// $Id: Graph2D.cpp,v 1.35 2004/04/24 05:31:13 geuzaine Exp $
+// $Id: Graph2D.cpp,v 1.37 2004/10/11 19:18:59 geuzaine Exp $
 //
 // Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
 //
@@ -106,16 +106,6 @@ static void Draw_Graph2D(Post_View * v,
   double Abs, Val, ValMin = 0., ValMax = 0., AbsMin, AbsMax;
   double p1[3], p2[3];
 
-  if(!v->TransparentScale) {
-    glColor4ubv((GLubyte *) & CTX.color.bg);
-    glBegin(GL_QUADS);
-    glVertex2d(bb[0], CTX.viewport[3] - bb[1]);
-    glVertex2d(bb[2], CTX.viewport[3] - bb[1]);
-    glVertex2d(bb[2], CTX.viewport[3] - bb[3]);
-    glVertex2d(bb[0], CTX.viewport[3] - bb[3]);
-    glEnd();
-  }
-
   gl_font(CTX.gl_font_enum, CTX.gl_fontsize);
   font_h = gl_height(); // total font height
   font_a = gl_height() - gl_descent();  // height above ref pt
@@ -130,8 +120,14 @@ static void Draw_Graph2D(Post_View * v,
     ValMax = v->CustomMax;
     break;
   case DRAW_POST_RANGE_PER_STEP:
-    ValMin = v->TimeStepMin[v->TimeStep];
-    ValMax = v->TimeStepMax[v->TimeStep];
+    if(v->TimeStepMin && v->TimeStepMax){
+      ValMin = v->TimeStepMin[v->TimeStep];
+      ValMax = v->TimeStepMax[v->TimeStep];
+    }
+    else{ // possible if the view is empty
+      ValMin = v->Min;
+      ValMax = v->Max;
+    }
     break;
   }
 
