@@ -1,4 +1,4 @@
-// $Id: Iso.cpp,v 1.22 2004/04/27 00:11:55 geuzaine Exp $
+// $Id: Iso.cpp,v 1.30 2004/08/07 06:59:16 geuzaine Exp $
 //
 // Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
 //
@@ -30,32 +30,46 @@
 
 extern Context_T CTX;
 
-// Iso line computation for triangles.
+// Draw an iso-line inside a triangle
 
-void CutTriangle1D(double *X, double *Y, double *Z, double *Val,
-                   double V, double *Xp, double *Yp, double *Zp, int *nb)
+void IsoTriangle(Post_View * View, double *X, double *Y, double *Z,
+		 double *Val, double V)
 {
-  *nb = 0;
+  // don't draw anything if the value is constant
+  if(Val[0] == Val[1] && Val[0] == Val[2])
+    return;
+
+  double Xp[3], Yp[3], Zp[3];
+  int nb = 0;
   if((Val[0] >= V && Val[1] <= V) || (Val[1] >= V && Val[0] <= V)) {
-    InterpolateIso(X, Y, Z, Val, V, 0, 1, &Xp[*nb], &Yp[*nb], &Zp[*nb]);
-    (*nb)++;
+    InterpolateIso(X, Y, Z, Val, V, 0, 1, &Xp[nb], &Yp[nb], &Zp[nb]);
+    nb++;
   }
   if((Val[0] >= V && Val[2] <= V) || (Val[2] >= V && Val[0] <= V)) {
-    InterpolateIso(X, Y, Z, Val, V, 0, 2, &Xp[*nb], &Yp[*nb], &Zp[*nb]);
-    (*nb)++;
+    InterpolateIso(X, Y, Z, Val, V, 0, 2, &Xp[nb], &Yp[nb], &Zp[nb]);
+    nb++;
   }
   if((Val[1] >= V && Val[2] <= V) || (Val[2] >= V && Val[1] <= V)) {
-    InterpolateIso(X, Y, Z, Val, V, 1, 2, &Xp[*nb], &Yp[*nb], &Zp[*nb]);
-    (*nb)++;
+    InterpolateIso(X, Y, Z, Val, V, 1, 2, &Xp[nb], &Yp[nb], &Zp[nb]);
+    nb++;
+  }
+  
+  if(nb == 2) {
+    double Raise[3][8];
+    for(int i = 0; i < 3; i++)
+      for(int l = 0; l < 2; l++)
+	Raise[i][l] = View->Raise[i] * V;
+    Draw_Line(View->LineType, View->LineWidth, Xp, Yp, Zp,
+	      Raise, View->Light);
   }
-
 }
 
-// Contour computation for triangles
+// Compute the polygon between the two iso-lines V1 and V2 in a
+// triangle
 
-void CutTriangle2D(double *X, double *Y, double *Z, double *Val,
-                   double V1, double V2, double *Xp2, double *Yp2,
-                   double *Zp2, int *Np2, double *Vp2)
+void CutTriangle(double *X, double *Y, double *Z, double *Val,
+		 double V1, double V2, double *Xp2, double *Yp2,
+		 double *Zp2, int *Np2, double *Vp2)
 {
   int i, io[3], j, iot, Np, Fl;
   double Xp[10], Yp[10], Zp[10], Vp[10];
@@ -211,22 +225,31 @@ void CutTriangle2D(double *X, double *Y, double *Z, double *Val,
   }
 }
 
-// Iso for lines
+// Draw an iso-point in a line
 
-void CutLine0D(double *X, double *Y, double *Z, double *Val,
-               double V, double *Xp, double *Yp, double *Zp, int *nb)
+void IsoLine(Post_View *View, double *X, double *Y, double *Z, 
+	     double *Val, double V)
 {
-  *nb = 0;
+  double Xp[2], Yp[2], Zp[2];
+
+  // don't draw anything if the value is constant
+  if(Val[0] == Val[1])
+    return;
 
   if((Val[0] >= V && Val[1] <= V) || (Val[1] >= V && Val[0] <= V)) {
     InterpolateIso(X, Y, Z, Val, V, 0, 1, Xp, Yp, Zp);
-    *nb = 1;
+    double Raise[3][8];
+    for(int i = 0; i < 3; i++)
+      Raise[i][0] = View->Raise[i] * V;
+    Draw_Point(View->PointType, View->PointSize, Xp, Yp, Zp, Raise, View->Light);
   }
 }
 
-void CutLine1D(double *X, double *Y, double *Z, double *Val,
-               double V1, double V2, double *Xp2, double *Yp2, double *Zp2,
-               int *Np2, double *Vp2)
+// Compute the line between the two iso-points V1 and V2 in a line
+
+void CutLine(double *X, double *Y, double *Z, double *Val,
+	     double V1, double V2, double *Xp2, double *Yp2, double *Zp2,
+	     int *Np2, double *Vp2)
 {
   int i, io[2];
 
@@ -281,10 +304,9 @@ void CutLine1D(double *X, double *Y, double *Z, double *Val,
     Vp2[1] = V2;
     InterpolateIso(X, Y, Z, Val, V2, io[0], io[1], &Xp2[1], &Yp2[1], &Zp2[1]);
   }
-
 }
 
-// compute the gradient of a linear interpolation in a tetrahedron
+// Compute the gradient of a linear interpolation in a tetrahedron
 
 void EnhanceSimplexPolygon(Post_View * View, int nb,    // nb of points in polygon 
                            double *Xp,  // x positions
@@ -328,11 +350,6 @@ void EnhanceSimplexPolygon(Post_View * View, int nb,    // nb of points in polyg
      v = unknown field we wanna draw
    */
 
-  if(!View->Light) {
-    norms = NULL;       // we don't need to compute these
-    return;
-  }
-
   double v1[3] = { Xp[2] - Xp[0], Yp[2] - Yp[0], Zp[2] - Zp[0] };
   double v2[3] = { Xp[1] - Xp[0], Yp[1] - Yp[0], Zp[1] - Zp[0] };
   double gr[3];
@@ -363,7 +380,7 @@ void EnhanceSimplexPolygon(Post_View * View, int nb,    // nb of points in polyg
   if(View->SmoothNormals) {
     if(preproNormals) {
       for(i = 0; i < nb; i++) {
-        View->add_normal(Xp[i], Yp[i], Zp[i], n[0], n[1], n[2]);
+        View->normals->add(Xp[i], Yp[i], Zp[i], n[0], n[1], n[2]);
       }
       return;
     }
@@ -372,11 +389,9 @@ void EnhanceSimplexPolygon(Post_View * View, int nb,    // nb of points in polyg
         norms[3 * i] = n[0];
         norms[3 * i + 1] = n[1];
         norms[3 * i + 2] = n[2];
-        if(!View->
-           get_normal(Xp[i], Yp[i], Zp[i], norms[3 * i], norms[3 * i + 1],
-                      norms[3 * i + 2])) {
-          //Msg(WARNING, "Oups, did not find smoothed normal");
-        }
+        View->normals->get(Xp[i], Yp[i], Zp[i], 
+			   norms[3 * i], norms[3 * i + 1], norms[3 * i + 2],
+			   View->AngleSmoothNormals);
       }
     }
   }
@@ -387,18 +402,30 @@ void EnhanceSimplexPolygon(Post_View * View, int nb,    // nb of points in polyg
       norms[3 * i + 2] = n[2];
     }
   }
-
 }
 
-void IsoSimplex(Post_View * View,
-                int preproNormals,
+static void affect(double *xi, double *yi, double *zi, int i,
+		   double *xp, double *yp, double *zp, int j)
+{
+  xi[i] = xp[j];
+  yi[i] = yp[j];
+  zi[i] = zp[j];
+}
+
+// Draw an iso-surface inside a tetrahedron
+
+void IsoSimplex(Post_View * View, int preproNormals,
                 double *X, double *Y, double *Z, double *Val,
-                double V)
+                double V, unsigned int color)
 {
   int nb;
   double Xp[6], Yp[6], Zp[6], PVals[6];
   double norms[12];
 
+  // don't draw anything if the value is constant
+  if(Val[0] == Val[1] && Val[0] == Val[2] && Val[0] == Val[3])
+    return;
+
   nb = 0;
   if((Val[0] >= V && Val[1] <= V) || (Val[1] >= V && Val[0] <= V)) {
     InterpolateIso(X, Y, Z, Val, V, 0, 1, &Xp[nb], &Yp[nb], &Zp[nb]);
@@ -425,6 +452,33 @@ void IsoSimplex(Post_View * View,
     nb++;
   }
 
+  // Remove identical nodes (this can happen if an edge actually
+  // belongs to the zero levelset). We should be doing this even for
+  // nb < 4, but it would slow us down even more... (And we don't
+  // really care if some nodes in a postprocessing element are
+  // identical.)
+  if(nb > 4) {
+    double xi[6], yi[6], zi[6];
+    affect(xi, yi, zi, 0, Xp, Yp, Zp, 0);
+    int ni = 1;
+    for(int j = 1; j < nb; j++) {
+      for(int i = 0; i < ni; i++) {
+	if(fabs(Xp[j] - xi[i]) < 1.e-12 &&
+	   fabs(Yp[j] - yi[i]) < 1.e-12 &&
+	   fabs(Zp[j] - zi[i]) < 1.e-12) {
+	  break;
+	}
+	if(i == ni-1) {
+	  affect(xi, yi, zi, i+1, Xp, Yp, Zp, j);
+	  ni++;
+	}
+      }
+    }
+    for(int i = 0; i < ni; i++)
+      affect(Xp, Yp, Zp, i, xi, yi, zi, i);
+    nb = ni;
+  }
+
   if(nb < 3 || nb > 4)
     return;
 
@@ -434,13 +488,42 @@ void IsoSimplex(Post_View * View,
   if(preproNormals)
     return;
 
-  double Raise[3][8];
-  for(int i = 0; i < 3; i++)
-    for(int k = 0; k < nb; k++)
-      Raise[i][k] = View->Raise[i] * V;
-
-  if(nb == 3)
-    Draw_Triangle(Xp, Yp, Zp, norms, Raise, View->Light);
-  else if(nb == 4)
-    Draw_Quadrangle(Xp, Yp, Zp, norms, Raise, View->Light);
+  if(View->TriVertexArray && View->TriVertexArray->fill){
+    for(int i = 2; i < nb; i++){
+      View->TriVertexArray->add(Xp[0] + View->Raise[0] * V,
+				Yp[0] + View->Raise[1] * V,
+				Zp[0] + View->Raise[2] * V,
+				norms[0], norms[1], norms[2], color);
+      View->TriVertexArray->add(Xp[i-1] + View->Raise[0] * V,
+				Yp[i-1] + View->Raise[1] * V,
+				Zp[i-1] + View->Raise[2] * V,
+				norms[3*(i-1)], norms[3*(i-1)+1], norms[3*(i-1)+2], color);
+      View->TriVertexArray->add(Xp[i] + View->Raise[0] * V,
+				Yp[i] + View->Raise[1] * V,
+				Zp[i] + View->Raise[2] * V,
+				norms[3*i], norms[3*i+1], norms[3*i+2], color);
+      View->TriVertexArray->num++;
+    }
+  }
+  else{
+    if(View->Light) glEnable(GL_LIGHTING);
+    glColor4ubv((GLubyte *) & color);
+    glBegin(GL_TRIANGLES);
+    for(int i = 2; i < nb; i++){
+      if(View->Light) glNormal3dv(&norms[0]);
+      glVertex3d(Xp[0] + View->Raise[0] * V, 
+		 Yp[0] + View->Raise[1] * V, 
+		 Zp[0] + View->Raise[2] * V);
+      if(View->Light) glNormal3dv(&norms[3*(i-1)]);
+      glVertex3d(Xp[i-1] + View->Raise[0] * V, 
+		 Yp[i-1] + View->Raise[1] * V, 
+		 Zp[i-1] + View->Raise[2] * V);
+      if(View->Light) glNormal3dv(&norms[3*i]);
+      glVertex3d(Xp[i] + View->Raise[0] * V, 
+		 Yp[i] + View->Raise[1] * V, 
+		 Zp[i] + View->Raise[2] * V);
+    }
+    glEnd();
+    glDisable(GL_LIGHTING);
+  }
 }
