@@ -1,4 +1,4 @@
-// $Id: 2D_Mesh.cpp,v 1.54 2004/04/18 03:36:07 geuzaine Exp $
+// $Id: 2D_Mesh.cpp,v 1.68 2004/07/14 22:42:26 geuzaine Exp $
 //
 // Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
 //
@@ -28,15 +28,16 @@
 #include "Create.h"
 #include "2D_Mesh.h"
 #include "Context.h"
+#include "Interpolation.h"
 
 extern Mesh *THEM;
 extern Context_T CTX;
 
 PointRecord *gPointArray;
 DocRecord *BGMESH, *FGMESH;
-int is_3D = 0;
 double LC2D;
 
+static int is_3D = 0;
 static Surface *THESURFACE, *THESUPPORT;
 
 void ProjetteSurface(void *a, void *b)
@@ -302,13 +303,13 @@ int mesh_domain(ContourPeek * ListContours, int numcontours,
   Conversion(doc);
   remove_all_dlist(doc->numPoints, doc->points);
 
-  if(!is_3D || CTX.mesh.constrained_bgmesh) {
+  if(!is_3D || CTX.mesh.constrained_bgmesh)
     BGMESH = doc;
-    InitBricks(BGMESH);
-  }
   else
     BGMESH = NULL;
 
+  InitBricks(BGMESH);
+
   /* elimination des triangles exterieurs + verification de l'existence
      des edges (coherence) */
 
@@ -714,12 +715,10 @@ void filldel(Delaunay * deladd, int aa, int bb, int cc,
   deladd->v.voisin2 = NULL;
   deladd->v.voisin3 = NULL;
 
-  CircumCircle(points[aa].where.h,
-               points[aa].where.v,
-               points[bb].where.h,
-               points[bb].where.v,
-               points[cc].where.h,
-               points[cc].where.v, &deladd->t.xc, &deladd->t.yc);
+  CircumCircle(points[aa].where.h, points[aa].where.v,
+               points[bb].where.h, points[bb].where.v,
+               points[cc].where.h, points[cc].where.v, 
+	       &deladd->t.xc, &deladd->t.yc);
 
   pt2.h = deladd->t.xc;
   pt2.v = deladd->t.yc;
@@ -772,11 +771,78 @@ void ActionEndTheCurve(void *a, void *b)
   End_Curve(c);
 }
 
+void ActionInvertTri(void *a, void *b)
+{
+  Simplex *s = *(Simplex **) a;
+  Vertex *tmp = s->V[1];
+  s->V[1] = s->V[2];
+  s->V[2] = tmp;
+}
+
+void ActionInvertQua(void *a, void *b)
+{
+  Quadrangle *q = *(Quadrangle **) a;
+  Vertex *tmp = q->V[1];
+  q->V[1] = q->V[3];
+  q->V[3] = tmp;
+}
+
+// A little routine that re-orients the surface meshes so that they
+// match the geometrical orientations. It would be better to generate
+// the meshes correctly in the first place (!), but it's actually
+// pretty nice to do it here since we use many algorithms (some
+// closed) and since the mean plane computation doesn't take the
+// original orientation into account.
+
+void ReOrientSurfaceMesh(Surface *s)
+{
+  if(!List_Nbr(s->Generatrices))
+    return;
+
+  Curve *c;
+  List_Read(s->Generatrices, 0, &c);
+
+  if(List_Nbr(c->Vertices) < 2)
+    return;
+
+  // take the first edge (even if there are holes, we know that this
+  // edge belongs to the exterior loop, due to the ordering of
+  // s->Generatrices)
+  Vertex *v1, *v2;
+  List_Read(c->Vertices, 0, &v1);
+  List_Read(c->Vertices, 1, &v2);
+  Edge edge;
+  edge.V[0] = v1;
+  edge.V[1] = v2;
+
+  if(Tree_Nbr(s->Simplexes)){
+    EdgesContainer edges;
+    edges.AddSimplexTree(s->Simplexes);
+    Edge *e = (Edge*)Tree_PQuery(edges.AllEdges, &edge);
+    if(e && e->V[0]->Num == v2->Num && e->V[1]->Num == v1->Num){
+      Tree_Action(s->Simplexes, ActionInvertTri);
+      if(Tree_Nbr(s->Quadrangles)) // for partially recombined meshes
+	Tree_Action(s->Quadrangles, ActionInvertQua);
+    }
+  }
+
+  if(Tree_Nbr(s->Quadrangles)){
+    EdgesContainer edges;
+    edges.AddQuadrangleTree(s->Quadrangles);
+    Edge *e = (Edge*)Tree_PQuery(edges.AllEdges, &edge);
+    if(e && e->V[0]->Num == v2->Num && e->V[1]->Num == v1->Num){
+      Tree_Action(s->Quadrangles, ActionInvertQua);
+      if(Tree_Nbr(s->Simplexes)) // for partially recombined meshes
+	Tree_Action(s->Simplexes, ActionInvertTri);
+    }
+  }
+}
+
 void Maillage_Surface(void *data, void *dum)
 {
   Surface **pS, *s;
   Tree_T *tnxe;
-  int ori, nbqua = 0;
+  int ori;
 
   pS = (Surface **) data;
   s = *pS;
@@ -804,86 +870,77 @@ void Maillage_Surface(void *data, void *dum)
     Tree_Delete(s->Vertices);
   s->Vertices = Tree_Create(sizeof(Vertex *), compareVertex);
 
-  Msg(STATUS3, "Meshing Surface %d", s->Num);
+  Msg(STATUS3, "Meshing surface %d", s->Num);
 
   if(MeshTransfiniteSurface(s) ||
      MeshEllipticSurface(s) ||
      MeshCylindricalSurface(s) ||
-     MeshParametricSurface(s) || Extrude_Mesh(s)) {
+     MeshParametricSurface(s) ||
+     Extrude_Mesh(s)) {
     Tree_Action(THEM->Points, PutVertex_OnSurf);
     Tree_Action(s->Vertices, PutVertex_OnSurf);
     Tree_Action(s->Vertices, Add_In_Mesh);
-    THEM->Statistics[5] += Tree_Nbr(THESURFACE->Vertices);
-    THEM->Statistics[7] += Tree_Nbr(THESURFACE->Simplexes);
-
-    /* void TRIE_MON_GARS(void *a, void *b);
-       Tree_Action (THES->Simplexes, TRIE_MON_GARS);
-       Link_Simplexes(NULL, THES->Simplexes);
-       void  constraint_the_edge (int ,int ,int);
-       constraint_the_edge (6, 45, 85);
-     */
-    return;
   }
+  else{
+    int TypSurface = s->Typ;
+    Plan_Moyen(pS, dum);
+    Tree_Action(THEM->Points, Freeze_Vertex);
+    Tree_Action(s->Vertices, Freeze_Vertex);
+    Tree_Action(THEM->Points, Projette_Plan_Moyen);
+    Tree_Action(s->Vertices, Projette_Plan_Moyen);
+    Tree_Action(THEM->Curves, ActionEndTheCurve);
+    s->Typ = MSH_SURF_PLAN;
+    End_Surface(s, 0);
 
-  int TypSurface = s->Typ;
-  Plan_Moyen(pS, dum);
-  Tree_Action(THEM->Points, Freeze_Vertex);
-  Tree_Action(s->Vertices, Freeze_Vertex);
-  Tree_Action(THEM->Points, Projette_Plan_Moyen);
-  Tree_Action(s->Vertices, Projette_Plan_Moyen);
-  Tree_Action(THEM->Curves, ActionEndTheCurve);
-  s->Typ = MSH_SURF_PLAN;
-  End_Surface(s);
+    ori = Calcule_Contours(s);
+    
+    if(CTX.mesh.algo2d == DELAUNAY_ISO)
+      Maillage_Automatique_VieuxCode(s, THEM, ori);
+    else if(CTX.mesh.algo2d == DELAUNAY_ANISO)
+      AlgorithmeMaillage2DAnisotropeModeJF(s);
+    else
+      Mesh_Triangle(s);
+    
+    if(CTX.mesh.nb_smoothing) {
+      Msg(STATUS3, "Smoothing surface %d", s->Num);
+      tnxe = Tree_Create(sizeof(NXE), compareNXE);
+      create_NXE(s->Vertices, s->Simplexes, tnxe);
+      for(int i = 0; i < CTX.mesh.nb_smoothing; i++)
+	Tree_Action(tnxe, ActionLiss);
+      delete_NXE(tnxe);
+    }
+    
+    if(s->Recombine){
+      Msg(STATUS3, "Recombining surface %d", s->Num);
+      Recombine(s->Vertices, s->Simplexes, s->Quadrangles, s->RecombineAngle);
+    }
 
-  ori = Calcule_Contours(s);
+    s->Typ = TypSurface;
+    
+    if(s->Typ != MSH_SURF_PLAN) {
+      if(s->Extrude)
+	s->Extrude->Rotate(s->plan);
+      Tree_Action(s->Vertices, Calcule_Z);
+      if(s->Extrude)
+	s->Extrude->Rotate(s->invplan);
+    }
+    else
+      Tree_Action(s->Vertices, Calcule_Z_Plan);
 
-  if(CTX.mesh.algo == DELAUNAY_ISO)
-    Maillage_Automatique_VieuxCode(s, THEM, ori);
-  else if(CTX.mesh.algo == DELAUNAY_ANISO)
-    AlgorithmeMaillage2DAnisotropeModeJF(s);
-  else
-    Mesh_Shewchuk(s);
-
-  if(CTX.mesh.nb_smoothing) {
-    Msg(STATUS3, "Mesh smoothing");
-    tnxe = Tree_Create(sizeof(NXE), compareNXE);
-    create_NXE(s->Vertices, s->Simplexes, tnxe);
-    for(int i = 0; i < CTX.mesh.nb_smoothing; i++)
-      Tree_Action(tnxe, ActionLiss);
-    delete_NXE(tnxe);
+    Tree_Action(s->Vertices, Projette_Inverse);
+    Tree_Action(THEM->Points, Projette_Inverse);
+    
+    Tree_Action(THEM->Points, deFreeze_Vertex);
+    Tree_Action(s->Vertices, deFreeze_Vertex);
+    
+    Tree_Action(THEM->Points, PutVertex_OnSurf);
+    Tree_Action(s->Vertices, PutVertex_OnSurf);
+    Tree_Action(s->Vertices, Add_In_Mesh);
+    
+    Tree_Action(THEM->Curves, ActionEndTheCurve);
+    End_Surface(s->Support, 0);
+    End_Surface(s, 0);
+    
+    ReOrientSurfaceMesh(s);
   }
-
-
-  if(s->Recombine)
-    nbqua = Recombine(s->Vertices, s->Simplexes, s->RecombineAngle);
-
-  s->Typ = TypSurface;
-
-  if(s->Typ != MSH_SURF_PLAN) {
-    if(s->Extrude)
-      s->Extrude->Rotate(s->plan);
-    Tree_Action(s->Vertices, Calcule_Z);
-    if(s->Extrude)
-      s->Extrude->Rotate(s->invplan);
-  }
-  else
-    Tree_Action(s->Vertices, Calcule_Z_Plan);
-
-  Tree_Action(s->Vertices, Projette_Inverse);
-  Tree_Action(THEM->Points, Projette_Inverse);
-
-  Tree_Action(THEM->Points, deFreeze_Vertex);
-  Tree_Action(s->Vertices, deFreeze_Vertex);
-
-  Tree_Action(THEM->Points, PutVertex_OnSurf);
-  Tree_Action(s->Vertices, PutVertex_OnSurf);
-  Tree_Action(s->Vertices, Add_In_Mesh);
-
-  Tree_Action(THEM->Curves, ActionEndTheCurve);
-  End_Surface(s->Support);
-  End_Surface(s);
-
-  THEM->Statistics[5] += Tree_Nbr(THESURFACE->Vertices);
-  THEM->Statistics[7] += Tree_Nbr(THESURFACE->Simplexes);       // tri+qua
-  THEM->Statistics[8] += nbqua;
 }
