@@ -1,4 +1,4 @@
-// $Id: 2D_Mesh_Aniso.cpp,v 1.37 2004/03/23 02:14:34 geuzaine Exp $
+// $Id: 2D_Mesh_Aniso.cpp,v 1.43 2004/08/09 10:29:07 geuzaine Exp $
 //
 // Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
 //
@@ -39,15 +39,6 @@ inline void cgsmpl(Simplex * s, double &x, double &y)
   y = (1. / 3.) * (s->V[0]->Pos.Y + s->V[1]->Pos.Y + s->V[2]->Pos.Y);
 }
 
-MeshParameters::MeshParameters()
-  : NbSmoothing(3),
-    DelaunayAlgorithm(DELAUNAY_ANISO),
-    DelaunayInsertionMethod(INSERTION_CENTROID),
-    DelaunayQuality(QUALITY_EDGES_BASED), InteractiveDelaunay(false)
-{
-  ;
-}
-
 extern Simplex MyNewBoundary;
 extern Mesh *THEM;
 extern double MAXIMUM_LC_FOR_SURFACE;
@@ -66,16 +57,18 @@ static double volume;
 static List_T *coquille;
 static Edge *THEEDGE;
 
-extern void LCBGM(double X, double Y, double Z, double *l);
-
 double Interpole_lcTriangle(Simplex * s, Vertex * vv)
 {
   double Xp, Yp, X[3], Y[3], det, u, v, q1, q2, q3;
 
-  if(THEM->BGM.Typ == ONFILE) {
-    double LL;
-    LCBGM(vv->Pos.X, vv->Pos.Y, 0.0, &LL);
-    return LL * CTX.mesh.lc_factor;
+  if(THEM->BGM.Typ == ONFILE){
+    Vertex *v2 = Create_Vertex(-1, vv->Pos.X, vv->Pos.Y, 0.0, 0.0, 0.0);
+    Vertex *dum;
+    Calcule_Z_Plan(&v2, &dum);
+    Projette_Inverse(&v2, &dum);
+    double val = Lc_XYZ(v2->Pos.X, v2->Pos.Y, v2->Pos.Z, THEM);
+    Free_Vertex(&v2, 0);
+    return val;
   }
 
   Xp = vv->Pos.X;
@@ -296,29 +289,6 @@ int Intersect_Edges_2d(Edge * a, Edge * b)
   return 0;
 }
 
-int compareedgeptr(const void *a, const void *b)
-{
-  int i1, i2, j1, j2;
-  Edge *q, *w;
-
-  q = *(Edge **) a;
-  w = *(Edge **) b;
-  i1 = IMAX(q->V[0]->Num, q->V[1]->Num);
-  i2 = IMAX(w->V[0]->Num, w->V[1]->Num);
-  j1 = IMIN(q->V[0]->Num, q->V[1]->Num);
-  j2 = IMIN(w->V[0]->Num, w->V[1]->Num);
-
-  if(i1 < i2)
-    return (1);
-  if(i1 > i2)
-    return (-1);
-  if(j1 < j2)
-    return (1);
-  if(j1 > j2)
-    return (-1);
-  return 0;
-}
-
 void putaindecoquille_2D(void *a, void *b)
 {
   Edge *e = (Edge *) a;
@@ -359,7 +329,7 @@ void Recover_Edge(Surface * s, Edge * e, EdgesContainer & Edges)
     List_Read(coquille, i, &me);
     v[0] = me->V[0];
     v[1] = me->V[1];
-    List_Suppress(coquille, &me, compareedgeptr);
+    List_Suppress(coquille, &me, compareEdgePtr);
     Edges.SwapEdge(v);
     if(Edges.Search(e->V[0], e->V[1]))
       break;
@@ -384,7 +354,7 @@ void constraint_the_edge(int isurf, int iv1, int iv2)
 
   if(!v1 || !v2)
     return;
-  EdgesContainer EdgesOnSurface(s->Simplexes, false);
+  EdgesContainer EdgesOnSurface(s->Simplexes);
   e.V[0] = v1;
   e.V[1] = v2;
   if(!EdgesOnSurface.Search(v1, v2)) {
@@ -399,7 +369,7 @@ void missing_edges_2d(Surface * s)
   Vertex *v1, *v2;
   Edge e;
 
-  EdgesContainer EdgesOnSurface(s->Simplexes, false);
+  EdgesContainer EdgesOnSurface(s->Simplexes);
 
   for(i = 0; i < List_Nbr(s->Generatrices); i++) {
     List_Read(s->Generatrices, i, &c);
@@ -570,7 +540,7 @@ bool draw_simplex2d(Surface * sur, Simplex * s, bool nouv)
   double x[3], y[3], z[3];
   Vertex v1, v2, v3;
 
-  if(!THEM->MeshParams.InteractiveDelaunay)
+  if(!CTX.mesh.interactive)
     return false;
 
   if(s == &MyNewBoundary || !s || !s->iEnt)
@@ -967,11 +937,10 @@ Vertex *NewVertex_2D(Simplex * s)
      0.0, lc, 0.0);
    */
 
-  if(THEM->MeshParams.DelaunayInsertionMethod == INSERTION_CENTROID)
-    v =
-      Create_Vertex(++THEM->MaxPointNum, s->Center.X, s->Center.Y, 0.0, lc,
-                    0.0);
-  else if(THEM->MeshParams.DelaunayInsertionMethod == INSERTION_EDGE) {
+  if(1){ // INSERTION_CENTROID
+    v = Create_Vertex(++THEM->MaxPointNum, s->Center.X, s->Center.Y, 0.0, lc, 0.0);
+  }
+  else{ // INSERTION_EDGE
     Vertex *vv[2];
     double l = THEM->Metric->getWorstEdge(s, PARAMETRIC, vv);
     double f = 0.5;
