@@ -1,4 +1,4 @@
-// $Id: 3D_Mesh.cpp,v 1.58 2004/04/18 03:36:07 geuzaine Exp $
+// $Id: 3D_Mesh.cpp,v 1.63 2004/06/28 20:36:14 geuzaine Exp $
 //
 // Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
 //
@@ -897,8 +897,10 @@ void Maillage_Volume(void *data, void *dum)
   }
   else if(MeshTransfiniteVolume(v)) {
   }
-  else if(v->Typ == 99999) {
-
+  else if((v->Typ != 99999) && Mesh_Netgen(v)) {
+  }
+  else if((v->Typ == 99999) && (CTX.mesh.algo3d != FRONTAL_NETGEN)) {
+    
     Simplexes_New = List_Create(10, 10, sizeof(Simplex *));
     Simplexes_Destroyed = List_Create(10, 10, sizeof(Simplex *));
 
@@ -910,6 +912,7 @@ void Maillage_Volume(void *data, void *dum)
     POINTS_LIST = List_Create(100, 100, sizeof(Vertex *));
     LOCAL->Simplexes = v->Simplexes;
     LOCAL->Vertices = v->Vertices;
+    
 
     for(i = 0; i < List_Nbr(v->Surfaces); i++) {
       List_Read(v->Surfaces, i, &s);
@@ -1021,6 +1024,7 @@ void Maillage_Volume(void *data, void *dum)
     POINTS_TREE = THEM->Simplexes;
     Tree_Action(v->Simplexes, add_points);
 
+#if 0 // this is full of bugs :-)
     if(CTX.mesh.quality) {
       extern void SwapEdges3D(Mesh * M, Volume * v, double GammaPrescribed,
                               bool order);
@@ -1031,10 +1035,11 @@ void Maillage_Volume(void *data, void *dum)
       Msg(STATUS3, "Swapping edges (last pass)");
       SwapEdges3D(THEM, v, CTX.mesh.quality, true);
     }
+#endif
 
-#if 0 // this is wrong
+#if 0 // this is full of bugs, too :-)
     if(CTX.mesh.nb_smoothing) {
-      Msg(STATUS3, "Laplacian smoothing");
+      Msg(STATUS3, "Smoothing volume %d", v->Num);
       tnxe = Tree_Create (sizeof (NXE), compareNXE);
       create_NXE (v->Vertices, v->Simplexes, tnxe);
       for (int i = 0; i < CTX.mesh.nb_smoothing; i++)
@@ -1049,9 +1054,4 @@ void Maillage_Volume(void *data, void *dum)
     List_Delete(Simplexes_Destroyed);
   }
 
-  THEM->Statistics[6] += Tree_Nbr(v->Vertices);
-  THEM->Statistics[9] += Tree_Nbr(v->Simplexes);
-  THEM->Statistics[10] += Tree_Nbr(v->Hexahedra);
-  THEM->Statistics[11] += Tree_Nbr(v->Prisms);
-  THEM->Statistics[12] += Tree_Nbr(v->Pyramids);
 }
