@@ -1,4 +1,4 @@
-// $Id: 3D_Mesh_Old.cpp,v 1.11 2004/04/18 03:36:07 geuzaine Exp $
+// $Id: 3D_Mesh_Old.cpp,v 1.13 2004/05/26 00:33:37 geuzaine Exp $
 //
 // Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
 //
@@ -895,7 +895,7 @@ void Maillage_Volume(void *data, void *dum)
 
     if(CTX.mesh.nb_smoothing) {
       /*
-         Msg(STATUS3, "Laplacian smoothing");
+         Msg(STATUS3, "Smoothing volume %d", v->Num);
          tnxe = Tree_Create (sizeof (NXE), compareNXE);
          create_NXE (v->Vertices, v->Simplexes, tnxe);
          for (int i = 0; i < CTX.mesh.nb_smoothing; i++)
@@ -910,18 +910,4 @@ void Maillage_Volume(void *data, void *dum)
     List_Delete(Simplexes_Destroyed);
   }
 
-  THEM->Statistics[6] += Tree_Nbr(v->Vertices);
-  THEM->Statistics[9] += Tree_Nbr(v->Simplexes);
-  THEM->Statistics[10] += Tree_Nbr(v->Hexahedra);
-  THEM->Statistics[11] += Tree_Nbr(v->Prisms);
-
-  if(v->Typ == 99999) {
-    Gamma_Maillage(THEM, &THEM->Statistics[17], &THEM->Statistics[18],
-                   &THEM->Statistics[19]);
-    Eta_Maillage(THEM, &THEM->Statistics[20], &THEM->Statistics[21],
-                 &THEM->Statistics[22]);
-    R_Maillage(THEM, &THEM->Statistics[23], &THEM->Statistics[24],
-               &THEM->Statistics[25]);
-  }
-
 }
