@@ -1,4 +1,4 @@
-// $Id: Create.cpp,v 1.48 2004/02/28 00:48:49 geuzaine Exp $
+// $Id: Create.cpp,v 1.63 2004/08/12 16:57:32 geuzaine Exp $
 //
 // Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
 //
@@ -41,7 +41,7 @@ int compareNXE(const void *a, const void *b)
 
   q = (NXE *) a;
   w = (NXE *) b;
-  return (compareVertex(&q->v, &w->v));
+  return compareVertex(&q->v, &w->v);
 }
 
 int compareFxE(const void *a, const void *b)
@@ -50,7 +50,7 @@ int compareFxE(const void *a, const void *b)
 
   q = (FxE *) a;
   w = (FxE *) b;
-  return (compareFace(&q->Sorted, &w->Sorted));
+  return compareFace(&q->Sorted, &w->Sorted);
 }
 
 int compareSurfaceLoop(const void *a, const void *b)
@@ -71,33 +71,6 @@ int compareEdgeLoop(const void *a, const void *b)
   return ((*q)->Num - (*w)->Num);
 }
 
-int compareHexahedron(const void *a, const void *b)
-{
-  Hexahedron **q, **w;
-
-  q = (Hexahedron **) a;
-  w = (Hexahedron **) b;
-  return ((*q)->Num - (*w)->Num);
-}
-
-int comparePrism(const void *a, const void *b)
-{
-  Prism **q, **w;
-
-  q = (Prism **) a;
-  w = (Prism **) b;
-  return ((*q)->Num - (*w)->Num);
-}
-
-int comparePyramid(const void *a, const void *b)
-{
-  Pyramid **q, **w;
-
-  q = (Pyramid **) a;
-  w = (Pyramid **) b;
-  return ((*q)->Num - (*w)->Num);
-}
-
 int compareQuality(const void *a, const void *b)
 {
   double d;
@@ -507,10 +480,13 @@ void End_Curve(Curve * c)
 
 }
 
-void End_Surface(Surface * s)
+void End_Surface(Surface * s, int reset_orientations)
 {
   int i;
   Vertex *v;
+  
+  if(reset_orientations) 
+    List_Reset(s->Orientations);
 
   if(!s->Control_Points || !List_Nbr(s->Control_Points))
     return;
@@ -558,7 +534,6 @@ Curve *Create_Curve(int Num, int Typ, int Order, List_T * Liste,
   pC->Num = Num;
   THEM->MaxLineNum = IMAX(THEM->MaxLineNum, Num);
   pC->Simplexes = Tree_Create(sizeof(Simplex *), compareSimplex);
-  pC->TrsfSimplexes = List_Create(1, 10, sizeof(Simplex *));
   pC->Method = LIBRE;
   pC->degre = Order;
   pC->Circle.n[0] = 0.0;
@@ -609,8 +584,10 @@ Curve *Create_Curve(int Num, int Typ, int Order, List_T * Liste,
       List_Read(Liste, j, &iPnt);
       if((v = FindPoint(iPnt, THEM)))
         List_Add(pC->Control_Points, &v);
-      else
-        Msg(FATAL, "Unknown control point %d in Curve %d", iPnt, pC->Num);
+      else{
+        Msg(GERROR, "Unknown control point %d in Curve %d", iPnt, pC->Num);
+	pC->Dirty = 1;
+      }
     }
   }
   else {
@@ -624,21 +601,20 @@ Curve *Create_Curve(int Num, int Typ, int Order, List_T * Liste,
   }
   else {
     if((v = FindPoint(p1, THEM))) {
-      pC->beg = v;
       Msg(INFO, "Curve %d first control point %d ", pC->Num, v->Num);
+      pC->beg = v;
     }
     else {
-      List_Read(pC->Control_Points, 0, &pC->beg);
       Msg(GERROR, "Unknown control point %d in Curve %d", p1, pC->Num);
+      pC->Dirty = 1;
     }
     if((v = FindPoint(p2, THEM))) {
-      pC->end = v;
       Msg(INFO, "Curve %d first control point %d ", pC->Num, v->Num);
+      pC->end = v;
     }
     else {
-      List_Read(pC->Control_Points, List_Nbr(pC->Control_Points) - 1,
-                &pC->end);
       Msg(GERROR, "Unknown control point %d in Curve %d", p2, pC->Num);
+      pC->Dirty = 1;
     }
   }
 
@@ -654,7 +630,6 @@ void Free_Curve(void *a, void *b)
     List_Delete(pC->Vertices);
     Tree_Action(pC->Simplexes, Free_Simplex);
     Tree_Delete(pC->Simplexes);
-    List_Delete(pC->TrsfSimplexes);
     Free(pC->k);
     List_Delete(pC->Control_Points);
     Free(pC->cp);
@@ -680,19 +655,23 @@ Surface *Create_Surface(int Num, int Typ)
   for(i = 0; i < 5; i++)
     pS->ipar[i] = 0;
   pS->Recombine = 0;
-  pS->RecombineAngle = 30;
+  pS->RecombineAngle = 75;
+  pS->TrsfPoints = List_Create(4, 4, sizeof(Vertex *));
   pS->Simplexes = Tree_Create(sizeof(Simplex *), compareQuality);
-  pS->TrsfSimplexes = List_Create(1, 10, sizeof(Simplex *));
+  pS->Quadrangles = Tree_Create(sizeof(Quadrangle *), compareQuadrangle);
   pS->Vertices = Tree_Create(sizeof(Vertex *), compareVertex);
   pS->TrsfVertices = List_Create(1, 10, sizeof(Vertex *));
   pS->Contours = List_Create(1, 1, sizeof(List_T *));
-  pS->Orientations = NULL;
+  pS->Orientations = List_Create(20, 2, sizeof(Vertex));
   pS->Support = pS;
   pS->Control_Points = List_Create(1, 10, sizeof(Vertex *));
   pS->Generatrices = NULL;
   pS->Edges = NULL;
   pS->Extrude = NULL;
   pS->STL = NULL;
+  pS->TriVertexArray = NULL;
+  pS->QuadVertexArray = NULL;
+  pS->normals = new smooth_normals;
   return (pS);
 }
 
@@ -700,18 +679,27 @@ void Free_Surface(void *a, void *b)
 {
   Surface *pS = *(Surface **) a;
   if(pS) {
+    List_Delete(pS->TrsfPoints);
     Tree_Action(pS->Simplexes, Free_Simplex);
     Tree_Delete(pS->Simplexes);
-    List_Delete(pS->TrsfSimplexes);
+    Tree_Action(pS->Quadrangles, Free_Quadrangle);
+    Tree_Delete(pS->Quadrangles);
     Tree_Delete(pS->Vertices);
     List_Delete(pS->TrsfVertices);
     List_Delete(pS->Contours);
+    List_Delete(pS->Orientations);
     List_Delete(pS->Control_Points);
     List_Delete(pS->Generatrices);
     if(pS->Edges) {
       Tree_Action(pS->Edges, Free_Edge);
       Tree_Delete(pS->Edges);
     }
+    if(pS->TriVertexArray)
+      delete pS->TriVertexArray;
+    if(pS->QuadVertexArray)
+      delete pS->QuadVertexArray;
+    if(pS->normals)
+      delete pS->normals;
     Free(pS);
     pS = NULL;
   }
@@ -732,16 +720,22 @@ Volume *Create_Volume(int Num, int Typ)
   pV->Method = LIBRE;
   for(i = 0; i < 8; i++)
     pV->ipar[i] = 0;
+  pV->TrsfPoints = List_Create(6, 6, sizeof(Vertex *));
   pV->Surfaces = List_Create(1, 2, sizeof(Surface *));
+  pV->SurfacesOrientations = List_Create(1, 2, sizeof(int));
   pV->Simplexes = Tree_Create(sizeof(Simplex *), compareQuality);
   pV->Vertices = Tree_Create(sizeof(Vertex *), compareVertex);
   pV->Hexahedra = Tree_Create(sizeof(Hexahedron *), compareHexahedron);
   pV->Prisms = Tree_Create(sizeof(Prism *), comparePrism);
   pV->Pyramids = Tree_Create(sizeof(Pyramid *), comparePyramid);
-  pV->Simp_Surf = Tree_Create(sizeof(Simplex *), compareSimplex);       // for old extrusion mesh generator
   pV->Extrude = NULL;
   pV->Edges = NULL;
   pV->Faces = NULL;
+  // for old extrusion mesh generator
+  pV->Simp_Surf = Tree_Create(sizeof(Simplex *), compareSimplex);
+  pV->Quad_Surf = Tree_Create(sizeof(Simplex *), compareQuadrangle);
+  pV->TriVertexArray = NULL;
+  pV->QuadVertexArray = NULL;
   return pV;
 }
 
@@ -749,113 +743,37 @@ void Free_Volume(void *a, void *b)
 {
   Volume *pV = *(Volume **) a;
   if(pV) {
-    List_Delete(pV->Surfaces);  //surfaces freed elsewhere
     Tree_Action(pV->Simplexes, Free_Simplex);
+    Tree_Action(pV->Hexahedra, Free_Hexahedron);
+    Tree_Action(pV->Prisms, Free_Prism);
+    Tree_Action(pV->Pyramids, Free_Pyramid);
+    Tree_Action(pV->Edges, Free_Edge);
+  }
+  Free_Volume_But_Not_Elements(a, b);
+}
+
+void Free_Volume_But_Not_Elements(void *a, void *b)
+{
+  Volume *pV = *(Volume **) a;
+  if(pV) {
+    List_Delete(pV->TrsfPoints);
+    List_Delete(pV->Surfaces);  // surfaces freed elsewhere
+    List_Delete(pV->SurfacesOrientations);
     Tree_Delete(pV->Simplexes);
     Tree_Delete(pV->Simp_Surf); // for old extrusion mesh generator
-    Tree_Delete(pV->Vertices);  //vertices freed elsewhere
-    Tree_Action(pV->Hexahedra, Free_Hexahedron);
+    Tree_Delete(pV->Quad_Surf); // for old extrusion mesh generator
+    Tree_Delete(pV->Vertices);  // vertices freed elsewhere
     Tree_Delete(pV->Hexahedra);
-    Tree_Action(pV->Prisms, Free_Prism);
     Tree_Delete(pV->Prisms);
-    Tree_Action(pV->Pyramids, Free_Pyramid);
     Tree_Delete(pV->Pyramids);
-    Tree_Action(pV->Edges, Free_Edge);
     Tree_Delete(pV->Edges);
     Tree_Delete(pV->Faces);
+    if(pV->TriVertexArray)
+      delete pV->TriVertexArray;
+    if(pV->QuadVertexArray)
+      delete pV->QuadVertexArray;
     Free(pV);
     pV = NULL;
   }
 }
 
-Hexahedron *Create_Hexahedron(Vertex * v1, Vertex * v2, Vertex * v3,
-                              Vertex * v4, Vertex * v5, Vertex * v6,
-                              Vertex * v7, Vertex * v8)
-{
-  Hexahedron *h;
-
-  h = (Hexahedron *) Malloc(sizeof(Hexahedron));
-  h->iEnt = -1;
-  h->iPart = -1;
-  h->Num = ++THEM->MaxSimplexNum;
-  h->Visible = VIS_MESH;
-  h->V[0] = v1;
-  h->V[1] = v2;
-  h->V[2] = v3;
-  h->V[3] = v4;
-  h->V[4] = v5;
-  h->V[5] = v6;
-  h->V[6] = v7;
-  h->V[7] = v8;
-  h->VSUP = NULL;
-
-  return (h);
-}
-
-void Free_Hexahedron(void *a, void *b)
-{
-  Hexahedron *pH = *(Hexahedron **) a;
-  if(pH) {
-    Free(pH);
-    pH = NULL;
-  }
-}
-
-Prism *Create_Prism(Vertex * v1, Vertex * v2, Vertex * v3,
-                    Vertex * v4, Vertex * v5, Vertex * v6)
-{
-  Prism *p;
-
-  p = (Prism *) Malloc(sizeof(Prism));
-  p->iEnt = -1;
-  p->iPart = -1;
-  p->Num = ++THEM->MaxSimplexNum;
-  p->Visible = VIS_MESH;
-  p->V[0] = v1;
-  p->V[1] = v2;
-  p->V[2] = v3;
-  p->V[3] = v4;
-  p->V[4] = v5;
-  p->V[5] = v6;
-  p->VSUP = NULL;
-
-  return (p);
-}
-
-void Free_Prism(void *a, void *b)
-{
-  Prism *pP = *(Prism **) a;
-  if(pP) {
-    Free(pP);
-    pP = NULL;
-  }
-}
-
-Pyramid *Create_Pyramid(Vertex * v1, Vertex * v2, Vertex * v3,
-                        Vertex * v4, Vertex * v5)
-{
-  Pyramid *p;
-
-  p = (Pyramid *) Malloc(sizeof(Pyramid));
-  p->iEnt = -1;
-  p->iPart = -1;
-  p->Num = ++THEM->MaxSimplexNum;
-  p->Visible = VIS_MESH;
-  p->V[0] = v1;
-  p->V[1] = v2;
-  p->V[2] = v3;
-  p->V[3] = v4;
-  p->V[4] = v5;
-  p->VSUP = NULL;
-
-  return (p);
-}
-
-void Free_Pyramid(void *a, void *b)
-{
-  Pyramid *p = *(Pyramid **) a;
-  if(p) {
-    Free(p);
-    p = NULL;
-  }
-}
