@@ -1,4 +1,4 @@
-// $Id: Simplex.cpp,v 1.29 2004/02/07 01:40:22 geuzaine Exp $
+// $Id: Simplex.cpp,v 1.34 2004/07/21 22:19:56 geuzaine Exp $
 //
 // Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
 //
@@ -27,10 +27,7 @@
 #include "Context.h"
 
 extern Context_T CTX;
-extern Mesh *THEM;
-
-int Simplex::TotalAllocated = 0;
-int Simplex::TotalNumber = 0;
+extern Mesh *THEM, *LOCAL;
 
 extern Simplex MyNewBoundary;
 
@@ -38,28 +35,23 @@ int FACE_DIMENSION = 2;
 
 Simplex::Simplex()
 {
-  TotalAllocated++;
-  TotalNumber++;
   VSUP = NULL;
   V[0] = V[1] = V[2] = V[3] = NULL;
   S[0] = S[1] = S[2] = S[3] = NULL;
   iEnt = -1;
   iPart = -1;
   Quality = 0.;
-  Num = TotalNumber;
+  Num = ++TotalNumber;
   Visible = VIS_MESH;
 }
 
 Simplex::Simplex(Vertex * v1, Vertex * v2, Vertex * v3, Vertex * v4)
 {
-  TotalAllocated++;
-  TotalNumber++;
   VSUP = NULL;
   S[0] = S[1] = S[2] = S[3] = NULL;
   Quality = 0.;
   Fourre_Simplexe(v1, v2, v3, v4);
-  Num = TotalNumber;
-  THEM->MaxSimplexNum = IMAX(THEM->MaxSimplexNum, Num);
+  Num = ++TotalNumber;
   iEnt = -1;
   iPart = -1;
   Visible = VIS_MESH;
@@ -67,7 +59,7 @@ Simplex::Simplex(Vertex * v1, Vertex * v2, Vertex * v3, Vertex * v4)
 
 Simplex::~Simplex()
 {
-  TotalAllocated--;
+  if(VSUP) Free(VSUP);
 }
 
 int Simplex::CircumCircle(double x1, double y1,
@@ -162,7 +154,8 @@ double Simplex::Volume_Simplexe2D()
 {
   return ((V[1]->Pos.X - V[0]->Pos.X) *
           (V[2]->Pos.Y - V[1]->Pos.Y) -
-          (V[2]->Pos.X - V[1]->Pos.X) * (V[1]->Pos.Y - V[0]->Pos.Y));
+          (V[2]->Pos.X - V[1]->Pos.X) * 
+	  (V[1]->Pos.Y - V[0]->Pos.Y));
 }
 
 void Simplex::center_tet(double X[4], double Y[4], double Z[4], double res[3])
@@ -210,9 +203,7 @@ double Simplex::matsimpl(double mat[3][3])
   mat[2][0] = V[1]->Pos.Z - V[0]->Pos.Z;
   mat[2][1] = V[2]->Pos.Z - V[0]->Pos.Z;
   mat[2][2] = V[3]->Pos.Z - V[0]->Pos.Z;
-  return (mat[0][0] * (mat[1][1] * mat[2][2] - mat[1][2] * mat[2][1]) -
-          mat[1][0] * (mat[0][1] * mat[2][2] - mat[2][1] * mat[0][2]) +
-          mat[2][0] * (mat[0][1] * mat[1][2] - mat[1][1] * mat[0][2]));
+  return det3x3(mat);
 }
 
 double Simplex::rhoin()
@@ -256,8 +247,7 @@ double Simplex::EtaShapeMeasure()
       lij2 += DSQR(lij(i, j));
     }
   }
-  return 12. * pow(9. / 10. * DSQR(fabs(Volume_Simplexe())),
-                   1. / 3.) / (lij2);
+  return 12. * pow(9. / 10. * DSQR(fabs(Volume_Simplexe())), 1./3.) / (lij2);
 }
 
 double Simplex::RhoShapeMeasure()
@@ -337,15 +327,10 @@ void Simplex::Fourre_Simplexe(Vertex * v1, Vertex * v2, Vertex * v3,
     F[3].V[1] = v3;
     F[3].V[2] = v4;
   }
+
   if(!v4) {
     N = 3;
-    if(FACE_DIMENSION == 1) {
-      //qsort(F[0].V,3,sizeof(Vertex*),compareVertex);
-      Center_Circum();
-      Quality = (double)N *Radius / (V[0]->lc + V[1]->lc + V[2]->lc
-                                     + ((V[3]) ? V[3]->lc : 0.0));
-    }
-    else {
+    if(FACE_DIMENSION != 1) {
       qsort(F[0].V, 3, sizeof(Vertex *), compareVertex);
       return;
     }
@@ -356,19 +341,13 @@ void Simplex::Fourre_Simplexe(Vertex * v1, Vertex * v2, Vertex * v3,
 
   Center_Circum();
 
-  /*
-     extern Mesh *THEM, *LOCAL;
-     if (LOCAL && N == 4 && CTX.mesh.algo == DELAUNAY_ISO && THEM->BGM.Typ == ONFILE){
-     Quality = fabs(Radius) / Lc_XYZ(Center.X, Center.Y, Center.Z, LOCAL);
-     if(Quality < 0.){
-     Msg(WARNING, "Negative simplex quality !?");
-     Quality = 4 * Radius / (V[0]->lc + V[1]->lc + V[2]->lc + V[3]->lc);
-     }
-     }
-   */
-
-  Quality = (double)N *Radius / (V[0]->lc + V[1]->lc + V[2]->lc
-                                 + ((V[3]) ? V[3]->lc : 0.0));
+  if(N == 4 && LOCAL && THEM && THEM->BGM.Typ == ONFILE){
+    Quality = fabs(4. * Radius / Lc_XYZ(Center.X, Center.Y, Center.Z, LOCAL));
+  }
+  else{
+    Quality = (double)N *Radius / (V[0]->lc + V[1]->lc + V[2]->lc + 
+				   ((V[3]) ? V[3]->lc : 0.0));
+  }
 
   for(i = 0; i < N; i++)
     qsort(F[i].V, N - 1, sizeof(Vertex *), compareVertex);
@@ -393,29 +372,11 @@ void Free_Simplex(void *a, void *b)
   }
 }
 
-// to avoid the renumbering of the nodes and all the 'Fourre_Simplex' stuff
-Simplex *Create_Quadrangle(Vertex * v1, Vertex * v2, Vertex * v3, Vertex * v4)
-{
-  Simplex *s;
-  s = new Simplex();
-  s->V[0] = v1;
-  s->V[1] = v2;
-  s->V[2] = v3;
-  s->V[3] = v4;
-  return s;
-}
-
 int compareSimplex(const void *a, const void *b)
 {
-  Simplex **q, **w;
-
-  /* Les simplexes sont definis une seule fois :
-     1 pointeur par entite -> on compare les pointeurs */
-
-  q = (Simplex **) a;
-  w = (Simplex **) b;
-  //if((*q)->iEnt != (*w)->iEnt) return (*q)->iEnt - (*w)->iEnt;
-  return ((*q)->Num - (*w)->Num);
+  Simplex *q = *(Simplex **) a;
+  Simplex *w = *(Simplex **) b;
+  return (q->Num - w->Num);
 }
 
 int Simplex::Pt_In_Simplexe(Vertex * v, double uvw[3], double tol)
@@ -602,20 +563,20 @@ int Simplex::Pt_In_Simplex_2D(Vertex * v)
 
 void Simplex::ExportLcField(FILE * f)
 {
-  if(!V[3]) {
+  if(!V[2])
+    fprintf(f, "SL(%f,%f,%f,%f,%f,%f){%12.5E,%12.5E};\n",
+            V[0]->Pos.X, V[0]->Pos.Y, V[0]->Pos.Z, V[1]->Pos.X, V[1]->Pos.Y,
+            V[1]->Pos.Z, V[0]->lc, V[1]->lc);
+  else if(!V[3])
     fprintf(f, "ST(%f,%f,%f,%f,%f,%f,%f,%f,%f){%12.5E,%12.5E,%12.5E};\n",
             V[0]->Pos.X, V[0]->Pos.Y, V[0]->Pos.Z, V[1]->Pos.X, V[1]->Pos.Y,
             V[1]->Pos.Z, V[2]->Pos.X, V[2]->Pos.Y, V[2]->Pos.Z, V[0]->lc,
             V[1]->lc, V[2]->lc);
-  }
-  else {
-    fprintf(f,
-            "SS(%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f){%12.5E,%12.5E,%12.5E,%12.5E};\n",
-            V[0]->Pos.X, V[0]->Pos.Y, V[0]->Pos.Z, V[1]->Pos.X, V[1]->Pos.Y,
-            V[1]->Pos.Z, V[2]->Pos.X, V[2]->Pos.Y, V[2]->Pos.Z, V[3]->Pos.X,
-            V[3]->Pos.Y, V[3]->Pos.Z, V[0]->lc, V[1]->lc, V[2]->lc, V[3]->lc);
-  }
-
+  else
+    fprintf(f, "SS(%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f){%12.5E,%12.5E,%12.5E,%12.5E};\n",
+	    V[0]->Pos.X, V[0]->Pos.Y, V[0]->Pos.Z, V[1]->Pos.X, V[1]->Pos.Y,
+	    V[1]->Pos.Z, V[2]->Pos.X, V[2]->Pos.Y, V[2]->Pos.Z, V[3]->Pos.X,
+	    V[3]->Pos.Y, V[3]->Pos.Z, V[0]->lc, V[1]->lc, V[2]->lc, V[3]->lc);
 }
 
 double Simplex::AireFace(Vertex * V[3])
@@ -800,8 +761,7 @@ bool Simplex::SwapFace(int iFac, List_T * newsimp, List_T * delsimp)
   s3 = Create_Simplex(s->F[iFac].V[2], s->F[iFac].V[0], o[0], o[1]);
 
   double vol1 = s->Volume_Simplexe() + Volume_Simplexe();
-  double vol2 =
-    s1->Volume_Simplexe() + s2->Volume_Simplexe() + s3->Volume_Simplexe();
+  double vol2 = s1->Volume_Simplexe() + s2->Volume_Simplexe() + s3->Volume_Simplexe();
 
   if(fabs(fabs(vol1) - fabs(vol2)) > 1.e-5 * (fabs(vol1) + fabs(vol2))) {
     delete s1;
@@ -813,39 +773,10 @@ bool Simplex::SwapFace(int iFac, List_T * newsimp, List_T * delsimp)
   double gamma1 = GammaShapeMeasure();
 
   if(s1->GammaShapeMeasure() < gamma1 ||
-     s2->GammaShapeMeasure() < gamma1 || s3->GammaShapeMeasure() < gamma1)
+     s2->GammaShapeMeasure() < gamma1 || 
+     s3->GammaShapeMeasure() < gamma1)
     return false;
 
   return true;
 }
 
-int compareFace(const void *a, const void *b)
-{
-  Face *q, *w;
-
-  q = (Face *) a;
-  w = (Face *) b;
-
-  if(!q->V[0] || !w->V[0])
-    Msg(FATAL,
-        "Bad face (are you trying to generate hexahedra with a Delaunay?!)");
-
-  if(q->V[0]->Num > w->V[0]->Num)
-    return (1);
-  if(q->V[0]->Num < w->V[0]->Num)
-    return (-1);
-
-  if(q->V[1]->Num > w->V[1]->Num)
-    return (1);
-  if(q->V[1]->Num < w->V[1]->Num)
-    return (-1);
-
-  if(FACE_DIMENSION == 1 || !q->V[2] || !w->V[2])
-    return 0;
-
-  if(q->V[2]->Num > w->V[2]->Num)
-    return (1);
-  if(q->V[2]->Num < w->V[2]->Num)
-    return (-1);
-  return (0);
-}
