@@ -20,11 +20,17 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
-int  ParseFile(char *filename, int silent, int close);
+int  ParseFile(char *filename, int silent, int close, int warn_if_missing=0);
 void ParseString(char *str);
 void OpenProblem(char *filename);
-int MergeProblem(char *filename);
+void OpenProblemMacFinder(const char *filename);
+int MergeProblem(char *filename, int warn_if_missing=0);
 void SystemCall(char *command);
 void FixRelativePath(char *in, char *out);
+void FixWindowsPath(char *in, char *out);
+void SetBoundingBox(double xmin, double xmax,
+		    double ymin, double ymax, 
+		    double zmin, double zmax);
+void SetBoundingBox(void);
 
 #endif
