@@ -37,6 +37,8 @@ class GMSH_ExtractPlugin : public GMSH_Post_Plugin
   void catchErrorMessage(char *errorMessage) const;
   int getNbOptions() const;
   StringXNumber* getOption(int iopt);  
+  int getNbOptionsStr() const;
+  StringXString* getOptionStr(int iopt);  
   Post_View *execute(Post_View *);
 };
 
