@@ -1,4 +1,4 @@
-// $Id: Levelset.cpp,v 1.13 2004/03/13 19:24:12 geuzaine Exp $
+// $Id: Levelset.cpp,v 1.16 2004/11/09 16:27:53 remacle Exp $
 //
 // Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
 //
@@ -38,6 +38,7 @@ GMSH_LevelsetPlugin::GMSH_LevelsetPlugin()
   _valueIndependent = 0; // "moving" levelset
   _valueView = -1; // use same view for levelset and field data
   _valueTimeStep = -1; // use same time step in levelset and field data views
+  _recurLevel = 4;
   _orientation = GMSH_LevelsetPlugin::NONE;
 }
 
@@ -231,16 +232,15 @@ void GMSH_LevelsetPlugin::executeList(Post_View * iView, List_T * iList,
     return;
   
   if(iNbElm != dNbElm) {
-    Msg(WARNING, "View[%d] and View[%d] have a different number of elements (%d != %d)",
+    Msg(GERROR, "View[%d] and View[%d] have a different number of elements (%d != %d)",
 	iView->Index, dView->Index, iNbElm, dNbElm);
     return;
   }
 
   int dTimeStep = _valueTimeStep;
   if(dTimeStep >= dView->NbTimeStep) {
-    Msg(WARNING, "Wrong time step %d in View[%d]: reverting to time step 0",
-	dTimeStep, dView->Index);
-    dTimeStep = 0;
+    Msg(GERROR, "Wrong time step %d in View[%d]", dTimeStep, dView->Index);
+    return;
   }
 
   int iNb = List_Nbr(iList) / iNbElm;
@@ -339,6 +339,10 @@ Post_View *GMSH_LevelsetPlugin::execute(Post_View * v)
   Post_View *w;
   vector<Post_View *> out;
 
+  if (v->adaptive && v->NbST)
+    v->setAdaptiveResolutionLevel ( _recurLevel , this );
+
+
   if(_valueView < 0) {
     w = v;
   }
@@ -522,8 +526,7 @@ Post_View *GMSH_LevelsetPlugin::execute(Post_View * v)
   executeList(v, v->TY, v->NbTY, 9, w, w->TY, w->NbTY, 9, 5, 8, exnPyr, out);
 
   for(unsigned int i = 0; i < out.size(); i++) {
-    // create time data
-    // FIXME: todo
+    // FIXME: create time data
     // finalize
     char name[1024], filename[1024];
     sprintf(name, "%s_Levelset_%d", v->Name, i);
@@ -531,19 +534,50 @@ Post_View *GMSH_LevelsetPlugin::execute(Post_View * v)
     EndView(out[i], 1, filename, name);
   }
 
-  // remove empty views
-  List_T *to_remove = List_Create(10, 10, sizeof(int));
-  for(int i = 0; i < List_Nbr(CTX.post.list); i++) {
-    w = (Post_View*) List_Pointer(CTX.post.list, i);
-    if(w->empty())
-      List_Insert(to_remove, &w->Num, fcmp_int);
-  }
-  for(int i = 0; i < List_Nbr(to_remove); i++) {
-    int num;
-    List_Read(to_remove, i, &num);
-    RemoveViewByNumber(num);
-  }
-  List_Delete(to_remove);
-
   return 0;
 }
+
+/*
+  On high order maps, we draw only the elements that have a 
+  cut with the levelset, this is as accurate as it should be
+*/
+
+
+static bool recur_sign_change (_triangle *t, double val, const GMSH_LevelsetPlugin *plug)
+{
+
+  if (!t->t[0])
+    {
+      double v1 = plug->levelset (t->p[0]->X,t->p[0]->Y,t->p[0]->Z,t->p[0]->val);
+      double v2 = plug->levelset (t->p[1]->X,t->p[1]->Y,t->p[1]->Z,t->p[1]->val);
+      double v3 = plug->levelset (t->p[2]->X,t->p[2]->Y,t->p[2]->Z,t->p[2]->val);
+      if ( v1 * v2 > 0 && v1 * v3 > 0)
+	t->visible = false;
+      else
+	t->visible = true;
+      return t->visible;
+    }
+  else
+    {
+      bool sc1= recur_sign_change(t->t[0],val,plug);
+      bool sc2= recur_sign_change(t->t[1],val,plug);
+      bool sc3= recur_sign_change(t->t[2],val,plug);
+      bool sc4= recur_sign_change(t->t[3],val,plug);
+      if (sc1 || sc2 || sc3 || sc4)
+	{
+	  if (!sc1) t->t[0]->visible = true;
+	  if (!sc2) t->t[1]->visible = true;
+	  if (!sc3) t->t[2]->visible = true;
+	  if (!sc4) t->t[3]->visible = true;
+	  return true;
+	}
+      t->visible = false;
+      return false;
+    }      
+}
+
+void GMSH_LevelsetPlugin::assign_specific_visibility () const
+{
+  _triangle *t  = *_triangle::all_triangles.begin();
+  t->visible = !recur_sign_change (t, _valueView, this);
+}
