@@ -39,18 +39,14 @@
 #include "Views.h"
 
 class PluginDialogBox;
+class Vertex;
 
 class GMSH_Plugin
 {
 public :
-  // output file name
-  char outputFileName[256];
-
-  //input file name
-  char inputFileName[256];
-
   // a dialog box for user interface
   PluginDialogBox *dialogBox;
+
   // this is there for internal use, this variable will be used by the
   // PluginManager, just forget it
   void *hlib;
@@ -61,6 +57,8 @@ public :
 		GMSH_POST_PLUGIN, 
 		GMSH_SOLVE_PLUGIN} GMSH_PLUGIN_TYPE ;
 
+  GMSH_Plugin() : dialogBox(0), hlib(0) {;};
+  
   // returns the type of plugin for downcasting GMSH_Plugin into
   // GMSH_CAD_Plugin, GMSH_Mesh_Plugin and GMSH_Post_Plugin
   virtual GMSH_PLUGIN_TYPE getType() const = 0;
@@ -71,10 +69,15 @@ public :
   // show the message and hopefully continue
   virtual void catchErrorMessage(char *errorMessage) const = 0;
 
-  // gmsh style option, ca be loaded, saved and set
+  // gmsh style numeric options
   virtual int getNbOptions() const = 0;
   virtual StringXNumber *getOption(int iopt) = 0;
-  virtual void save() = 0;
+
+  // gmsh style string options
+  virtual int getNbOptionsStr() const = 0;
+  virtual StringXString *getOptionStr(int iopt) = 0;
+
+  // run the plugin
   virtual void run() = 0;
 };
 
@@ -86,13 +89,37 @@ class GMSH_Post_Plugin : public GMSH_Plugin
 {
 public:
   inline GMSH_PLUGIN_TYPE getType() const { return GMSH_Plugin::GMSH_POST_PLUGIN; }
-  virtual void save(){
-    Msg(WARNING, "Plugin().Save is deprecated: use 'Save View[num]' instead");
-  }
+  virtual int getNbOptionsStr() const { return 0; };
+  virtual StringXString *getOptionStr(int iopt) { return NULL; };
   virtual void run(){ execute(0); }
   // If returned pointer is the same as the argument, then view is
   // simply modified, else, a new view is added in the view list
   virtual Post_View *execute(Post_View *) = 0;
+  virtual void assign_specific_visibility () const {}
+};
+
+/*
+  A solver plugin. The idea here is to be able to associate
+  some properties to physical entities. The goal is to be able
+  to interface gmsh with a solver (ABAQUS...) i.e. create the 
+  input file for the solver.
+*/
+
+class GMSH_Solve_Plugin : public GMSH_Plugin
+{
+public:
+  virtual int getNbOptionsStr() const { return 0; };
+  virtual StringXString *getOptionStr(int iopt) { return NULL; };
+  virtual int getNbOptions() const { return 0; };
+  virtual StringXNumber *getOption(int iopt) { return NULL; };
+  inline GMSH_PLUGIN_TYPE getType() const { return GMSH_Plugin::GMSH_SOLVE_PLUGIN; }
+  virtual void run() {};// do nothing
+  virtual void popupPropertiesForPhysicalEntity (int dim) = 0;// popup dialog box
+  virtual void receiveNewPhysicalGroup (int dim, int id) = 0;// add the given group to the solver data's
+  virtual void readSolverFile  ( const char * ) = 0;  // load the solver input file related to the gmsh geo file
+  virtual void writeSolverFile ( const char *) const = 0;  // save the solver file  
+  virtual bool GL_enhancePoint ( Vertex *v) { return false; }; // enhance graphics for a giver geo point
+  virtual bool GL_enhanceLine  ( int CurveId , Vertex *v1, Vertex *v2) { return false; }; // enhance graphics for a giver geo line
 };
 
 #endif
